/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TCQPQandTQDL.cpp
 * Description    : Connect to DL Datastore
 *                  Create a parametric query string and parametric query object
 *                  Create a text query string and text query object
 *                  Create a combined query object and submit for execution 
 *                  get the results in a collection.
 *                  Disconnect from DL Datastore
 */

/**
 * Test Combined Query, Parametric and Text Query, no scope
 */
#include <iostream.h>
#include <stdio.h>                
#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKDatastoreTS.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <dkXDO.hpp>
#include "DKParametricQuery.hpp"
#include "DKTextQuery.hpp"
#include "DKCombinedQuery.hpp"
#include "DKResults.hpp"

void main(int argc, char *argv[])
{
     if (argc < 5) {
        cout<<"Usage: "<<endl;
        cout<<"  TCQPQandTQ <DLserver> <userid> <password> <TMserver> " << endl;
        return;
     }
     
     // input parameters
     DKString libsrv = DKString(argv[1]);  // DL library server name, e.g. LIBSRVRX
     DKString userid = DKString(argv[2]);  // userid
     DKString pw     = DKString(argv[3]);  // password
     DKString tmsrv  = DKString(argv[4]);  // text search server or alias name, case sensitive, e.g. TMSRVRX

     cout << " Write down the VMSize and PRESS ENTER to continue " << endl;
     getchar();

     DKDatastoreDL dsDL;  // must be outside try/catch block
     DKDatastoreTS dsTS;
     try {
       
       cout << " *** Test Combined Query : Parametric and Text Query, no scope " << endl;
       
       // connect to datastore
       cout << "connecting to datastore DL: " << libsrv << endl; 
       dsDL.connect(libsrv, userid, pw ,"");
       cout << "datastore DL connected" << endl; 

       cout << "connecting to datastore TS: " << tmsrv <<  endl; 
       dsTS.connect(tmsrv,"",' ');
       cout << "datastore TS connected" << endl; 

       DKDDO* pDDO = NULL;
       dkXDO* pXDO = NULL;
       DKPidXDO* pidXDO = NULL;
       int i = 0;
       short j = 0;
       int resultCounter = 0;
       int numDataItems = 0;
       short propid = 0;
       DKString pidString;
       DKString strDataName;
       DKString className;  
       DKString className2;
       dkCollection* pCol = NULL;
       dkIterator* pIter = NULL;
       DKAny   any;
       DKAny* pAny;
       dkDataObjectBase* pDO = NULL;

       // prepare parametric query string and create parametric query object
       DKString pcmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,COND=(DLSEARCH_Volume > 1));";
       pcmd = pcmd + "OPTION=(CONTENT=NO;";
       pcmd = pcmd + "TYPE_QUERY=STATIC;";
       pcmd = pcmd + "TYPE_FILTER=FOLDERDOC)";
       cout << " parametric query string " << pcmd << endl;
       DKParametricQuery* pq = (DKParametricQuery*) dsDL.createQuery(pcmd,DK_CM_PARAMETRIC_QL_TYPE, NULL);

       // create text query string and create text query object
       DKString tcmd = "SEARCH=(COND=(IBM));";
       tcmd = tcmd + "OPTION=(SEARCH_INDEX=TMINDEX)";
       cout << " text query string " << tcmd << endl;
       DKTextQuery* tq = (DKTextQuery*) dsTS.createQuery(tcmd,DK_CM_TEXT_QL_TYPE, NULL);

       // create combined query object
       DKCombinedQuery cq;
       DKNVPair par[3];
       par[0].set(DK_CM_PARM_QUERY, pq);
       par[1].set(DK_CM_TEXT_QUERY, tq);
       par[2].setName(DK_CM_PARM_END);

       // execute query object
       cout << "executing combined query (pq and tq)" << endl;
       cq.execute(par);
       cout << "cq query executed" << endl;

       any = cq.result();
       DKResults* res = (DKResults*) any.value();
       if (res == NULL) {
           cout << " CQ results is NULL " << endl;  
           return;
       }
       else if (res->cardinality() == 0) {
           cout << " CQ results is empty " << endl;  
           return;
       }
       cout << " CQ number of results = " << res->cardinality() << endl;  
      
       // retrieve the result
       unsigned short sVal;
       dkIterator* iter = res->createIterator();
       cout << "process query results" << endl;
       while (iter->more())
       {
          pAny = iter->next();
          pDDO = (DKDDO*) (pAny->value());
          if (pDDO != NULL)     {
             cout << "=====> Item : " << ++resultCounter << " <=====" << endl;
             i++;
             numDataItems = pDDO->dataCount();
             propid = pDDO->propertyId(DK_CM_PROPERTY_ITEM_TYPE);
             if (propid > 0)  {
                sVal = pDDO->getProperty(propid);
                switch (sVal) {
                   case DK_CM_DOCUMENT :
                        cout << "Item is document " << endl;
                        break;
                   case DK_CM_FOLDER :
                        cout << "Item is folder " << endl;
                        break;
                } 
             }  

             cout << " Pid string : " << ((DKPid&)pDDO->getPid()).pidString() << endl;
             cout << " Number of Data Items " << numDataItems << endl;
             for (j = 1; j <= numDataItems; j++) {
                 any = pDDO->getData(j);
                 strDataName = pDDO->getDataName(j);
                 cout << j << ": Attribute Name " << strDataName << endl;
                 cout <<    "    Attribute Type " << (short) any.typeCode() << endl;
                 switch (any.typeCode()) {
                 case DKAny::tc_collection:
                   {
                      cout << "   Attribute Value is a collection " << endl;    
                      pCol = (dkCollection*) any;
                      pIter = pCol->createIterator();
                      i = 0;
                      while (pIter->more()) {
                    	 i++;
                    	 pAny = pIter->next();
                    	 pDO  = (dkDataObjectBase*) pAny->value();
                    	 if (pDO->protocol() == DK_CM_PDDO) {
                    	     cout << "    DKDDO object " << i << " in collection" << endl;
                             cout << "    object's pid is " << ((DKPid&)((DKDDO*)pDO)->getPid()).pidString() << endl;
                    	 }
                    	 else if (pDO->protocol() == DK_CM_XDO) {
                    	     cout << "    dkXDO object " << i << " in collection" << endl;
                    	     pXDO = (dkXDO*) pDO;
                    	     pidXDO = (DKPidXDO*) pXDO->getPid();
                    	     cout << "      object's pid is " << pidXDO->pidString() << endl;
                    	     pXDO->retrieve();
                    	 } 
                      } // end while
                      delete pIter;
                   }
                   break;
                 default:
                   cout << "   Attribute Value : " << any << endl;
                   break;
                 }
             }       
          } // if pDDO == NULL
       }
       delete iter;

       cout << " beware, deleting the query objects " << endl;
       delete pq;
       delete tq;
       delete res; // cq results

       cout << "query results processed" << endl;
       dsDL.disconnect();
       dsTS.disconnect();
       cout << "datastores disconnected" << endl; 
    }
    catch (DKException& exc)
    {
       cout << "Exception caught in TCQPQandTQ: " << endl;
       cout << "Error id " << exc.errorId() << endl;
       cout << "Exception id " << exc.exceptionId() << endl;
       cout << "Exception Class Name: " << exc.name() << endl;
       for (unsigned long i = 0; i< exc.textCount();i++) {
      	   cout << "Error text: " << exc.text(i) << endl;
       }
       for (unsigned long g=0; g< exc.locationCount();g++)  {
      	   const DKExceptionLocation* p = exc.locationAtIndex(g);
      	   cout << "Filename  : " << p->fileName() << endl;
      	   cout << "Function  : " << p->functionName() << endl;
      	   cout << "LineNumber: " << p->lineNumber() << endl;
       }
    }
    cout << "done ..." << endl;
    cout << " Write down the VMSize and PRESS ENTER to end " << endl;
    getchar();
}
