/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TExecPSTFed.cpp
 * Description    : Retrieve search template from database and execute a query.
 *                : Need to run the following program first
 *                  TADLServersFed <cmbdb> <userid> <pw> add
 *                  TADLsmFed      <cmbdb> <userid> <pw> add
 *                  TADLstFed      <cmbdb> <userid> <pw> add
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKAny.hpp>
#include <DKDatastoreFed.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKException.hpp>
#include <DKEnvironment.hpp>
#include <DKAttrDefFed.hpp>
#include <DKEntityDefFed.hpp>
#include <DKAttrMappingFed.hpp>
#include <DKAttrMapDef.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>
#include <DKSearchTemplateFed.hpp>
#include <DKSearchCriteria.hpp>

void displayDDO(DKDDO* ddo);


void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   //--------------------------------------------------------------// 
   // Checking for input parameters                                //
   //--------------------------------------------------------------//   
   if (argc < 4)  {
      cout <<"Usage syntax: "<<endl;
      cout <<"\t TExecPSTFed <feddb> <userid> <pw> " << endl;
      return;
   }
   // ### get input parameters
   DKString libsrv(argv[1]);
   DKString userid(argv[2]);
   DKString pw    (argv[3]);
   DKDatastoreFed dsFed; // must be outside try block to avoid exception trap error.
   
   try {
      // All set; Connect and execute the compound query expression
      cout << " *** connecting to " << libsrv << " *** " << endl;
      dsFed.connect(libsrv, userid, pw);
      cout << " \t -----> connected " << endl;
      cout << "     user " << dsFed.userName() << ", dsName " << dsFed.datastoreName() << endl;
   
      //--------------------------------------------------------------// 
      // Retreive ST from datastore                                   //
      //--------------------------------------------------------------//   
      DKNVPair optL[2];

      optL[0].set("MAX_RESULTS", DKString("6"));
      optL[1].set("CONTENT", DKString("ATTRONLY"));
      int size = 2;
      
      dkCollection* coll = dsFed.listSearchTemplates();
      dkIterator* iter = NULL;

      if (coll != NULL) {
          iter = coll->createIterator();
          cout << " " << endl;
          cout << "-- list Search template -- " << endl;
          DKSearchTemplateFed* stObj;
          int flag;

          while (iter->more()) {
              stObj = (DKSearchTemplateFed*)(void *)(*iter->next());
              stObj->setOptionList(optL, size);
                
              cout << " " << endl;
              cout << "==============Retrieving Search template: " << stObj->getName() << endl;
              cout << "    Description: " << stObj->getDescription() << endl;
                        
              dkCollection* scColl = stObj->listCriteria();
			  flag =0;
              dkIterator* scIter = scColl->createIterator();
              while (scIter->more() && flag==0) {
                   DKSearchCriteria* scObj = (DKSearchCriteria*) (void *)(*scIter->next());
                   cout << "       Criteria Name: " << scObj->getName() << endl;
                        
                   DKString name = scObj->getName();
                   if (name == "Title"){
                       scObj->setOperator(DK_CM_OPCODE_EQ);
                       scObj->setValue("'BASUKI'");
                       stObj->update(scObj);
                       flag = 1;
                   }
                   else if (name == "UserID"){
                       scObj->setOperator(DK_CM_OPCODE_LIKE);
                       scObj->setValue("'Progr%'");
                       stObj->update(scObj);
                       flag = 1;
                   }
                   else if (name == "Date"){
                       scObj->setOperator(DK_CM_OPCODE_NEQ);
                       scObj->setValue("NULL");
                       stObj->update(scObj);
                       flag = 1;
                   } else
                       flag = 0;
              } // while more search criteria
              delete scIter;
              delete scColl;

              //  create compound query expressions for each back-end
              if (flag == 1){
                    cout << " ** creates a compound query expression " << endl;
                    DKCQExpr *cqe = stObj->createQueryExpr(); 
                    cout <<" SQL string of cqe = " << cqe->toSQL() << endl;

                    //  execute query
                    dkResultSetCursor* rsc = dsFed.execute(cqe);

                    //  get result set cursor
                    if (rsc == NULL) 
                        cout << " *** cursor is null" << endl;
                    else {
                        DKDDO* ddo;
     
                        while (rsc->isValid())  {
                            cout << "before fetch, position = " << rsc->getPosition() << endl;
                            ddo = rsc->fetchNext();
                            cout << "after fetch, position = " << rsc->getPosition() << endl;
     
                            if (ddo != NULL) {  
                                cout << " pidString : " << ((DKPid&)ddo->getPid()).pidString() << endl;
                                displayDDO(ddo);
                                delete ddo;
                            } 
                            else 
                                break;
                         }
                         delete rsc;
                    }
                    delete cqe;
               } 
               delete stObj;
            } // while more search template
            delete iter;
            delete coll;
        }
	    else 
	        cout << "No Search template defines" << endl;

      // disconnects
      cout << "\n\n *** disconnect from the datastore Fed. ***" << endl;
      dsFed.disconnect();
      cout << " *** datastore Fed disconnected ***" << endl;
     
   }
   catch(DKException &exc)
   {
       cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
         cout << "Error text: " << exc.text(i) << endl;
     
      for (unsigned long g=0; g< exc.locationCount();g++) {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
     
      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}


// simple method to show DDO hit 
void displayDDO(DKDDO* ddo) 
{
   DKString   dataName;
   short      cnt;
   DKAny      any;
   
   if (ddo != NULL) {
      cnt = ddo->dataCount();
      for (short i = 1; i <= cnt; i++) {
          any = ddo->getData(i);
          dataName = ddo->getDataName(i);
          cout << i << ": Attribute Name :" << dataName << endl;
          if (!any.isNull())
               cout << ",\t   Value :" << any << endl;
          else cout << ",\t   Value : null"  << endl;
      }  
   }
}

