/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

PURPOSE:
  This sample program demonstrates how to create an EIP advanced work flow service
  instance, and how to make the linkage between the work flow service instance and
  the EIP federated data base.

USAGE:
  TConnectWFFed <EIPFederatedDB> <userid> <password>

EXTERNAL DEPENDENCIES:
  EIP federated connector installed
  EIP federated database (FED DB) active
  MQSeries server active and MQSeries runtime libraries installed
  MQSeries WorkFlow (MQSWF) server active and MQSWF runtime libraries installed

PROGRAM LOGIC:
  Get input parameters
  Create a WF service (svc) instance and a FED DB datastore (ds) instance
  Connect to the FED DB ds
  Establish the reference from the WF svc to the FED DB ds
  Connect to MQSWF
  Output successful connection message
  Disconnect MQSWF
  Disconnect FED DB ds

*****************************************************************************************/

#include <iostream.h>
#include <DKConstant.h>
#include <DKWorkFlowServiceFed.hpp>
#include <DKDatastoreFed.hpp>
#include <DKException.hpp>
#include <DKString.hpp>

void main(int argc, char *argv[])
{
  if (argc < 4)
  {
    cout<<"Usage: " << endl;
    cout<<"  TConnectWFFed <EIPFederatedDB> <userid> <password>" << endl;
    return;
  }

  // Get input parameters
  DKString wfsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  
  cout << "begin..." << endl;  

  try
  {
    DKWorkFlowServiceFed svWF;
    DKDatastoreFed dsFed;

    cout << "connecting to EIP federated database" << endl;
    dsFed.connect(wfsrv, userid, pw, "");
    cout << "EIP federated database connected" << endl;    
    svWF.setDatastore(& dsFed);

    cout << "connecting to workflow service" << endl;
    svWF.connect(wfsrv, userid, pw, "");
    cout << "workflow service connected" << endl;
    cout << "connected successfully; service type " << svWF.serviceType() << endl;

    cout << "disconnecting workflow service" << endl;
    svWF.disconnect();
    cout << "workflow service disconnected" << endl;

    dsFed.disconnect();
    cout << "done ..." << endl;

  }
  catch(DKException & exc)
  {
  	cout << endl;
    cout << "Exception Class : " << exc.name() << endl;

    cout << "Error id        : " << exc.errorId() << endl;
    cout << "Error message   : " << exc.text() << endl << endl;

    cout << "Details:" << endl;
    cout << exc.toString() << endl;
  }

  cout << "done ..." << endl;

}

