/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Attribute Groups
    Groups of attributes can be created and used to easily add, retrieve,
    and access entire groups of attributes to items and sub-components.
    Attributes may belong to any number of Attribute Groups, from zero to 
    many.  The same attribute name may appear in multiple attribute groups, 
    and since a DDO may contain multiple attribute groups, the same attribute
    name may appear in the DDO, but each attribute is completely seperate.

    When adding attributes, use the individual attribute definition names.  However, 
    once they are part of an attribute group, in order to access a particular 
    attribute of a group, the correct attribute name follows the following format:
    
        Grouped Attribute's Name Format:  <Attribute Group Name>.<Attribute Name>

    When an attribute is added to an attribute group, this will only impact Component 
    Types that are created after the addion.  Existing component types will remain 
    unchanged.
    
    Names
        Attribute names are limited to 15 characters.  Use the description
        field to add more detailed information.

        Names only need to be unique among those of its own namespace.  The 
        Attribute namespace is seperate from the Entity Type namespace, Link Type 
        Namespace, etc.

Reference Attribute (Group):
    Please consult the SReferenceAttrDefCreationICM Sample.

---------------------------------
Insurance Model Attribute Groups:
---------------------------------
NOTE: In this sample, the prefix "S_" will be used to indicate that the attribute
      group is used by and was create by the samples.  This helps avoid using
      attribute group names that may already be in use.

    Attribute Group    Attributes     Part of Insurance Model Used In
    ---------------  ---------------  -------------------------------
    S_carAccident    S_accidentDate   Adjuster Report
                     S_licensePlate

---------------------------------------
Attribute Groups Defined in this Sample
---------------------------------------

    Attribute Group    Attributes 
    ---------------  ---------------
    S_carAccident    S_accidentDate
                     S_licensePlate
    --------------------------------
    S_address        S_street
                     S_city
                     S_state
                     S_zipCode
    --------------------------------

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeGroupDefCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeGroupDefCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining & Explaining Attribute Groups, Insurance Model Attribute Groups
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Attribute Groups
 *                    Attribute Group Name Requirements
 *                    Defining attribute groups used by samples & Insurance Model
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeGroupDefCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: If the attribute have not yet been defined, please run
 *                    SAttributeDefinitionCreationICM first.  If the attribute groups are defined
 *                    already, please run SAttributeGroupDefDeletionICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeGroupDefCreationICM_launch.cpp
 *                    SAttributeGroupDefCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeGroupDefCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                            << endl;
            cout << "  SAttributeGroupDefCreationICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."             << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "=============================================="     << endl;
        cout << "IBM Information Integrator for Content    v" << ver << endl;
        cout << "Sample Program:  SAttributeGroupDefCreationICM"     << endl;
        cout << "----------------------------------------------"     << endl;
        cout << " Database: " << database                            << endl;
        cout << " UserName: " << userName                            << endl;
        cout << "==============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Define an Attribute Group Definition
            //--------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that it is used by and was create by the samples.  This helps 
            // avoid using attribute group names that may already be in use.
            //
            // For more information on Retrieving Attribute Definitions, please
            // consult Sample SAttributeDefinitionRetrievalICM.
            //
            // Defining "Car Accident" attribute group for the Insurance Model's
            // Adjuster Report.  It contains two attributes, Accident Date, and
            // License Plate.
            cout << "Defining Attribute Group 'S_carAccident'..." << endl;

                DKAttrGroupDefICM* attrGroup = new DKAttrGroupDefICM(dsICM); // create a new attribute group
                attrGroup->setName("S_carAccident");                         // Set a name, maximum 15 characters
                attrGroup->setDescription("Car Accident");                   // For more detail than name
                
                // Retrieve Attribute Definitions For Those to Add to Group
                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();               // Get the datastore definition object from the connected datastore.
                DKAttrDefICM* licensePlate = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_licensePlate"); // Retrieve the definition.
                DKAttrDefICM* accidentDate = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_accidentDate"); // Retrieve the definition.
                
                attrGroup->addAttr(licensePlate);  // Add the Attribute Definitions to the Attribute Group
                attrGroup->addAttr(accidentDate);
                
                attrGroup->add();                  // add it to the persistent datastore

                delete(attrGroup);                 // Free Memory.  Attr Group will delete Attr Definitions within.

            cout << "Defined Attribute Group 'S_carAccident'." << endl;

            //--------------------------------------------------------------
            // Define another Attribute Group
            //--------------------------------------------------------------
            // Defining "Address" attribute group.  It contains attributes 
            // that make up an address.
            cout << "Defining Attribute Group 'S_address'..." << endl;

                attrGroup = new DKAttrGroupDefICM(dsICM);     // create a new attribute group
                attrGroup->setName("S_address");              // Set a name, maximum 15 characters
                attrGroup->setDescription("Address");         // For more detail than name
                
                // Retrieve Attribute Definitions For Those to Add to Group
                DKAttrDefICM* street  = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_street"); // Retrieve the definition.
                DKAttrDefICM* city    = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_city"); 
                DKAttrDefICM* state   = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_state"); 
                DKAttrDefICM* zipCode = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_zipCode"); 
                
                attrGroup->addAttr(street);  // Add the Attribute Definitions to the Attribute Group
                attrGroup->addAttr(city);
                attrGroup->addAttr(state);
                attrGroup->addAttr(zipCode);
                
                attrGroup->add();                  // add it to the persistent datastore

                delete(attrGroup);                 // Free Memory.  Attr Group will delete Attr Definitions within.

            cout << "Defined Attribute Group 'S_address'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
