/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Updating Item Type or Component Type Definitions
    An Item Type may be modified just in the same way that values were set
    in the Item Type Creation sample.  Definition object may be accessed
    as shown in the Item Type Retrieval Sample.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKItemTypeDefICM.hpp>
#include <DKComponentTypeDefICM.hpp>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include "SConnectDisconnectICM.hpp"
#include "SItemTypeRetrievalICM.hpp"
#include "SItemTypeUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemTypeUpdateICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying & Updating Item Type Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Item Types in Memory
 *                    Saving / Updating Item Type Definitions in Datastore Definition
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemTypeUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Item Types & Component Types must be defined.  If not, please run Sample 
 *                    SItemTypeCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemTypeRetrievalICM.hpp/cpp
 *                    SItemTypeUpdateICM_launch.cpp
 *                    SItemTypeUpdateICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemTypeUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SItemTypeUpdateICM <database> <userName> <password>"    << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemTypeUpdateICM"             << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------------
            // Retrieving an Individual Item Type Definition
            //-------------------------------------------------------------------
            // Retrieve a single Item Type Definition given the Item Type Name.
            //
            // NOTE:  When retrieving Item Type, Component Type, or any Entity Definitions, if it 
            //        does not exist, null is returned.  Usage of the null object will result in 
            //        a NullPointerException.
            cout << "Retrieving Item Type Definition of 'S_simple'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKItemTypeDefICM* itemType = (DKItemTypeDefICM*) dsDefICM->retrieveEntity("S_simple"); // Retrieve the Item Type Definition.

            cout << "Retrieved Item Type Definition of 'S_simple'." << endl;

            //-------------------------------------------------------------------
            // Modify Definition
            //-------------------------------------------------------------------
            // Any part of the Item Type Definition may be modified.  For more
            // information about accessing Attribute Definitions, Mulivalue Attribute
            // Definitions, Child Component Type Definitions, and all Sub-Entities,
            // please refer to the SItemTypeRetrievalICM Sample, which covered this
            // thoroughly.  Once it is shown how to access each part, it should be
            // very easy to modify and continue to the next section here.
            //
            // In this case, a property of the main Item Type Definition will be 
            // modified.
            cout << "Modifying Item Type Definition for 'S_simple'..." << endl;

                itemType->setDescription("My Simple Item Type's Description.");

            cout << "Modified Item Type Definition for 'S_simple'." << endl;

            //-------------------------------------------------------------------
            // Save / Update Defintion Changes in Datastore Definition
            //-------------------------------------------------------------------
            // Like Attribute Definitions, its own update() method can be used or
            // the datastore definitions's update(entityType) method may be used.
            cout << "Saving / Updating Item Type Definitin Changes..." << endl;

                itemType->update(); // Save the Changes

            cout << "Saving / Updating Item Type Definitin Changes..." << endl;

            //-------------------------------------------------------------------
            // Printing Updated Item Type Definition
            //-------------------------------------------------------------------
            // Uses wrapper function defined in SItemTypeRetrievalICM.
            cout << "Printing the Updated Item Type Definition of 'S_simple'..." << endl;

                SItemTypeRetrievalICM::printItemType(itemType); // Uses wrapper funciton defined at end of sample file.
                delete(itemType);                               // Free Memory

            cout << "Printed the Updated Item Type Definition of 'S_simple'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
