/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKLinkTypeDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SLinkTypeDefinitionRetrievalICM.hpp"
#include "SLinkTypeDefinitionUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: SLinkTypeDefinitionUpdateICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, & Saving Link Type Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Link Type Definitions in Memory
 *                    Saving / Updating Link Type Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SLinkTypeDefinitionUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Link Types must already be defined.  If not, please run Sample
 *                    SLinkTypeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SLinkTypeDefinitionRetrievalICM.hpp/cpp
 *                    SLinkTypeDefinitionUpdateICM_launch.cpp
 *                    SLinkTypeDefinitionUpdateICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SLinkTypeDefinitionUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                            << endl;
            cout << "  SLinkTypeDefinitionUpdateICM <database> <userName> <password>"    << endl;
            cout << "  *** Some parameters not specified, using defaults..."             << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================="     << endl;
        cout << "IBM Information Integrator for Content   v" << ver << endl;
        cout << "Sample Program:  SLinkTypeDefinitionUpdateICM"     << endl;
        cout << "---------------------------------------------"     << endl;
        cout << " Database: " << database                           << endl;
        cout << " UserName: " << userName                           << endl;
        cout << "=============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //----------------------------------------------------------------
            // Retrieve & Print the S_otherPolicies Link Type Definition
            //----------------------------------------------------------------
            // For more information about the defined Link Types, please consult
            // the SLinkTypeDefinitionCreationICM sample.
            cout << "Retrieving the Link Type Definition of 'S_otherPolicies'..." << endl;

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKLinkTypeDefICM* linkTypeDef = (DKLinkTypeDefICM*) dsDefICM->retrieveLinkType("S_otherPolicies"); // Retrieve the definition.
                SLinkTypeDefinitionRetrievalICM::printLinkTypeDef(linkTypeDef); // Use wrapper funciton from the Retrieval Sample.
                
            cout << "Retrieved the Link Type Definition of 'S_otherPolicies'." << endl;

            //----------------------------------------------------------------
            // Modify the S_otherPolicies Link Type Definition in Memory
            //----------------------------------------------------------------
            cout << "Modify the Link Type Definition 'S_otherPolicies' in Memory..." << endl;
            
                linkTypeDef->setDescription("Links Insurance Policies Together");

            cout << "Modified the Link Type Definition of 'S_otherPolicies' in Memory." << endl;

            //----------------------------------------------------------------
            // Update the S_otherPolicies Link Type Definition in the Datastore Definition.
            //----------------------------------------------------------------
            // The Link Type Definition can be updated through its own update() method
            // or by using the Datastore Definition's update() method, passing the attribute
            // definition as the parameter.
            cout << "Updating the Link Type Definition 'S_otherPolicies' in Datastore Definition..." << endl;

                linkTypeDef->update(); // save the Link Type Definition changes.

            cout << "Modified the Link Type Definition of 'S_otherPolicies' in Datastore Definition." << endl;

            //----------------------------------------------------------------
            // Print the Link Type Definition with changes.
            //----------------------------------------------------------------
            cout << "Printing Link Type Definition..." << endl;

                SLinkTypeDefinitionRetrievalICM::printLinkTypeDef(linkTypeDef); // Sample's Wrapper            
                delete(linkTypeDef);

            cout << "Printed Link Type Definition." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
