/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Resource Item Storage
    Resource Item's content is stored in a Resource Manager.  It is possible to specify
    which Resource Manager & which SMS Collection within the Resource Manager that each
    Resource Item is to be stored.  Multiple Resource Managers and multiple SMS 
    Collections may be available for use.
    
    Default Resource Manager & SMS Collection
        The Item Type Definition contains information that specifies the default
        Resource Manager and default SMS Collection that Items of that Item Type
        are to be stored if not otherwise specified.  Please refer to the corresponding
        wrapper function at the bottom of this sample, and to the Item Type Creation
        sample, SItemTypeCreationICM for more information and examples of how to 
        set and modify these properties.
    
    For more information on Resource Manager Definitions or SMS Collections, please refer
    to the corresponding section below.

SMS Collections
    SMS Collections are a logical grouping of objects.  Within each Resource Manager
    defined in the system, an SMS Collection may be defined as a grouping or collection
    that Resource Items may belong to.  One advantage of grouping & seperating Resource
    Items in a particular Resource Manger server is for administrative purposes to help
    manage and diagnose needs.  

    RM Pre-defined SMS Collections:
        The Non-Media colleciton listed below is created automatically with a Resource Manager
        Definition.
    
         Collection      Constant                                  Description
    -------------------  ----------------------------------------  ----------------------------
    Non-Media (Default)  RM_SMS_COLLECTION_NONMEDIA                Default Collection.
                                                                   May be used for non-media.
    ...........................................................................................
           Media         RM_SMS_COLLECTION_MEDIA                   Used for Media Types.
    ...........................................................................................
           Video         RM_SMS_COLLECTION_VIDEO                   Used with Video Charger.
    -------------------------------------------------------------------------------------------
    Note:  Contstants shown above are defined in DKConstantICM.h

    The ICM Content Manager v8.2 install on Windows will create a Resource Manager object
    named "RMDB" by default and one SMS Collection, Non-Media by default.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"
#include "SSMSCollectionDefRetrievalICM.hpp"
#include "SSMSCollectionDefCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: SMS Collection Definition, Explanation, Setup / Configuration.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Setting up an SMS Collection
 *                    Pre-defined SMS Collections
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SSMSCollectionDefCreationICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <RM User Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, password, or
 *                          RM username are not specified.  All others are required.
 *
 *                    ex: SSMSCollectionDefCreationICM RMDB
 *                    ex: SSMSCollectionDefCreationICM MY_RM icmadmin icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefCreationICM_launch.cpp
 *                    SSMSCollectionDefCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSMSCollectionDefCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        char * rmUserName                  = "icmadmin";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 6) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: " << endl;
            cout << "   SSMSCollectionDefCreationICM"                                            << endl;
            cout << "                         <Existing RM User-Defined Reference Name>"         << endl;
            cout << "                         <RM User Name>"                                    << endl;
            cout << "                         <database> <userName> <password>"                  << endl;
            cout << endl;
            cout << "  NOTE: The specified reference name must be already defined, such as the"  << endl;
            cout << "        default 'RMDB' specified during install.  If a new RM definition"   << endl;
            cout << "        is to be configured, please refer to SResourceMgrDefCreationICM"    << endl;
            cout << "        Sample."                                                            << endl;
            cout << endl;
            cout << "  NOTE: Defaults will be used if the database, username, password, or"      << endl;
            cout << "        RM username are not specified.  All others are required."           << endl;
            cout << endl;
            cout << "  ex: SSMSCollectionDefCreationICM RMDB"                                      << endl;
            cout << "  ex: SSMSCollectionDefCreationICM MY_RM icmadmin icmnlsdb icmadmin password" << endl;
        }
        if((argc < 2) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "=========================================================" << endl;
            cout << "PROGRAM TERMINATED:  RM Reference Name Must Be Specified." << endl;
            cout << "=========================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 6)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];
        if (argc > 2) rmUserName                 = argv[2];

        // Store optional commandline arguments.
        if (argc > 3)  database = argv[3];
        if (argc > 4)  userName = argv[4];
        if (argc > 5)  password = argv[5];
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================="         << endl;
        cout << "IBM Information Integrator for Content   v" << ver     << endl;
        cout << "Sample Program:  SSMSCollectionDefCreationICM"         << endl;
        cout << "---------------------------------------------"         << endl;
        cout << "           Database:  " << database                    << endl;
        cout << "           UserName:  " << userName                    << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName  << endl;       
        cout << "        RM UserName:  " << rmUserName                  << endl;
        cout << "=============================================\n"       << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //-------------------------------------------------------------
            // Obtain the RM Code, Needed for SMS Collection Definition
            //-------------------------------------------------------------
            cout << "Obtaining RM Code of RM '" << rmUserDefinedReferenceName << "'..." << endl;

                if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false) // if it is not setup, stop here
                    throw DKException(DKString("Resource Manager Definition Named '")+DKString(rmUserDefinedReferenceName)+DKString("' Not Defined."));

                short rmCode = SResourceMgrDefRetrievalICM::getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.

            cout << "Obtained RM Code of RM '" << rmUserDefinedReferenceName << "'." << endl;

            //-------------------------------------------------------------
            // Setup New SMS Collection Definition (if not already)
            //-------------------------------------------------------------
            // Setup a new SMS Collection Definition if not already defined.
            //
            // This sample will setup the system's pre-determined collection names.
            // User-Defined Collections may be substituted.  Any unique name can be
            // specified.
            cout << "Setting up system pre-determined SMS Collection Definitions (if not already)..." << endl;

                // Only if not already defined, define the specified SMS Collection.  Please
                // refer to SSMSCollectionDefRetrievalICM for more information.
                if(SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_NONMEDIA, rmCode)==false)
                    setupSMSCollectionForRM(dsICM, RM_SMS_COLLECTION_NONMEDIA, rmCode, rmUserName); // Uses wrapper at bottom of sample.

                if(SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_MEDIA, rmCode)==false)
                    setupSMSCollectionForRM(dsICM, RM_SMS_COLLECTION_MEDIA, rmCode, rmUserName); // Uses wrapper at bottom of sample.

                if(SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_VIDEO, rmCode)==false)
                    setupSMSCollectionForRM(dsICM, RM_SMS_COLLECTION_VIDEO, rmCode, rmUserName); // Uses wrapper at bottom of sample.
                
            cout << "Set up system pre-determined SMS Collection Definitions (if not already)." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // setupSMSCollectionForRM
    //******************************************************************
   /**
    * Setup the SMS Collection for the specified Resource Manger.
    * @param dsICM       Connected DKDatastoreICM object.
    * @param newSMSCollectionUserDefinedReferenceName  A new User-Defined Reference name to refer to it as in the RM.
    * @param rmCode      Resource Manager ID Code.
    * @param rmUserName  UserName to log onto the Resource Manager.
    * @return Returns the SMS Collection's ID Code.
    **/
    short SSMSCollectionDefCreationICM::setupSMSCollectionForRM(DKDatastoreICM* dsICM, char* newSMSCollectionUserDefinedReferenceName, short rmCode, char* rmUserName){
        cout << "Setting up SMS Collection with new user defined reference name '" << newSMSCollectionUserDefinedReferenceName << "' for RM with ID Code '" << rmCode << "'..." << endl;
        
        // If the SMS Collection is already setup, throw error.
        if(SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(dsICM, newSMSCollectionUserDefinedReferenceName,rmCode))
            throw DKException(DKString("An SMS Collection with Reference Name '")+DKString(newSMSCollectionUserDefinedReferenceName)+DKString("' for Resource Manager with ID Code '")+DKString(rmCode)+DKString("' already exists."));

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                              // Get the Resource Manager configuration.

        // Setup New SMS Collection Definition Object
        DKSMSCollectionDefICM* smsCollDef = new DKSMSCollectionDefICM(); // Create new SMS Collection Definition.
        smsCollDef->setName(newSMSCollectionUserDefinedReferenceName);   // Set a new User Defined Reference Name to refer to it later as.
        smsCollDef->setRMCode((short)rmCode);                            // Set the RM Code for the Resource Manager that it belongs to.
        smsCollDef->setUserid(rmUserName);                               // Set the Resource Manager's User Name to use.

        rmCfg->addSMSCollection(smsCollDef);                 // Add the SMS Collection to the Resource Manager Configuration.

        short collCode = smsCollDef->getSMSCollectionCode();  // Get the SMS Collection's ID Code
        cout << "Added SMS Collection.  ID Code: '" << collCode << "'." << endl;

        // Double Checking...
        DKSMSCollectionDefICM* retrSMS = rmCfg->retrieveSMSCollection(newSMSCollectionUserDefinedReferenceName, rmCode); // Retrieve the SMS Collection.
        short retrCollCode = retrSMS->getSMSCollectionCode(); // Get the retrieve collections ID Code.
        if(collCode != retrCollCode)// if they aren't the same, there is a problem.
            throw DKException(DKString("Retrieved SMS Collection Definition's ID Code ('")+DKString(retrCollCode)+DKString("') does not match the one just added ('")+DKString(collCode)+DKString("')."));

        delete(smsCollDef);         // Free Memory
        delete(retrSMS);

        cout << "Setup SMS Collection with for RM." << endl;
        return(collCode);
    }//end setupSMSCollectionForRM
