/************************************************************************/
/* @copyright(disclaimer)                                                               */
/* IBM Enterprise Information Portal for Multiplatforms V6.1  (program number 5648-D03) */
/* (c ) Copyright IBM Corp. 1998, 1999. All Rights Reserved.                            */
/* IBM Content Manager for Multiplatforms V6.1  (program number 5648-D04)               */
/* (c ) Copyright IBM Corp. 1994, 1999. All Rights Reserved.                            */
/*                                                                                      */
/* US Government Users Restricted Rights                                                */
/* Use, duplication or disclosure restricted by GSA ADP Schedule                        */
/* Contract with IBM Corp.  Licensed Materials - Property of IBM                        */
/*                                                                                      */
/* DISCLAIMER OF WARRANTIES :                                                           */
/*                                                                                      */
/* Permission is granted to copy and modify this code, and to                           */
/* distribute modified versions provided that both the copyright                        */
/* notice and this permission notice appear in all copies and modified                  */
/* versions. THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM DISCLAIMS                   */
/* ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE. IBM                  */
/* WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE                 */
/* OF THE CODE.                                                                         */
/*                                                                                      */
/* @endCopyright                                                                        */
/************************************************************************/

// sample connect/execute/disconnect to DL
/*
 * Filename       : TEvaluateDL.cpp
 * Description    : Connect to DL Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DL Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKSearchEngineInfoDL.hpp>
#include <DKStorageManageInfoDL.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobDL.hpp>
#include <DKPidXDODL.hpp>
#include <DKEnvironment.hpp>

void processResults(dkCollection* results);
void processFolder(dkCollection* results);
void processParts(dkCollection* results);
void processDDO(DKDDO *p);
void processXDO(dkXDO *pXDO);

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;

    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TEvaluateDL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    try {
    DKDatastoreDL dsDL;
    DKAny aRes;
    DKResults* pCol = 0;
    cout << "Datastore DL created" << endl;
    cout << "connecting to datastore" << endl;
    dsDL.connect(libsrv,userid,pw);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    DKString cmd = "SEARCH=(INDEX_CLASS=DataType_Index,";
    cmd += "MAX_RESULTS=5,";
    cmd += "COND=(Data_FString <> NULL));";
    //    DKString cmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,";
    // cmd += "MAX_RESULTS=5,";
    // cmd += "COND=(DLSEARCH_DocType <> NULL));";
    cmd += "OPTION=(CONTENT=YES;";
    cmd += "TYPE_QUERY=DYNAMIC;TYPE_FILTER=FOLDERDOC)";
    cout << "query string " << cmd << endl;
    cout << "executing query" << endl;
    aRes = dsDL.evaluate(cmd);
    pCol = (DKResults*)((dkCollection*)aRes);
    cout << "query executed" << endl;
    cout << "process results" << endl;
    processResults(pCol);
    cout << "results processed" << endl;
    dsDL.disconnect();
    cout << "datastore disconnected" << endl;
    }
    catch(DKException &exc)
    {

      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;

}

void processDDO(DKDDO* p)
{
 DKPid *pid = 0;
 DKString strPid;
 DKAny a;
 ushort j = 0;
 ushort k = 0;
 ushort val = 0;
 ushort cnt = 1;
 DKString strData = "";
 DKString strDataName = "";
 long lVal = 0;
 short sVal = 0;
 double dVal = 0;
 DKDate dateVal;
 DKTime timeVal;
 DKTimestamp timestampVal;
 DKAny aProp;
 ushort usDataPropType = 0;
 dkCollection* pCol = 0;
 ushort numDataItems = 0;
 pid = (DKPid*)p->getPidObject();
 strPid = pid->pidString();
 cout << "pid string " << strPid << endl;
 k = p->propertyId(DK_CM_PROPERTY_ITEM_TYPE);
 if (k > 0)
 {
  a = p->getProperty(k);
  val = a;
  cout << "******************************" << endl;
  switch (val)
  {
   case DK_CM_DOCUMENT :
   {
    cout << "Item is document " << endl;
    break;
   }
   case DK_CM_FOLDER :
   {
    cout << "Item is folder " << endl;
    break;
   }
  }
  cout << "******************************" << endl;
 }
 k = p->propertyId(DK_CM_PROPERTY_CHECKOUT);
 if (k > 0)
 {
  a = p->getProperty(k);
  strData = a;
  cout << "checkout prop " << strData << endl;
 }
 numDataItems = p->dataCount();
 cout << "Number of Data Items " << numDataItems << endl;
 for (j = 1; j <= numDataItems; j++)
 {
  a = p->getData(j);
  strDataName = p->getDataName(j);
  aProp = p->getDataPropertyByName(j,DK_CM_PROPERTY_TYPE);
  usDataPropType = (ushort)aProp;
  switch (usDataPropType)
  {
   case DK_CM_DATAITEM_TYPE_STRING :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " String value :  null " << endl;
    }
    else
    {
     strData = a;
     cout << "attribute name : " << strDataName << " String value : " << strData << endl;
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_LONG :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " Long value :  null " << endl;
    }
    else
    {
     lVal = a;
     cout << "attribute name : " << strDataName << " Long value : " << lVal << endl;
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_SHORT :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " Short value :  null " << endl;
    }
    else
    {
     sVal = a;
     cout << "attribute name : " << strDataName << " Short value : " << sVal << endl;
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_DOUBLE :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " Double value :  null " << endl;
    }
    else
    {
     dVal = a;
     cout << "attribute name : " << strDataName << " Double value : " << dVal << endl;
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_DATE :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " Date value :  null " << endl;
    }
    else
    {
     dateVal = a;
     cout << "attribute name : " << strDataName << " Date value : " << dateVal << endl;
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_TIME :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " Time value :  null " << endl;
    }
    else
    {
     timeVal = a;
     cout << "attribute name : " << strDataName << " Time value : " << timeVal << endl;
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_TIMESTAMP :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " Timestamp value :  null " << endl;
    }
    else
    {
     timestampVal = a;
     cout << "attribute name : " << strDataName << " Timestamp value : " << timestampVal << endl;
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_COLLECTION_XDO :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " XDO Collection value :  null " << endl;
    }
    else
    {
     pCol = a;
     cout << "attribute name : " << strDataName << " XDO Collection value : " << pCol << endl;
     processParts(pCol);
    }
    break;
   }
   case DK_CM_DATAITEM_TYPE_COLLECTION_DDO :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " DDO Collection value :  null " << endl;
    }
    else
    {
     pCol = a;
     cout << "attribute name : " << strDataName << " DDO Collection value : " << pCol << endl;
     processFolder(pCol);
    }
    break;
   }
   default :
   {
    if (a.typeCode() == DKAny::tc_null)
    {
     cout << "attribute name : " << strDataName << " Unknown type value :  null " << endl;
    }
    else
    {
     cout << "attribute name : " << strDataName << " Unknown type value : " << a << endl;
    }
    break;
   }
  }
 }
}

void processXDO(dkXDO* pXDO)
{
 DKPidXDODL *pidXDO = 0;
 DKSearchEngineInfoDL* pSrchEx = 0;
 DKStorageManageInfoDL* pSmi = 0;
 DKBlobDL *pBlob = (DKBlobDL*)pXDO;
 DKString strPid;
 pidXDO = (DKPidXDODL*)pXDO->getPidObject();
 strPid = pidXDO->pidString();
 cout << "XDO pid string " << strPid << endl;
 if (pBlob->isCategoryOf(DK_DL_INDEXED_OBJECT))
 {
   cout << "blob is a SEARCH INDEXED object" << endl;
   pSrchEx = (DKSearchEngineInfoDL*)pBlob->getExtension("DKSearchEngineInfoDL");
   if (pSrchEx != 0)
   {
      DKString searchEngine = pSrchEx->getSearchEngine();
      DKString searchIndex  = pSrchEx->getSearchIndex();
      DKString searchInfo   = pSrchEx->getSearchInfo();
      cout << "searchEngine=" << searchEngine << " searchIndex=" << searchIndex << " searchInfo=" << searchInfo << endl; 
   }
 }
 pSmi = (DKStorageManageInfoDL*)pBlob->getExtension("DKStorageManageInfoDL");
 if (pSmi != 0)
 {
    long lRetention = pSmi->getRetention();
    DKString smsRetention(lRetention);
    DKString smsCollection   = pSmi->getCollectionName();
    DKString smsMgmtClass    = pSmi->getManagementClass();
    DKString smsStorageClass = pSmi->getStorageClass();
    DKString smsObjServer    = pSmi->getStoreSite();
    cout << "smsRetention=" << smsRetention << " smsCollection=" << smsCollection << " smsMgmtClass=" << smsMgmtClass << " smsStorageClass=" << smsStorageClass << " smsObjServer=" << smsObjServer << endl; 
 }
}

void processResults(dkCollection* results)
{
 dkIterator* iter = results->createIterator();
 DKDDO* item;
 DKAny *element;
 long i = 0;
 cout << "Begin Results **************** " << results << endl;
 cout << " number of items in DKResults " << results->cardinality() << endl;
 while(iter->more()) {
    element = iter->next();
    i++;
    item = (DKDDO*) element->value();
    if (!item) continue;
    cout << "*** DDO number " << i << " ***" << endl;
    //item->retrieve();
    processDDO(item);
 }
 delete iter;
 cout << "End Results **************** " << results << endl;
}

void processFolder(dkCollection* results)
{
 dkIterator* iter = results->createIterator();
 DKDDO* item;
 DKAny *element;
 long i = 0;
 cout << "Begin Folder **************** " << results << endl;
 cout << " number of items in DKFolder " << results->cardinality() << endl;
 while(iter->more()) {
    element = iter->next();
    i++;
    item = (DKDDO*) element->value();
    if (!item) continue;
    cout << "*** DDO number " << i << " ***" << endl;
    //  item->retrieve();
    processDDO(item);
 }
 delete iter;
 cout << "End Folder **************** " << results << endl;
}

void processParts(dkCollection* results)
{
 dkIterator* iter = results->createIterator();
 dkXDO* item;
 DKAny *element;
 long i = 0;
 cout << "Begin Parts **************** " << results << endl;
 cout << " number of items in DKParts " << results->cardinality() << endl;
 while(iter->more()) {
    element = iter->next();
    i++;
    item = (dkXDO*) element->value();
    if (!item) continue;
    cout << "*** XDO number " << i << " ***" << endl;
    //item->retrieve();
    processXDO(item);
 }
 delete iter;
 cout << "End Parts **************** " << results << endl;
}

