/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TADLServers.cpp
 * Description    : test add, delete, list servers
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKAny.hpp>
#include <DKDatastoreFed.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKException.hpp>
#include <DKEnvironment.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;

   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 5)  {
       cout <<"Usage: "<<endl;
       cout <<"\t TADLServers <feddb> <userid> <pw> <add/del/list>" << endl;
       return;
    }
    // get input parameters
    DKString libsrv(argv[1]);
    DKString userid(argv[2]);
    DKString pw    (argv[3]);
    DKString cmd   (argv[4]);
    DKDatastoreFed dsFed;  // must be outside try block to avoid exception trap error.

    try {

      // All set; Connect to federate database
      cout << " *** Connecting to " << libsrv << " *** " << endl;
      dsFed.connect(libsrv, userid, pw);
      cout << " \t -----> connected " << endl;
      cout << "     user " << dsFed.userName() << ", dsName " << dsFed.datastoreName() << endl;


      // get the Def and Admin
      dkDatastoreDef* defFed   = dsFed.datastoreDef();
      DKDatastoreAdminFed* adm = (DKDatastoreAdminFed*) defFed->datastoreAdmin();

      // create your own servers : barney, mufasa, and talisman2 : once !!!

      // (1) this is for mufasa
      DKServerMapDefFed mufasa(&dsFed);
      mufasa.setName("LSMUFASA");
      mufasa.setServerType("DL");
      mufasa.setConnectString("");
      mufasa.setDescription("LSMUFASA : DL Library Server at Mufasa (NT)");
      DKTimestamp tsMuf;
      mufasa.setRefreshTime(tsMuf.asString(NULL));
      if (cmd == "add") {
         cout << "Add Server: mufasa, LSMUFASA - DL Library Server on AIX " << endl;
         adm->addServer(&mufasa);
      }
      else if (cmd == "del") {
         cout << "Delete Server: mufasa, LSMUFASA - DL Library Server on AIX " << endl;
          adm->removeServer(mufasa.getName());
      }

      // (2) this is for talisman2 a DL Library Server on NT
      DKServerMapDefFed talisman2(&dsFed);
      talisman2.setName("LSCHIUN");
      talisman2.setServerType("DL");
      talisman2.setConnectString("");
      talisman2.setDescription("LSCHIUN : DL Library Server at talisman2 (NT) ");
      DKTimestamp ts2;
      talisman2.setRefreshTime(ts2.asString(NULL));
      if (cmd == "add") {
         cout << "Add Server: talisman2, LSCHIUN - DL Library Server on NT " << endl;
         adm->addServer(&talisman2);
      }
      else if (cmd == "del") {
         cout << "Delete Server: talisman2, LSCHIUN - DL Library Server on NT " << endl;
         adm->removeServer(talisman2.getName());
      }

      // (3) this is for TMMUF - TM Text Search Server in Mufasa
      DKServerMapDefFed tmmuf(&dsFed);
      tmmuf.setName("TMMUF");
      tmmuf.setServerType("TS");
      tmmuf.setConnectString("");
      tmmuf.setDescription("TMMUF : TM Text Search Server at Mufasa (AIX)");
      
      DKTimestamp tsTM;
      tmmuf.setRefreshTime(tsTM.asString(NULL));
      if (cmd == "add") {
          cout << "Add Server: mufasa, TMMUF - TM Text Search Server on AIX" << endl;
          adm->addServer(&tmmuf);
          adm->assocSearchServer("LSMUFASA","TMMUF");
      }
      else if (cmd == "del") {
          cout << "Delete Server: mufasa, TMMUF - TM Text Search Server on AIX" << endl;
          adm->removeServer(tmmuf.getName());
          adm->disassocSearchServer("LSMUFASA","TMMUF");
      }
         
      // list back-end servers
      cout << "\nList server names " << endl;
      long len, i;
      DKString* servers = adm->listServerNames(len);
      DKServerMapDefFed* serverdat = NULL;

      for (i = 0; i < len; i++) {
           cout << "server [" <<  i << "] = " << servers[i] << endl;
          serverdat = adm->getServer(servers[i]);
          cout << " DKServerMapDefFed = " << serverdat->getName() << endl;
          cout << "     name: " << serverdat->getName()           << endl;
          cout << "     type: " << serverdat->getServerType()     << endl;
          cout << " connStr : " << serverdat->getConnectString()  << endl;
          cout << "    desc : " << serverdat->getDescription()    << endl;
          cout << " refTime : " << serverdat->getRefreshTime()    << endl;
          delete serverdat;

      }  
      if (len == 0)
         cout << "No server defined." << endl;
      else
         delete [] servers;

      // disconnects
      cout << "\n\n *** disconnect from the datastore Fed. ***" << endl;
      dsFed.disconnect();
      cout << " *** datastore Fed disconnected ***" << endl;
      
    }
    catch(DKException &exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
      
      cout << "Exception Class Name: " << exc.name() << endl;
    }
   
    cout << "done ..." << endl;   
}
