/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolV4.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */
#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKBlobV4.hpp>
#include <DKDatastoreV4.hpp>
#include <DKDatastorePool.hpp>
#include <DKDatastoreDefV4.hpp>
#include <DKAttrDefV4.hpp>
#include <DKServerDefV4.hpp>
#include <DKIndexClassDefV4.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolV4 <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKString factoryV4LName = "cmbv4fac816d";

   try {
      DKDatastorePool* dsrV4 = new DKDatastorePool("V4", factoryV4LName);
      dsrV4->setDatastoreName(libsrv);
      dsrV4->setMaxPoolSize(10);
      dsrV4->setMinPoolSize(3);
      dsrV4->setTimeOut(1); 

      dsrV4->initConnections(userid, pw, 2);
      //dsrV4->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrV4->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrV4->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrV4->getTimeOut()<<endl;

      dkDatastore *dsV41, *dsV42, *dsV43, *dsV44;;
      dsV41 = dsrV4->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsV41(ds1) =  " << dsV41 << endl;
      dsV42 = dsrV4->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsV42(ds2) =  " << dsV42 << endl;

      DKDatastoreSession *dsIdV41, *dsIdV42, *dsIdV43, *dsIdV44;
      dsIdV41 = dsrV4->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdV41(dsid1)="<< dsIdV41->getDatastore()
        <<", sessionId=" << dsIdV41->getId()<<endl;
      dsIdV42 = dsrV4->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdV42(dsid2)="<<dsIdV42->getDatastore()
        <<", sessionId=" << dsIdV42->getId()<<endl;

      dsrV4->returnConnection(dsIdV42);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdV42(dsid2) =  " << 
        dsIdV42->getDatastore() << ", sessionId=" << dsIdV42->getId()<<endl;
      dsrV4->returnConnection(dsV42);                       //ds2 in _freePool
      cout<<" #returnConnection, dsV42(ds2) =  " << dsV42 <<endl; 

      dsIdV43 = dsrV4->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdV43(ds2)=" <<
        dsIdV43->getDatastore() << ", sessionId=" << dsIdV43->getId()<<endl;
      dsIdV44 = dsrV4->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdV44(dsid3)=" <<
        dsIdV44->getDatastore() << ", sessionId=" << dsIdV44->getId() <<endl;


      dsIdV44 = dsrV4->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdV44(dsid4)=" <<
        dsIdV44->getDatastore() << ", sessionId=" << dsIdV44->getId()<<endl;

      dsV41 = dsrV4->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsV41(ds3) =  " << dsV41 <<endl;
      dsV42 = dsrV4->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsV42(ds4) =  " << dsV42 <<endl;
 
      dsV43 = dsrV4->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsV43(ds5) =  " << dsV43 <<endl;
      dsV44 = dsrV4->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsV44(ds6) =  " << dsV44 <<endl;


      dsIdV43 = dsrV4->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdV43(dsid2)=" <<
      dsIdV43->getDatastore() << ", sessionId=" << dsIdV43->getId()<<endl;


/**/
      //**** use dsV42 to do ListCatalog
      cout<<"||**** start: use dsV42 to do ListCatalog ****||"<<endl;
      DKSequentialCollection *pCol = 0;
      dkIterator *pIter = 0;
      DKSequentialCollection *pCol2 = 0;
      dkIterator *pIter2 = 0;
      DKServerDefV4 *pSV = 0;
      DKIndexClassDefV4 *pEnt = 0;
      DKString strServerName;
      DKString strServerType;
      DKString strIndexClass;
      DKAttrDefV4 *pAttr = 0;
      DKAny a;
      DKAny *pA = 0;
      long i = 0;
      long j = 0;
      cout << "list servers" << endl;
      a = dsV42->listDataSources();
      pCol = (DKSequentialCollection*)((dkCollection*)a);
      pIter = pCol->createIterator();
      while (pIter->more() == TRUE)
      {
       i++;
       pSV = (DKServerDefV4*)((void*)(*pIter->next()));
       strServerName = pSV->getName();
       strServerType = pSV->getServerType();
       cout << "Server Name [" << i << "] - " << strServerName << " Server Type - " << strServerType << endl; 
       delete pSV;
      }
      delete pIter;
      delete pCol;
      cout << i << " servers listed" << endl;
      cout << "list index classes" << endl;
      pCol = (DKSequentialCollection*)((dkCollection*)dsV42->listEntities());
      pIter = pCol->createIterator();
      i = 0;
      while (pIter->more() == TRUE)
      {
       i++;
       pEnt = (DKIndexClassDefV4*)((void*)(*pIter->next()));
       strIndexClass = pEnt->getName();
       cout << "index class name [" << i << "] - " << strIndexClass << endl;
       cout << "  list attributes for " << strIndexClass << " index class" << endl;
       pCol2 = (DKSequentialCollection*)((dkCollection*)dsV42->listEntityAttrs(strIndexClass));
       pIter2 = pCol2->createIterator();
       j = 0;
       while (pIter2->more() == TRUE)
       {
        j++;
        pA = pIter2->next();
        pAttr = (DKAttrDefV4*) pA->value();
        cout << "    Attribute name [" << j << "] - " << pAttr->getName() << endl;
        cout << "      datastoreName " << pAttr->datastoreName() << endl;
        cout << "      datastoreType " << pAttr->datastoreType() << endl;
        cout << "      attributeOf   " << pAttr->getEntityName() << endl;
        cout << "      type          " << pAttr->getType() << endl;
        cout << "      size          " << pAttr->getSize() << endl;
        cout << "      id            " << pAttr->getId() << endl;
        cout << "      nullable      " << pAttr->isNullable() << endl;
        cout << "      precision     " << pAttr->getPrecision() << endl;
        cout << "      scale         " << pAttr->getScale() << endl;
        cout << "      string type   " << pAttr->getStringType() << endl;
        delete pAttr;
       }
       cout << "  " << j << " attributes listed for " << strIndexClass << " index class" << endl;
       delete pIter2;
       delete pCol2;
       delete pEnt;
      }
      delete pIter;
      delete pCol;
      cout << i << " index classes listed" << endl;
      cout<<"||**** end: use dsV42 to do ListCatalog ****||"<<endl;
      //****end of use dsV42 to do ListCatalog 
/**/
      dsrV4->returnConnection(dsV42);
      cout<<" #returnConnection, dsV42(ds4) =  "<< dsV42 <<endl;
      dsV44 = dsrV4->getConnection(userid, pw);
      cout<<" #getConnection, dsV44 =  "<< dsV44 <<endl;
      cout<<"    dsV44 == dsV42 =  " <<(dsV44 == dsV42)<<endl;
/**
      //**** use dsV44 to retrieve XDO - need to change the itemId and partId  
      cout<<"||**** start: use dsV44 to retrieve XDO ****||"<<endl;
       //DKString itemId="DB00301AA.AAC";  //stealth3
       DKString itemId="DA99103AA.AAG"; 
       int partId = 1;
       DKBlobV4* axdo = new DKBlobV4((DKDatastoreV4*)dsV44);
       DKPidXDOV4*  apid = new DKPidXDOV4;
       apid ->setPartId(partId);
       apid ->setPrimaryId(itemId);
       axdo ->setPidObject(apid);
       cout<<"itemId= "<<((DKPidXDOV4*)axdo->getPidObject())->getPrimaryId()<<endl;
       cout<<"partId= "<<((DKPidXDOV4*)axdo->getPidObject())->getPartId()<<endl;
       cout<<"before retrieve..."<<endl;
       cout <<" length of lobdata = "<<axdo->length()<<endl;
       cout<<"about to call retrieve()..."<<endl;
       axdo->retrieve();
       cout<<"after retrieve..."<<endl;
       cout <<" length of lobdata = "<<axdo->length()<<endl;
       cout <<" mimeType = "<<axdo->mimeType()<<endl;
       cout <<" size of lobdata = "<<axdo->getSize()<<endl;
       cout<<" createdTimestamp="<<axdo->getCreatedTimestamp()<<endl;
       cout<<" updatedTimestamp="<<axdo->getUpdatedTimestamp()<<endl;
       //cout<<"about to do open()..."<<endl;
       //axdo->setInstanceOpenHandler("notepad", TRUE);
       //axdo->open();
       delete axdo;
       delete apid;   
      cout<<"||**** end: use dsV44 to retrieve XDO ****||"<<endl;
      //****end of use dsV44 to retrieve XDO
**/
      dsrV4->returnConnection(dsV41);
      cout<<" #returnConnection, dsV41(ds3) =  " << dsV41 <<endl;
      dsrV4->returnConnection(dsV43);
      cout<<" #returnConnection, dsV43(ds5) =  " << dsV43 <<endl;
      dsrV4->returnConnection(dsIdV44); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdV44(dsid4) =  " <<
        dsIdV44->getDatastore() << ", sessionId=" << dsIdV44->getId() <<endl;
     

      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;


      dsV42 = (DKDatastoreV4*)dsrV4->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsV42(ds7) =  " << dsV42<<endl;
      cout<<" #call delete dsrV4..." <<endl;
      delete dsrV4;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

