/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/********************************************************
 * txdoCRUDV4.cpp - test the xdo CRUD and open.
 * (the default open handler is "iexplore", the directory
 *  where the iexplore is should be in the PATH.  If
 *  the retrieved file is a text file then set the open	
 *  handler to "notepad".  This test case will run in	
 *  Window NT, for AIX you need to set open handler to "xv".
 ********************************************************/
#include <iostream.h>
#include <stdio.h>  
#include <stdlib.h>
#include <DKConstantV4.h>
#include <DKString.hpp>
#include <DKDatastoreV4.hpp>
#include <DKException.hpp>
#include <DKHandle.hpp>
#include <DKBlobV4.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //== Following default partId and itemId will be provided if you
    //== run this test case without parameters, If you know existing
    //== partId and itemId, invoke: txdoCRUDV4 <partId> <itemId>
    //== unsigned long    partId = 17;  
    //== DKString   itemId = "R4BEI096O5GM3DDB";
    //==   
    //== Following fileName will be used for add a new part object,
    //== It must exist in the same directory, change the fileName
    //== if you want to add other file
    //== DKString   fileName = "choice.gif";
    //==   
    //== Following fileName will be used for update the part object,
    //== It must exist in the same directory, change the fileName
    //== if you want to update with other file
    //== fileName = "cheetah.gif";

    unsigned long    partId = 1;
    DKString itemId = "DB02175AA.AAF";
    DKString fileName1 = "TParamF1.txt";
    DKString fileName2 = "TParamF2.txt";
     
    cout <<"argc is "<<argc<<endl;
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TxdoCRUDV4 <libsrv> <userid> <pw> <partId> <itemId>" << endl;
     return;
    }
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);

    if (argc == 4)
    {
      cout<<"no parameter for partId, itemId, following default will be provided:"<<endl;
      cout<<"   default partId = "<<partId<<endl;
      cout<<"   default itemId = "<<itemId<<endl;
    }
    else if (argc == 5)
    {
     partId = atoi(argv[4]);
     cout<<"partId entered: "<<argv[4]<<endl;
     cout<<"   The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 6)
    {
     partId = atoi(argv[4]);
     itemId = DKString(argv[5]);
     cout<<"partId/itemId entered: "<<argv[4]<<" "<<argv[5]<<endl;
    }

    DKDatastoreV4 dsV4; // must be outside try/catch block

    try {
    //======== connect to Datastore =======================
    cout << "Connecting to datastore, please wait...." << endl;
    dsV4.connect(libsrv, userid, pw);
    cout << "Datasore connected" << endl;
        
    DKBlobV4* axdo = new DKBlobV4(&dsV4);
    DKPidXDOV4  apid;
    //== create pid ==
    apid.setPartId(partId);
    apid.setPrimaryId(itemId);
    axdo ->setPidObject(&apid);
 
    //cout <<"repType="<< axdo->getRepType()<<endl;  
    //cout <<"contentclass="<< axdo->getContentClass()<<endl;
    cout <<"itemId=" << axdo->getItemId()<<endl;
    cout <<"partId="<<axdo->getPartId()<<endl;
    
    //====== retrieve the existing part object =============
    cout <<"before retrieve() size="<<axdo->getSize()<<endl;
    cout <<"about to call retrieve()"<<endl;
    axdo ->retrieve(); 
    cout <<"after retrieve() size="<<axdo->getSize()<<endl;  
    cout <<"  contentclass="<< axdo->getContentClass()<<endl;

     cout <<"aff type= "<<axdo->getAffiliatedType()<<endl;
     int atype = axdo->getAffiliatedType();
     if (atype == DK_ANNOTATION)
     {
        DKAnnotationV4* ann = (DKAnnotationV4*)axdo->getExtension("DKAnnotationV4");
        cout<<"  partId="<<ann->getPart()<<endl;
        cout<<"  pageNumber= "<<ann->getPageNumber()<<endl;
        cout<<"  X= "<<ann->getX()<<endl;
        cout<<"  Y= "<<ann->getY()<<endl;	
     }

    cout <<"about to do open()"<<endl;
    axdo->setInstanceOpenHandler("notepad ", TRUE);   //use lviewpro in NT
    //axdo->open("mine.txt");     //iexplore requires the file extension  

    //====== add a new part object =========================
    //== before add, the content length is the previous object
    //delete axdo;
    //axdo = new DKBlobV4(&dsV4);

    partId = 0;                          //let V4 to decide the partId
    apid.setPartId(partId);
    apid.setPrimaryId(itemId);
    axdo->setPidObject(&apid); 
    axdo->setContentClass(DK_V4_CC_ASCII); //set content class GIF for part object
    cout <<"contentclass="<< axdo->getContentClass()<<endl;
    cout <<"objecttype=" << axdo->getObjectType()<<endl;
    cout <<"itemId=" << axdo->getItemId()<<endl;
    cout <<"partId="<<axdo->getPartId()<<endl;
    cout<<"before add, content length = "<<axdo->length()<<endl;
    cout <<"about to do add()..."<<endl;
    axdo->setContentFromClientFile(fileName1); //set XDO content from a file
    axdo->add();                    //add part object to V4 with same itemId
    cout<<"The added partId(assign by system) = ";
    cout<< ((DKPidXDOV4*)(axdo->getPidObject()))->getPartId()<<endl;
    cout<<"After add, do retrieve and open() to verify it is added.."<<endl;
    cout <<"before retrieve() size="<<axdo->getSize()<<endl;
    axdo->retrieve();
    cout<<"after add & retrieve, content length = "<<axdo->length()<<endl;
    cout <<"after retrieve() size="<<axdo->getSize()<<endl;
    //axdo->setInstanceOpenHandler("notepad ", TRUE);   //use lviewpro in NT
    //axdo->open();     //iexplore requires the file extension 

    //====== update the added part object ==================
    //== before update, the content length is the same
    cout<<"before update, content length = "<<axdo->length()<<endl;
    cout<<"before update, content size = "<<axdo->getSize()<<endl;
    axdo->setContentFromClientFile(fileName2); //set XDO content from a file
    cout<<"about to do update()..."<<endl;
    axdo->update();  
    cout<<"After update, do retrieve and open() to verify it is updated."<<endl;
    axdo->retrieve();
    cout<<"after update & retrieve, content length = "<<axdo->length()<<endl;
    cout<<"after update & retrieve, content size = "<<axdo->getSize()<<endl;
    //axdo->open();     //iexplore requires the file extension


    //====== delete the added part object ==================
    cout<<"Delete the above added object from V4 datastore..."<<endl;
    axdo->del();
    cout<<"delete successfully!"<<endl;

    //======== disconnect Datastore ======================= 
    delete axdo;
    dsV4.disconnect();
    cout<<"datastore disconnected"<<endl;
    } //end of try

    catch(DKException &exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }

    cout << "done ..." << endl;

 }

