/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * This sample adds a note to a note log for a document or folder.
 * <p>
 * Note: CMv7, CMv8, VI/400, IP/390 support note log.   Other servers will throw not supported errors.
 */
public class TAddNote {

	public static void main(String[] args) throws Exception {
		if (args.length < 6) {
			System.out.print(
				"Adds a text note to the notelog of a document. \n"
					+ "Syntax: \n"
					+ "  java TAddNote <dstype> <server> <userid> <password> <entity> <condition> / <noteText>\n"
					+ "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, V4, IP) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, index class) to search. \n"
					+ "  <condition> is a search condition. \n"
					+ "  <noteText> is the text of the note.  It will be appended to the existing note log text for the item\n"
					+ "The condition and note text are separated with a slash. \n"
					+ "Examples: \n"
					+ "  java TAddNote ICM icmnlsdb icmadmin password Document @Title=\"Title\" / Add this note to the log \n");
			return;
		}

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        int i;
        for (i = 5; i < args.length && !args[i].equals("/"); i++) {
            condition += " " + args[i];
        }
        String note = "";
        for (i++; i < args.length && !args[i].equals("/"); i++) {
            note += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

		// Search for items to add the note to
        TSearch.search(queryService, dstype, entity, condition, false);
		searchResults.newResults(queryService.getResults());
		System.out.println(""+searchResults.getCount()+" items found.");
		for (i = 0; i < searchResults.getCount(); i++) {
			CMBItem item = searchResults.getItem(i);

			// Display the current note log on the item
			dataManagement.setDataObject(item);
			System.out.println("   Item's previous note log:");
			if (dataManagement.getNoteLogCount() == 0) {
				System.out.println("      none");
			} else {
				System.out.println(new String(dataManagement.getNoteLog(0).getData()));
			}

			// Add the note
			System.out.println("  Adding note.");
			addNote(dataManagement, item, note);

			// Display the updated note log on the item
			dataManagement.setDataObject(item);
			System.out.println("   Item's new note log: ");
			if (dataManagement.getNoteLogCount() == 0) {
				System.out.println("      none");
			} else {
				System.out.println(new String(dataManagement.getNoteLog(0).getData()));
			}
		}

		System.out.println("Disconnecting from server");
		connection.disconnect();

	}

	/**
	 * Adds a note to an item.
	 * @param dataManagement an instance of CMBDataManagement bean
	 * @param item an instance of CMBItem for the document
	 * @param text text for note
	 */
	public static void addNote(CMBDataManagement dataManagement, CMBItem item, String text) 
	throws CMBException {

		dataManagement.setDataObject(item);
		CMBObject notelog;
		if (dataManagement.getNoteLogCount() == 0) { // first note
			notelog = new CMBObject();
		} else {
			notelog = dataManagement.getNoteLog(0);
		}
		String noteLogAsText = new String(notelog.getData());
		noteLogAsText += "\n---\n" + text;
		notelog.setData(noteLogAsText.getBytes());
		notelog.setMimeType("text/plain");
		if (dataManagement.isSupported("checkOut")) { // some servers require checkout
			dataManagement.checkOut();
		}
		if (dataManagement.getNoteLogCount() == 0) {
			dataManagement.addNoteLog(notelog);
		} else {
			dataManagement.updateNoteLog(0,notelog);
		}
		if (dataManagement.isSupported("checkIn")) {
			dataManagement.checkIn();
		}
	}

}