/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import java.io.*;
import com.ibm.mm.beans.*;
import com.ibm.mm.viewer.*;

/**
 * Convert documents and view in a browser.  A search is performed
 * to find the documents to view.  Documents are paginated if possible and
 * viewed page-by-page.  If a document cannot be paginated, it is converted
 * to browser-friendly format.  If that can't be done, it is returned
 * in the original format to the browser (which may launch a plugin to
 * view the document).
 */
public class TView {
	
    static final String BROWSER = "C:\\Program Files\\Internet Explorer\\Iexplore.exe";
    static int suffix = 0; // an integer appended to generated file names

    public static void main(String args[]) throws Exception {
        if (args.length < 5) {
            System.out.println(
                "Syntax: \n"
                    + "  java TView <dstype> <server> <userid> <password> <entity> <prefix> <condition> / <options> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, table) to search. \n"
                    + "  <prefix> the prefix of name of the file to use for generated files. \n"
                    + "  <condition> the condition portion of the query string.  The syntax \n"
                    + "              of the condition varies depending on server type. \n"
                    + "  <options> any number of the following options: \n"
                    + "    NOBROWSE -- the browser will not be launched. \n"
                    + "    SCALE <n> -- the scale of generated pages. \n"
                    + "    ROTATE <n> -- the number of 90 degree rotations. \n"
                    + "    ANNOTATE -- show with annotations (if possible). \n"
                    + "Examples: \n"
                    + "  java TView ICM icmnlsdb icmadmin password Journal01 image @JTitle01 LIKE \"%\" \n"
                    + "  java TView DL libsrvrn frnadmin password NOINDEX image Userid LIKE '%' \n"
                    + "  java TView DB2 sample db2admin password EMP_PHOTO image PHOTO_FORMAT LIKE 'gif' \n"
                    + "  java TView OD odserver userid password Balance image CustName like '%DAGWOOD%'");
            System.exit(0);
        }

        // Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String prefix = args[5];
        String condition = "";
        int arg;
        for (arg = 6; arg < args.length && ! args[arg].equals("/"); arg++) {
            condition += " " + args[arg];
        }
        if (arg < args.length) {
        	arg++; // skip "/"
        }
        boolean browse = true;
        double scale = 1.0;
        int rotation = 0;
        boolean annotate = false;
       	while (arg < args.length) {
       		if (args[arg].equalsIgnoreCase("nobrowse")) {
       			browse = false;
       		} else if (args[arg].equalsIgnoreCase("scale")) {
       			arg++;
       			scale = Double.parseDouble(args[arg]);
       		} else if (args[arg].equalsIgnoreCase("rotate")) {
       			arg++;
       			rotation = Integer.parseInt(args[arg]);
       		} else if (args[arg].equalsIgnoreCase("annotate")) {
       			annotate = true;
       		} else {
       			System.out.println("Unknown option: "+ args[arg] +", ignored.");
       		}
        	arg++;
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();
        CMBDocumentServices documentServices = new CMBDocumentServices();
        documentServices.setDataManagement(dataManagement);

        // Connect to server
        TConnect.connect(connection, dstype, server, userid, password);

        // Search to find items to view
        TSearch.search(queryService, dstype, entity, condition, false);
        searchResults.newResults(queryService.getResults());
        System.out.println("There are " + searchResults.getCount() + " items found.");

        // View the items found
        if (searchResults.getCount() == 0) {
            System.out.println("No items deleted");
        } else {
            for (int i = 0; i < searchResults.getCount(); i++) {
                CMBItem item = searchResults.getItem(i);
                view(dataManagement, documentServices, item, prefix, scale, rotation, annotate, browse);
            }
        }

        // Disconnect from server
        TConnect.disconnect(connection);
        System.exit(0);
    } // main

	/**
	 * View the content of a document.  The document is
	 * converted to a browser viewable format using CMBDocumentServices
	 * bean if a conversion is supported.  If not, the document is written
	 * unconverted.  For documents that are paginated, each page is converted
	 * into a separate file.
	 * @param dataManagement an instance of CMBDataManagement bean
	 * @param documentServices an instance of CMBDocumentServices bean
	 * @param item an instance of CMBItem for the document to be viewed
	 * @param prefix the name to give all files generated
	 * @param scale the scale of pages (if paginated)
	 * @param rotation the rotation of pages (if paginated)
	 * @param annotate show annotations on page (if possible)
	 */
    public static void view(CMBDataManagement dataManagement, CMBDocumentServices documentServices, 
    						CMBItem item, String prefix, double scale, int rotation, boolean annotate, boolean browse)
        throws CMBException, Exception {
        try {
	        System.out.println("  Item = " + item.getName());
            dataManagement.setDataObject(item);
            System.out.println("    Mimetype = " + dataManagement.getItemMimeType());
            System.out.println("    Content size = " + dataManagement.getContent(0).getSize());
            CMBDocument doc = documentServices.loadDocument(item);
            if (doc.getCanPaginate()) {
                CMBPage[] pages = doc.getPages();
                System.out.println("    There are: " + doc.getPageCount() + " pages in this document");
                doc.setPreferredScale(scale);
                doc.setRotation(rotation);
                doc.setShowAnnotations(annotate);
                for (int j = 0; j < doc.getPageCount(); j++) {
                    viewPage(pages[j], prefix, browse);
                }
            } else {
                System.out.println("    The document cannot be paginated, viewing document in its entirety");
                viewDoc(doc, prefix, browse);
            }

            documentServices.dropDocument(doc);
        } catch (CMBNoContentException e) {
            System.out.println(e.getMessage());
        }
    } // view

    /**
     * View a page.  Scale is default 1.
     * @param page a CMBPage object representing the page to view.
     * @param prefix the prefix to give all files generated.
     */
    public static void viewPage(CMBPage page, String prefix, boolean browse) {
        String mimeType = page.getWriteMimeType();
        String fileName =
            System.getProperty("user.dir") + File.separator + prefix + (suffix++) + "." + TExport.getExtension(mimeType);
        try {
            DataOutputStream ostream = new DataOutputStream(new FileOutputStream(fileName));
            page.write(ostream);
            ostream.close();
            if (browse) {
	            browse(fileName);
            }
        } catch (Exception e1) {
            e1.printStackTrace();
        }
    } //viewPage

    /**
     * View a document.  It is converted to a browser-friendly format if possible.
     * @param doc an instance of CMBDocument for the document to view.
     * @param outfileprefix the prefix to give all files generated.
     */
    public static void viewDoc(CMBDocument doc, String prefix, boolean browse) {
        String mimeType = doc.getWriteMimeType();
        String fileName =
            System.getProperty("user.dir") + File.separator + prefix + (suffix++) + "." + TExport.getExtension(mimeType);
        try {
            File wfile = new File(fileName);
            DataOutputStream ostream = new DataOutputStream(new FileOutputStream(wfile));
            doc.write(ostream);
            ostream.close();
            if (browse) {
	            browse(fileName);
            }
        } catch (Exception e1) {
            e1.printStackTrace();
        }
    } //viewDoc
    
    /**
     * Browse the file.  This is performed only on windows.  For other platforms,
     * the temporary file for the document is left.  It should be deleted by the user.
     * @param fileName filename of file to browser.
     */
    public static void browse(String fileName) {
        String osname = System.getProperty("os.name");
        if (osname.startsWith("Windows")) {
            boolean viewfailed = false;
            Runtime rt = Runtime.getRuntime();
            Process p = null;
            String cmd = BROWSER + " File://" + fileName;
            try {
                p = rt.exec(cmd);
                p.waitFor();
                if (p.exitValue() != 0) {
                    viewfailed = true;
                }
            } catch (Throwable e) {
                viewfailed = true;
            }
            if (viewfailed) {
                System.out.println("      Browser launch failed.  File kept as " + fileName);
            } else {
                File file = new File(fileName);
                if (file.exists()) {
                    file.delete();
                }
            }
        } else {
        	System.out.println("      Browser not defined.  Temporary file kept as " + fileName);
        }
    } //browse

}