/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample submit a parametric query to DB2 datastore

/*
 * Filename       : TSampleSQryDB2.java
 * Description    : Connect to DB2 Datastore
 *                  Create query string
 *                  Create a sql query object and submit query
 *                  get the results in a collection.
 *                  display documents.
 *                  Disconnect from DB2 Datastore
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.Date;

// TSampleSQryDB2 class
public class TSampleSQryDB2 implements DKConstantDB2 {

  // Main method
  public static void main(String argv[]){
    
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //*** for input parameters
      String database = new String(""),   // database
             userid = new String(""),     //userid 
  	     pw = new String(""); // pw

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TSampleSQryDB2 " +
        "<database> <userid> <pw>" );
       return;
      }
      else
      {
       database = argv[0];
       userid = argv[1];
       pw = argv[2];
      } 

    DKDatastoreDB2 dsDB2 = null;
    try {
       dsDB2 = new DKDatastoreDB2();
       dkQuery pQry = null;
       String cmd = "";
       DKResults pResults = null;
       DKNVPair parms1[] = null;
       DKNVPair parms[] = new DKNVPair[2];
       String strMax = "5";
       parms[0] = new DKNVPair(DK_CM_PARM_MAX_RESULTS,strMax);
       parms[1] = new DKNVPair(DK_CM_PARM_END,null);
       System.out.println("connecting to datastore"); 
       dsDB2.connect(database,userid,pw,"");
       System.out.println("datastore connected " + database + " userid - " + userid); 
       cmd = "SELECT * FROM EMPLOYEE";
       System.out.println("query string " + cmd);
       System.out.println("create query");
       pQry = dsDB2.createQuery(cmd,DK_CM_SQL_QL_TYPE,parms1);
       System.out.println("executing query");
       pQry.execute(parms);
       System.out.println("query executed");
       System.out.println("number of results " + pQry.numberOfResults());
       System.out.println("process query results");
       pResults = (DKResults)pQry.result();
       processResults((dkCollection)pResults);
       System.out.println("query results processed");
       dsDB2.commit();
       dsDB2.disconnect();
       System.out.println("datastore disconnected"); 
       dsDB2.destroy();
    }
    catch(DKException exc)
    {
     try {
     dsDB2.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
     dsDB2.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
  }
 
  public static void processResults(dkCollection results) 
                throws DKException, Exception
  {
    Short    sVal;
    Integer  iVal;
    short    itemType = 0;
    DKDDO    item = null;
    short    i = 0;
    String   dataName;
    DKParts  parts = null;
    DKFolder folder = null;
    System.out.println( "number of items " + results.cardinality());
 
    dkIterator iter = results.createIterator();
 
    while(iter.more()) {
       item = (DKDDO) iter.next();
       if (item != null) {
	   //   	  item.retrieve(); //Only valid if Primary Key exists for Table
   	  sVal = (Short)item.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
   	  itemType = sVal.shortValue();
   	  switch(itemType) {
   	    case DK_CM_DOCUMENT:
	      {
   	      System.out.println(" Item is a document, number of data item : " + item.dataCount());
   	      for (i = 1; i <= item.dataCount(); i++) {
   		   dataName = item.getDataName(i);
   		   System.out.println("    data-item[" + i + "] = " + dataName + ", value " + item.getData(i));
   		   }
   	      }
   	      break;
   	    default:
   	      System.out.println("  Item is not recognized ");
   	      break;
   	  } 
       }
    } 
  }
}






