/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// test a retrieve of a document from DES, with mapping, to simulate a retrieve from Federated

/*
 * Filename       : TSamplePQryDES.java
 * Description    : Connect to DES Datastore
 *                  Create query string
 *                  Create a parametric query object and submit query
 *                  get the results in a collection.
 *                  display documents.
 *                  recursively display folder and its document/
 *                  folder content retrieving each document/folder
 *                  as it goes.
 *                  Disconnect from DES Datastore * Change History :
 *
 * Date        Name  Description
 * -------    -----  ---------------------------
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

public class TSamplePQryDES implements DKConstantDES {

  // Main method
  public static void main(String argv[]){
   
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.

   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
             pw = new String(""), // pw
             connect_string = "";

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length != 4)
      {
       System.out.println("Usage: " );
       System.out.println("java TExecuteDES " +
        "tcp/ipAddress " +
        "userid " +
        "userpw " +
        "PORT=80;DESAPPID=appid;DESAPPPW=apppw[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>]");
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
       connect_string = argv[3];
      }

    DKDatastoreDES dsDES = null;
	dkQuery pQry = null;
	DKResults pResults = null;
    try {
		dsDES = new DKDatastoreDES();
		dkResultSetCursor pCur = null;
		DKNVPair parms[] = null;
		System.out.println("connecting to datastore");
		dsDES.connect(libSrv,userid,pw,connect_string);
		System.out.println("datastore connected libSrv " + libSrv + " userid " +
							userid );

	String cmd = "SEARCH=(DATABASE=(\"Notes 5 Help\");COND=((IN \"Subject\" \"How\"));";
    cmd +=	"OPTION=(MAX_RESULTS=1;" + 
			"TIME_LIMIT=120))";

        System.out.println("query string " + cmd);
        System.out.println("create query");
        pQry = dsDES.createQuery(cmd,DK_CM_PARAMETRIC_QL_TYPE,parms);
        System.out.println("executing query");
        pQry.execute(parms);
        System.out.println("query executed");
        System.out.println("number of results " + pQry.numberOfResults());
        System.out.println("process query results");
        pResults = (DKResults)pQry.result();
        processResults((dkCollection)pResults);
        System.out.println("query results processed");
        dsDES.disconnect();
        System.out.println("datastore disconnected"); 
        dsDES.destroy();
     }
     catch(DKException exc){
     	try {
     	    dsDES.destroy();
     	}
     	catch (Exception e)
     	{
     	 e.printStackTrace();
     	}      
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
     }
     catch (Exception exc){
     	try {
     	    dsDES.destroy();
     	}
     	catch (Exception e)
     	{
     	 e.printStackTrace();
     	}      
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
     }
  }
 
  public static void processResults(dkCollection results) throws DKException, Exception
  {
     Short    sVal;
     short    itemType = 0;
     DKDDO    item = null;
     short    i = 0;
     String   dataName;
     DKParts  parts = null;
     DKFolder folder = null;
     System.out.println( "number of items " + results.cardinality());
 
     dkIterator iter = results.createIterator();
 
     while(iter.more()) {
        item = (DKDDO) iter.next();
        if (item != null) {
   	       item.retrieve();
   	       sVal = (Short)item.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
   	       itemType = sVal.shortValue();
   	       switch(itemType) {
   	          case DK_CM_DOCUMENT:
   	              System.out.println(" Item is a document, number of data item : " + item.dataCount());
   	              for (i = 1; i <= item.dataCount(); i++) {
   		              dataName = item.getDataName(i);
   		              System.out.println("    data-item[" + i + "] = " + dataName + ", value " + item.getData(i));
   		              if (dataName.equals(DKPARTS) == true) {
   		                  parts = (DKParts)item.getData(i);
   		                  if (parts != null) 
   			                 System.out.println("   this document has " + parts.cardinality() + " parts ");
   		              }
   	              }
   	              break;
   	          case DK_CM_FOLDER:
   	              System.out.println(" Item is a folder,    number of data item : " + item.dataCount());
   	              for (i = 1; i <= item.dataCount(); i++) {
   		              dataName = item.getDataName(i);
   		              System.out.println("    data-item[" + i + "] = " + dataName + ", value " + item.getData(i));
   		              if (dataName.equals(DKPARTS) == true) {
   		                 parts = (DKParts)item.getData(i);
   		                 if (parts != null) 
   			                 System.out.println("   this folder has " + parts.cardinality() + " parts ");
   		              }
   		              if (dataName.equals(DKFOLDER) == true) {
   		                 folder = (DKFolder)item.getData(i);
   		                 System.out.println("   this folder has " + folder.cardinality() + " documents ");
   		              }  
   	              }
   	              if (folder != null && folder.cardinality() > 0) {
   		          System.out.println("     --------- list of folder contents --------  ");
   		          processResults(folder);
   		          System.out.println("     ----------- end of list ------------------ ");
   	              }
   	              else 
                          System.out.println("  empty folder ");
   	                  break;
  	          default:
   	              System.out.println("  Item is not recognized ");
   	              break;
   	       } 
       }
    } 
  }
}
