//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


// sample connect/execute/disconnect to DES

/*
 * Filename       : TupdFldQryPropDES.java
 * Description    : Connect to DES Datastore.
 *                  Gets and displays specified folder properties
 *                  updates and redisplays specified folder properties
 *                  Gets and displays specified stored query properties
 *                  updates and redisplays specified stored query properties
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TupdFldQryPropDES class
public class TupdFldQryPropDES implements DKConstantDES
{


  // Main method
  public static void main(String argv[])
  {

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
             pw = new String(""), // pw
             connect_string = "";

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length != 4)
      {
       System.out.println("Usage: " );
       System.out.println("java TupdFldQryPropDES " +
        "tcp/ipAddress " +
        "userid " +
        "userpw " +
        "PORT=80;DESAPPID=appid;DESAPPPW=apppw[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>]");
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
       connect_string = argv[3];
      }

    DKDatastoreDES dsDES = null;
    try {
    dsDES = new DKDatastoreDES();
    System.out.println("connecting to datastore");
    dsDES.connect(libSrv,userid,pw,connect_string);
    System.out.println("datastore connected libSrv " + libSrv + " userid " +
                        userid );

    // Get folder properties
	DKNVPair[]prop = dsDES.getQueryFolderProperties("My Folder");
	if(prop != null){
		for(int m=0; m<prop.length; ++m){
		   System.out.println(prop[m].getName() + " -- "+ prop[m].getValue());
		   //update description
		   if(prop[m].getName().equals("_desc")){
			prop[m].setValue("Desc updated by TupdFldQryPropDES.java");
			System.out.println("_desc for folder is updated");
		   }
		}

	}
    // update folder properties
	dsDES.setQueryFolderProperties(prop);
    // Get folder properties
	prop = null;
	prop = dsDES.getQueryFolderProperties("My Folder");
	if(prop != null){
		System.out.println("***** Updated folder properties");
		for(int m=0; m<prop.length; ++m){
		   System.out.println(prop[m].getName() + " -- "+ prop[m].getValue());
		}

	}

	System.out.println("** Stored query properties");
	prop = dsDES.getStoredQueryProperties("My Folder", "Brian");
	if(prop != null){
		for(int m=0; m<prop.length; ++m){
		   System.out.println(prop[m].getName() + " -- "+ prop[m].getValue());
		   //update description
		   if(prop[m].getName().equals("_desc")){
			prop[m].setValue("Desc updated by TupdFldQryPropDES.java");
			System.out.println("_desc for stored query  is updated");
		   }
		}

	}
        // update stored query  properties
	dsDES.setStoredQueryProperties(prop);

	prop = null;
	prop = dsDES.getStoredQueryProperties("My Folder", "Brian");
	if(prop != null){
		System.out.println("****Updated stored query properties");
		for(int m=0; m<prop.length; ++m){
		   System.out.println(prop[m].getName() + " -- "+ prop[m].getValue());
		}

	}

    System.out.println("query results processed");
    dsDES.disconnect();
    System.out.println("datastore disconnected");
    dsDES.destroy();
    } // end try
    catch (DKException exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
  } // end main
}


