/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/******************************************************************************
 * Filename       : LoadSampleTSQBICDL.java
 *
 * Description    : This sample loader using java class library's API to
 * load sample data into the Digital Library with parts to be indexed by 
 * Text Search and QBIC.  You should run it with the provided input data file 
 * "sampleQBIC.dat". The physical files	for this loader are packaged in
 * "samples.jar".  You should use command "jar -xvf samples.jar" to unpack it.	
 * 
 * To run it as default, this loader assumes following are already defined: 
 * 
 *    Index Class Name - "DLSAMPLE" (defined with the key fields in step 4)
 *    Tex search server(search service name) - "TM"
 *    Text search index - "TMINDEX"
 *    QBIC DBName(database name) - "SAMPLEDB"
 *    QBIC Catalog - "SAMPLECAT"
 *    QBIC Host Alias of QBIC server - "qbicsrv"
 * 
 * Invoke Syntax:
 *
 *    java LoadSampleTSQBICDL <inputDataFile> <outputLog> <userid> <password> <libraryServer>
 *
 *    where:  
 *      <inputDataFile>: a input control file, see sampleQBIC.dat for example. 
 *      <outputLog>: the log file to store the process results, it can be used
 *                   for debug purpose, you can delete it later.  
 *      <userid>: the userid you use to logon the library server.
 *      <password>: the password to logon the library server.
 *      <libraryServer>: the library server name
 *
 * ### NOTE: to compile  ###
 *    NT:
 *      Edit the x:\cmbroot\cmbenv81.bat file to set the environment variables.
 *      for example:  set CMBROOT=e:\cmbroot
 *                    set DB2HOME=e:\sqllib
 *      under x:\cmbroot directory enter: cmbenv81   
 *
 *      under x:\cmbroot\samples\java\dl directory enter following to compile:
 *            javac LoadSampleTSQBICDL.java
 *
 *      The LoadSampleTSQBICDL.class will be created if compiled successfully.     
 *
 *    AIX:
 *      Make sure the CLASSPATH environment variable has the following values:
 *            /usr/lpp/cmb/lib/cmbcm81.jar
 *            /usr/lpp/cmb/lib/cmbdl81.jar
 *      under /usr/lpp/cmb/samples/java/dl enter following to compile:
 *            javac LoadSampleTSQBICDL.java 
 *
 *      The LoadSampleTSQBICDL.class will be created if compiled successfully.      
 *
 * ### NOTE: ###
 *    This loader also allow you to invoke with "indexClassName" as the input 
 *    parameter.  Please be aware that your index class's attribute names and
 *    input data file should match. For example you invoke as:    
 *
 *    java LoadSampleTSQBICDL sample.dat my.log frnadmin password LIBSRVRN MYINDEX
 *
 *    The index class name "DLSAMPLE" will be replaced with "MYINDEX".  
 ******************************************************************************
 * Following is the instruction to customize the loader to run your own data 
 ******************************************************************************
 * This loader can be modified to work with any index class defined with		
 * different attribute types(key fields).  Rules to follow:
 *
 *  1. The attribute name in the inputDataFile(ie. sampleQBIC.dat) should
 *     exactly match with the index class's attribute name(case sensitive). 
 *     for example: 
 *
 *     index class attribute names     in sampleQBIC.dat file
 *     ---------------------------   ----------------------
 *      DLSEARCH_Author              DLSEARCH_Author=Janet Hedges
 *      DLSEARCH_Publisher           DLSEARCH_Publisher=IBM
 *
 *     index class with your           your <inputDataFile> 
 *     defined attribute names         (ie. mine.dat)
 *     ---------------------------   ----------------------
 *      Department                   Department=U6M
 *      Manager                      Manager=John Smith
 *
 *  2. In your inputDataFile, the prefix for the parts to be added should be 
 *     "xdo", "smi", "html", "thmb" or "qbic".  The "xdo" is for a regular part,
 *     "smi" is for the part to be indexed by Text Search, "html" is for the 
 *     html part, "thmb" is for the thumbnail part and "qbic" is for the part
 *     to be indexed by the QBIC. 
 *
 *  3. The inputDataFile should use end= to represent the ending of an item.
 *
 *  4. Examples of index class's attributes and sampleQBIC.dat
 *
 *     index class attr. name     character length min - max
 *     ----------------------    ----------------------------
 *      DLSEARCH_Author            1 - 50
 *      DLSEARCH_Publisher         1 - 50
 *      DLSEARCH_DocType           1 - 32
 *      DLSEARCH_Date              1 - 50
 *      DLSEARCH_Title             1 - 100
 *      DLSEARCH_Subject           1 - 100
 *      DLSEARCH_Authors           1 - 100
 *      DLSEARCH_Editors           1 - 50
 *      DLSEARCH_Source            1 - 100
 *      DLSEARCH_Volume            1 - 20
 *      DLSEARCH_Issue             1 - 20
 *      DLSEARCH_Relation          1 - 64
 *
 *    (all above key field names are defined with key field type as
 *     "Variable character" and character type as "Extended alphanumeric", the
 *     character length min - max are shown as above)
 *
 *     the sampleQBIC.dat file: (shown as following)
 *    --------------------------------------------
 *   ########################################################
 *   DLSEARCH_Author=Anonymous
 *   DLSEARCH_Authors=
 *   DLSEARCH_Subject=Contents
 *   DLSEARCH_Editors=
 *   DLSEARCH_Publisher=IBM
 *   DLSEARCH_Date=1994
 *   DLSEARCH_Source=Software Quarterly
 *   DLSEARCH_Volume=1
 *   DLSEARCH_Issue=1
 *   DLSEARCH_Title=Contents of Software Quarterly Volume 1 Issue 1
 *   html=vol11\contents.htm
 *   thmb=images\321smt.gif
 *   xdo=images\sqcloud0.gif
 *   xdo=images\sqclouda.gif
 *   xdo=images\choice.gif
 *   qbic=images\321sm.gif
 *   xdo=images\sqhome.gif
 *   xdo=images\tellsq.gif
 *   xdo=images\getsq.gif
 *   xdo=images\software.gif
 *   xdo=vol11\contents.txt
 *   smi=vol11\contents.dat
 *   end=end
 *   ########################################################
 *
 *   Note: about the prefix - smi, qbic, html, thmb
 *     smi - for the part to be indexed by the Text Search, ie. contents.dat
 *     qbic - for the part to be indexed by QBIC, ie. 321sm.gif
 *     html - for the html part, ie. contents.htm
 *     thmb - for the thumbnail part, ie. 321smt.gif
 *
 *  5. Invoke example: (in NT system, with files in the following directories)
 *     x:\cmbroot\samples\java\dl\LoadSampleTSQBICDL.java -- this program
 *     x:\cmbroot\samples\java\dl\sampleQBIC.dat       -- inputDataFile
 *     x:\cmbroot\samples\java\dl\vol11\*.*        -- input text files are here
 *     x:\cmbroot\samples\java\dl\images\*.*       -- input image files are here
 *
 *     In x:\cmbroot\samples\java\dl] type as:
 *
 *     java LoadSampleTSQBICDL sampleQBIC.dat qbic.log frnadmin password LIBSRVRN 
 *
 *     (note: in AIX system path is /usr/lpp/cmb/samples/java/dl/  )
 *
 *  6. If physical files are in different directories then file path in
 *     sampleQBIC.dat should be modified.
 *
 *  7. This loader requires the DL indexClass("DLSAMPLE"), Text
 *     search server("TM"), Text search index("TMINDEX"), QBIC 
 *     database("SAMPLEDB"), QBIC catalog("SAMPLECAT") and QBIC server("qbicsrv")
 *     have been defined first.  The "DLSAMPLE" has attribute names defined as 4.,
 *     and all the field types are "variable character".
 *
 *     If you have different index class name, different
 *     text search server(search service name) or text search index,
 *     different database name, catalog name and host alias name for QBIC,
 *     please modify the following variables:
 *
 *     indexClassName = "DLSAMPLE",    //a DL index class name
 *     searchEngineTS = "SM",          //Text search engine name - SM 
 *     searchServerTS = "TM",          //Text search server name    
 *     searchIndexTS = "TMINDEX",      //Text search index name
 *     searchInfo = "ENU",             //ENU for english, FRA for french... 
 *     searchEngineQBIC = "QBIC",      //QBIC engine name - QBIC
 *     QBICDataBase = "SAMPLEDB",      //QBIC DataBase name
 *     QBICCatalog = "SAMPLECAT",      //QBIC catalog name
 *     searchServerQBIC = "qbicsrv";   //QBIC server name
 *
 *  8. If you want to use this loader to load data without part to be indexed	
 *     by Text Search, please comment out the lines indicated by the 
 *     |**** no text search by comment out following line ****| &  
 *     |**** end of comment out for no text search ***********|
 *     In this case, you don't need to define the TM search index name. 
 *
 *     Also, if you want to use this loader to load data without part to be 
 *     indexed by QBIC, please comment out the lines indicated by the 
 *     |**** no image search by comment out following line ****| &  
 *     |**** end of comment out for no image search ***********|
 *     In this case, you don't need to define the QBIC search index name. 
 * 
 *****************************************************************************/  
import java.util.*;
import java.io.*;
import java.lang.*;
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;

public class LoadSampleTSQBICDL
       implements DKConstantDL
{
  static String
      indexClassName = "DLSAMPLE",    //a DL index class name
      searchEngineTS = "SM",          //TextMiner engine name - SM 
      searchServerTS = "TM",          //TextMiner search server name    
      searchIndexTS = "TMINDEX",      //TextMiner search index name
      searchInfo = "ENU",             //ENU for english, FRA for french... 
      searchEngineQBIC = "QBIC",      //QBIC engine name - QBIC
      QBICDataBase = "SAMPLEDB",      //QBIC DataBase name
      QBICCatalog = "SAMPLECAT",      //QBIC catalog name
      searchServerQBIC = "qbicsrv";   //QBIC server name

  static String tocItemId = new String("");     //for DLSEARCH_relation attr. value
  static short docDataId = -1,
         itemDataId = -1,
         partsDataId = -1;

  final  static int  maxNo     = 50;
  static String attrName[]     = new  String[maxNo];
  static DKDatastoreDL dsDL;
  static BufferedWriter log;
  static short attrCount = 0;

  public static void main (String[] args)
  {
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    //sample loader always loading English data samples

    Locale origLocale = Locale.getDefault();
    boolean reSetL = false;
    if (!(Locale.getDefault().getCountry().equals("US")))
    { 
      reSetL = true;
      Locale.setDefault(new Locale("en", "US"));
    }

    try
    {
      boolean  smiSrv = false,            //flag for calling wakeUpService 
               qbicSrv = false;           //flag for calling invokeSearchEngine
 
      //*** for generated notepart file name
      String noteName = "notepart.txt";   
   
      int   smiPartId = 1,                //partId for TM search indexed part
            notePartId = 2,               //partId for notepart
            htmlPartId = 3,               //partId for html part 
            qbicPartId = 4,               //partId for QBIC search indexed part
            thmbPartId = 5,               //partId for thumbnail part
            partId = 10;                  //partId for regular xdo part

      boolean tocFlag = false,            // flag to show item is a toc item
              xdoFlag = false,            // flag to signal part is exist
              smiFlag = false,            // flag to signal TM search part is exist
              qbicFlag = false,           // flag to signal QBIC search part is exist 
              htmlFlag = false,           // flag to signal html part is exist 
              thmbFlag = false,           // flag to signal thumbnail part is exist  
              reset = false;              // reset tocItemId for new Toc        
      String  smiName = new String(""),   //filepathname of TM search part
              qbicName = new String(""),  //filepathname of QBIC search part
              htmlName = new String(""),  //filepathname of html part
              thmbName = new String("");  //filepathname of thumbnail part
      int     xdoFCount = 0;
      String  xdoPathNames[] = new  String[maxNo];
  
      //*** TextMiner search service name-search index name for API call      
      String  searchIndexTM = searchServerTS + "-" + searchIndexTS;

      //*** QBIC DataBase name-Catalog name-server Name for API call      
      String  searchIndexQBIC = QBICDataBase + "-" + 
                                QBICCatalog + "-" + searchServerQBIC;

      short dataId = 0, dataCount = 0;
      short i;
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (args.length < 5)
      {
       System.out.println("Invoke Syntax: " );
       System.out.println("  java LoadSampleTSQBICDL <inputDataFile> <outputLogFile> " +
        "<userid> <password> <libraryServer>" );
       return;
      }
      // ### get input parameters     
      File inputFile = new File(args[0]);
      File outputFile = new File(args[1]);
      String userid = args[2];
      String password = args[3];
      String libSrv = args[4];
      //----------------------------------------------------------------- 
      // support to run with user's own DL index class - at your own risk
      //----------------------------------------------------------------- 
      if (args.length > 5)
      {
       indexClassName = args[5];
      }   
      log   = new BufferedWriter(new FileWriter(outputFile));
      // *** for writing to log
      if (args.length == 5)  
      {    
       log.write("You enter: java TLoadSample " +
       args[0] + " " + args[1] + " " + args[2] + " " + args[3] + " " + args[4]);
      }
      else
      {
       log.write("You enter: java TLoadSample " + args[0] + " " +
       args[1] + " " + args[2] + " " + args[3] + " " + args[4] + " " + args[5]);
      }
      log.newLine();
 
      if (inputFile.exists() == false)
      {
       System.out.println("Input data file does not exist: " + args[0]);
       return;
      }

      //-------------------------------------------------------------- 
      // verify the TextMiner "TMINDEX" and QBIC dataBase & catalog 
      // are exist
      //-------------------------------------------------------------- 
      verifySetUp(userid, password);

      //-------------------------------------------------------------- 
      // connecting to data store
      //-------------------------------------------------------------- 
      dsDL = new DKDatastoreDL();
      System.out.println("Connecting datastore, please wait...");
      dsDL.connect(libSrv, userid, password,"");
      System.out.println("datastore connected " + libSrv);
      System.out.println("Loading " + args[0] + ",  please wait.... ");
      Integer opt = new Integer(DK_FALSE);
      dsDL.setOption(DK_DL_OPT_WAKEUPSRV, (Object)opt);

      DKDDO ddo = createDDO(indexClassName); 

      //-------------------------------------------------------------- 
      // start to read input data file
      //-------------------------------------------------------------- 
      String line, name, value;
      BufferedReader in
          = new BufferedReader(new FileReader(inputFile));
      DKParts parts = null;
      DKPid pid;
      while ((line  = in.readLine()) != null)
      {
       int n;
       name = value = "";
       line = line.trim();
       if (line.startsWith("#"))
                continue;
       n = line.indexOf("=");
       if (n == -1)
          continue;  
       name = (line.substring(0,n)).trim();
       value = (line.substring(n + 1)).trim();
       if (value.equals(""))
          continue;

       //-------------------------------------
       // read and store the attribute valude
       //-------------------------------------
       boolean find = false;
       for (i = 0; i < attrCount; i++)
       {
         if ((attrName[i].toUpperCase()).equals(
               name.toUpperCase()))
	     {
           dataId = ddo.dataId(attrName[i]);
           ddo.setData(dataId, value);
           find = true;
           break;
	     }     
       }
       if (find == true)
          continue;	    
       if (!(tocItemId.equals(""))) 
         ddo.setData(itemDataId, tocItemId);
         
       //-------------------------------------
       // read and store the xdo FilePathName
       //-------------------------------------
       if (name.toLowerCase().equals("xdo")) 
       {
         xdoFlag = true;
         xdoPathNames[xdoFCount] = value;
         xdoFCount = xdoFCount + 1;
       }
       else if (name.toLowerCase().equals("smi")) 
       {
         smiName = value;
         smiFlag = true;
         smiSrv = true; 
       }
       else if (name.toLowerCase().equals("qbic")) 
       {
         qbicName = value;
         qbicFlag = true;
         qbicSrv = true;
       }
       else if (name.toLowerCase().equals("html")) 
       {
         htmlName = value;
         htmlFlag = true;
       }
       else if (name.toLowerCase().equals("thmb"))
       {
         thmbName = value;
         thmbFlag = true;
       }
       else
       if (name.toLowerCase().equals("end"))
       {
         //-------------------------------------
         // start to create DDO for an item
         //-------------------------------------
         if (value.toLowerCase().equals("toc"))
             tocFlag = true;
         if (value.toLowerCase().equals("endtoc"))
             reset = true;
         System.out.println("start to create a DDO...");

         //*** for writing to log 
         log.write(" dataCount = " + ddo.dataCount() );
         log.newLine();

         parts = new DKParts();
	     int CClass;
         DKBlobDL  xdo;
         if (smiFlag)
	     {       
           xdo = createXDO(smiName, DK_DL_CC_ASCII, smiPartId);
/**** no text search by comment out following line ****/
           DKSearchEngineInfoDL aSrchEx = new DKSearchEngineInfoDL();
           aSrchEx.setSearchEngine(searchEngineTS);
           aSrchEx.setSearchIndex(searchIndexTM);
           aSrchEx.setSearchInfo(searchInfo);
           xdo.setExtension("DKSearchEngineInfoDL", (dkExtension)aSrchEx);       
/**** no text search by comment out following line ****/          
           parts.addElement(xdo);
         }
         if (htmlFlag)
	     {      
           xdo = createXDO(htmlName, DK_DL_CC_HTML, htmlPartId);         
           parts.addElement(xdo);
         }
         if (qbicFlag)
	     {  
           CClass = findContentClass(qbicName);      
           xdo = createXDO(qbicName, CClass, qbicPartId);
/**** no image search by comment out following line ****/ 
           DKSearchEngineInfoDL aSrchEx2 = new DKSearchEngineInfoDL();
           aSrchEx2.setSearchEngine(searchEngineQBIC);
           aSrchEx2.setSearchIndex(searchIndexQBIC);
           aSrchEx2.setSearchInfo(searchInfo);
           xdo.setExtension("DKSearchEngineInfoDL", (dkExtension)aSrchEx2);            
/**** no image search by comment out following line ****/          
           parts.addElement(xdo);
         }
         if (thmbFlag)
	     {  
           CClass = findContentClass(thmbName);      
           xdo = createXDO(thmbName, CClass, thmbPartId);         
           parts.addElement(xdo);
         }
         if (xdoFlag)
	     {
           partId = 10;
           for (i = 0; i < xdoFCount; i++)
	       {
             CClass = findContentClass(xdoPathNames[i]); 
	         parts.addElement(createXDO(xdoPathNames[i], CClass, partId)); 
             partId = partId + 1;
           }     
         }           
         //-------------------------------------
         // create the notepart file
         //-------------------------------------
         BufferedWriter note = new BufferedWriter(
                 new FileWriter(noteName));
         String tmp;
         if (smiFlag)
	     {
           tmp = prepareLine(smiName, smiPartId);
           note.write(tmp, 0, tmp.length());
           note.newLine();
           log.write(tmp);
           log.newLine();
	     }
         if (htmlFlag)
	     {
           tmp = prepareLine(htmlName, htmlPartId);
           note.write(tmp, 0, tmp.length());
           note.newLine();
           log.write(tmp);
           log.newLine();
	     }
         if (qbicFlag)
	     {
           tmp = prepareLine(qbicName, qbicPartId);
           note.write(tmp, 0, tmp.length());
           note.newLine();
           log.write(tmp);
           log.newLine();
	     }
         if (thmbFlag)
	     {
           tmp = prepareLine(thmbName, thmbPartId);
           note.write(tmp, 0, tmp.length());
           note.newLine();
           log.write(tmp);
           log.newLine();
	     }
         if (xdoFlag)
	     {
           partId = 10;
           for (i = 0; i < xdoFCount; i++)
	       {
             tmp = prepareLine(xdoPathNames[i], partId);
             note.write(tmp, 0, tmp.length());
             note.newLine();
             log.write(tmp);
             log.newLine();
             partId = partId + 1;
	       }
         }
         note.close();

	     parts.addElement(createXDO(noteName, DK_DL_CC_ASCII, notePartId)); 

         //*** add ddo
         ddo.setData(partsDataId, parts);
         ddo.add(); 
         log.write("Add success with itemId = " + ddo.getPidObject().getPrimaryId());
         log.newLine();
         log.write("===================================================");
         log.newLine();
         //*** if toc, save created itemId for DLSEARCH_relation value of latter items
         if (tocFlag)
	     {
           tocItemId = ddo.getPidObject().getPrimaryId();
           tocFlag = false;
	     }
         if (reset)
	     {
           tocItemId = "";
           reset = false;
	     }
         log.write("DLSEARCH_relation = " + tocItemId);
         log.newLine();
         parts.removeAllElements();
         //*** reset the flags
         smiFlag = xdoFlag = qbicFlag = htmlFlag = thmbFlag = false;
         xdoFCount = 0; 
         //*** reset attrValue
         for (dataId = 1; dataId <= ddo.dataCount(); dataId++)
         {
           if (dataId != docDataId) 
               ddo.setNull(dataId);
         }    
         //*** reset ddo itemid to blank
         pid = ddo.getPid();
         pid.setPrimaryId("");
         ddo.setPidObject(pid);
 
       } //if is "end"

      }  //end of read: while
      in.close();
      if (smiSrv == true)
      {
        //*** TextMiner search service name-search index name for API call      
/**** no text search by comment out following line ****/
        System.out.println("about to call invokeSearchEngine for TM..... ");
        log.write("about to call invokeSearchEngine(" + searchEngineTS + "," + searchIndexTM + ")");
        log.newLine();
        dsDL.invokeSearchEngine(searchEngineTS, searchIndexTM);
        log.write("invokeSearchEngine(..) done. ");
        log.newLine();
        System.out.println("invokeSearchEngine(..) done. ");
/**** end of comment out for no text search ***********/ 
      }
      if (qbicSrv == true)
      {
/**** no image search by comment out following line ****/
        opt = new Integer(DK_DL_SRCHINDEX_ONLINE);
        dsDL.setOption(DK_DL_OPT_SEARCHINDEX, (Object)opt);
        System.out.println("about to call invokeSearchEngine for QBIC..... ");
        log.write("about to call invokeSearchEngine(" + searchEngineQBIC + "," + searchIndexQBIC + ")");
        log.newLine();
        dsDL.invokeSearchEngine(searchEngineQBIC, searchIndexQBIC);
        log.write("invokeSearchEngine() done. ");
        log.newLine();
        System.out.println("invokeSearchEngine() done. ");
/**** end of comment out for no image search ***********/
      }
      log.close();
/**** no text search by comment out following line ****/       
      // do actuall TS indexing of the queue
      DKIndexFuncStatusTS pIndexFuncStatus = null;
      DKDatastoreTS dsTS = new DKDatastoreTS();
      System.out.println("connecting to TS datastore"); 
      dsTS.connect(searchServerTS,"","","");
      System.out.println("datastore connected " + searchServerTS); 
      System.out.println("start updating index " + searchIndexTS); 
      dsTS.startUpdateIndex(searchIndexTS);
      System.out.println("update index completed"); 
      System.out.println("**** get index status ****"); 
      pIndexFuncStatus = dsTS.getIndexFunctionStatus(searchIndexTS);
      System.out.println("****** index status ******"); 
      System.out.println("*isCompleted " + pIndexFuncStatus.isCompleted()); 
      System.out.println("*getEnabledId " + pIndexFuncStatus.getEnabledId()); 
      System.out.println("*getReasonCode " + pIndexFuncStatus.getReasonCode()); 
      System.out.println("*getFuncStopped " + pIndexFuncStatus.getFunctionStopped()); 
      System.out.println("*getStartedLast " + pIndexFuncStatus.getStartedLast()); 
      System.out.println("*getEndedLast " + pIndexFuncStatus.getEndedLast()); 
      System.out.println("*getStartedExecution " + pIndexFuncStatus.getStartedExecution()); 
      System.out.println("*getScheduledDocs " + pIndexFuncStatus.getScheduledDocs());
      System.out.println("*getDocsInPrimaryIndex " + pIndexFuncStatus.getDocsInPrimaryIndex()); 
      System.out.println("*getDocsInSecondaryIndex " + pIndexFuncStatus.getDocsInSecondaryIndex()); 
      System.out.println("*getDocMessages " + pIndexFuncStatus.getDocMessages()); 
      if (pIndexFuncStatus.isCompleted() == true)
      {
       System.out.println("***********indexing completed****************"); 
      }
      if (pIndexFuncStatus.getReasonCode() != 0)
      {
       dsTS.setIndexFunctionStatus(searchIndexTS,DK_TSINDEXACTID_RESET);
      }
      dsTS.disconnect();
      System.out.println("datastore TS disconnected"); 
/**** end of comment out for no text search ***********/ 

      System.out.println("Finish loading " + args[0] );
      dsDL.disconnect();
      System.out.println("datastore disconnected. Done....");

    } // end of try
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (IOException exc)
    {
     System.out.println("Exception name " );
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }  
    if (reSetL) Locale.setDefault(origLocale);  //re-set back to original Locale
 
  }
  //************************************************************
  // createDDO - create a ddo to be reuse for each document
  //************************************************************
  public static DKDDO  createDDO(String indexClassName)
            throws DKException, IOException, Exception
  {
     // setup ddo
     DKPid pid = new DKPid();
     pid.setObjectType(indexClassName);        
     DKDDO ddo = new DKDDO(dsDL, pid);
     ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE,
                         new Short(DK_CM_DOCUMENT));

     // get attribute collection
     DKSequentialCollection  collection;
     dkIterator iterator;
     DKAttrDefDL attrDef; 
     collection = (DKSequentialCollection)
                   dsDL.listEntityAttrs(indexClassName);
     iterator = collection.createIterator();
     int i = 0;
     short dataId = 0;
     String html = "html";
     //-------------------------------------------------------------- 
     // obtains data for setting up attrbuteList for ddo
     //-------------------------------------------------------------- 
     short sAttrType = 0;
     while (iterator.more())
     {
        attrDef = (DKAttrDefDL)iterator.next();
        attrName[i] = attrDef.getName();
        dataId = ddo.addData(attrName[i]);
        ddo.addDataProperty(dataId, DK_CM_PROPERTY_ATTRIBUTE_ID,
                             new Short(attrDef.getId()));
        sAttrType        = attrDef.getType();
        sAttrType        = translateAttrTypeToDataItemType(sAttrType);
        ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE,
                             new Short(sAttrType));
        ddo.addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE,
                             new Boolean(attrDef.isNullable()));
        ddo.addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT,
                             new Integer(DK_CM_ATTRIBUTE_READWRITE));
        if (attrName[i].equals("DLSEARCH_DocType"))
        { 
           docDataId = dataId;      
           ddo.setData(dataId, html);
        }
        else if (attrName[i].equals("DLSEARCH_Relation"))
        {
           itemDataId = dataId;
           ddo.setNull(dataId);
        }
        else 
           ddo.setNull(dataId);
        i++;
        attrCount++;
     }
     partsDataId = ddo.addData(DKPARTS);  
     ddo.addDataProperty(partsDataId, DK_CM_PROPERTY_TYPE,
              new Short(DK_CM_DATAITEM_TYPE_COLLECTION_XDO));   
     return ddo;
  }

  //**********************************************************
  // translateAttrTypeToDataItemType - change attribute type to
  //   ddo dataitem type 
  //**********************************************************
  public static short translateAttrTypeToDataItemType(short attrType)
  {
   short rAttrType = DK_CM_DATAITEM_TYPE_UNDEFINED;
   switch(attrType)
   {
    case DK_CM_VSTRING:
    case DK_CM_FSTRING:
    case DK_CM_DECIMAL:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_STRING;
    }
    break;
    case DK_CM_SHORT:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_SHORT;
    }
    break;
    case DK_CM_LONG:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_LONG;
    }
    break;
    case DK_CM_DATE:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_DATE;
    }
    break;
    case DK_CM_TIME:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIME;
    }
    break;
    case DK_CM_TIMESTAMP:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIMESTAMP;
    }
    break;
   }
   return rAttrType;
  }

  //**********************************************************
  // findContentClass - find content class from file extension
  //**********************************************************
  public static int findContentClass(String fullFileName)
            throws DKException, IOException
  {
     String suffix = "";
     int m, CClass;
     m = fullFileName.lastIndexOf(".");
     if (m != -1) suffix = fullFileName.substring(m + 1);
     if (suffix != "")
     {
       if ( (suffix.toLowerCase()).equals("gif") )
             CClass = DK_DL_CC_GIF;
       else if ( ((suffix.toLowerCase()).equals("html")) ||
                  ((suffix.toLowerCase()).equals("htm")) )
             CClass = DK_DL_CC_HTML;
       else if ( (suffix.toLowerCase()).equals("jpg") )
             CClass = DK_DL_CC_JPG;
       else if ( (suffix.toLowerCase()).equals("avi") )
             CClass = DK_DL_CC_AVI;
       else if ( (suffix.toLowerCase()).equals("wav") )
             CClass = DK_DL_CC_WAV;
       else if ( (suffix.toLowerCase()).equals("mid") )
             CClass = DK_DL_CC_MIDI;
       else if ( (suffix.toLowerCase()).equals("mov") )
            CClass = DK_DL_CC_MOV;
       else if ( (suffix.toLowerCase()).equals("wpd") )
             CClass = DK_DL_CC_WP;
       else if ( (suffix.toLowerCase()).equals("wpg") )
            CClass = DK_DL_CC_WP;
       else if ( (suffix.toLowerCase()).equals("wks") )
             CClass = DK_DL_CC_WKS;
       else if ( (suffix.toLowerCase()).equals("bmp") )
             CClass = DK_DL_CC_WINV3_BMP;
       else if ( (suffix.toLowerCase()).equals("mpg") )
             CClass = DK_DL_CC_MPEG;
       else CClass = DK_DL_CC_ASCII;   // Add more ContentClasses here!!!       
     }
     else
        CClass = DK_DL_CC_ASCII; 
     return CClass;
  }

  //*****************************************************
  // createXDO - create an xdo
  //*****************************************************
  public static DKBlobDL createXDO(String file, int CClass, int partId)
            throws DKException, IOException, Exception
  {  
     DKBlobDL xdo = new DKBlobDL(dsDL);
     xdo.setContentClass(CClass);
     if ((CClass == DK_DL_CC_ASCII) && (partId == 2))
         xdo.setAffiliatedType(DK_DL_NOTE); 
     else  
         xdo.setAffiliatedType(DK_DL_BASE);
     xdo.setContentFromClientFile(file);
     DKPidXDODL pidx = new DKPidXDODL();
     pidx.setPartId(partId);
     xdo.setPidObject(pidx);   
     log.write("creat XDO part: " + file  +
                  " : " + partId);
     log.newLine();
     return xdo;
  }
  //**********************************************************
  // prepareLine - prepare a line record for the notepart file
  //**********************************************************
  public static String  prepareLine(String fullFileName, int partId)
  {
     String lineStr;
     int k, m;
     m = fullFileName.lastIndexOf("\\");
     k = fullFileName.lastIndexOf("/");
     if ((k == -1) && (m == -1))
     { 
        lineStr = fullFileName + ":" + String.valueOf(partId);                   
     }
     else if (k == -1)
        lineStr = (fullFileName.substring(m + 1)) + ":"
              + String.valueOf(partId);
     else 
        lineStr = (fullFileName.substring(k + 1)) + ":" 
              + String.valueOf(partId);
     return lineStr;
  }
  //**********************************************************
  // verifySetUp - verify the TextMiner index "TMINDEX" and
  // the QBIC Database and QBIC catalog are defined
  //**********************************************************
  public static void  verifySetUp(String userId, String passWord)
            throws DKException, IOException, Exception
  {
     System.out.println("verifying: " + searchIndexTS );
     DKDatastoreTS dsTS = new DKDatastoreTS();
     DKIndexInfoTS pIndxInfo = null;
     System.out.println("connecting to datastoreTS... " );
     dsTS.connect(searchServerTS, "", ' '); 
     System.out.println("datastoreTS connected with server: " + searchServerTS );
     pIndxInfo = dsTS.getIndexInformation(searchIndexTS);
     System.out.println("Index Name = " + pIndxInfo.getIndexName());
     dsTS.disconnect();
     System.out.println("datastoreTS disconnected." );
     System.out.println("verifying QBIC server/database/catalog: " +
            searchServerQBIC + "/" + QBICDataBase + "/" + QBICCatalog);
     DKDatastoreQBIC dsQBIC = new DKDatastoreQBIC();
     System.out.println("connecting to datastoreQBIC... " );
     dsQBIC.connect(searchServerQBIC, userId, passWord, "");
     System.out.println("datastoreQBIC connected. " );
     System.out.println("open catalog " + QBICCatalog );
     dsQBIC.openCatalog(QBICDataBase, QBICCatalog);
     System.out.println("catalog opened successfully. " );
     dsQBIC.closeCatalog();
     dsQBIC.disconnect();
     System.out.println("datastoreQBIC disconnected. " );
     return;
  }

}


