/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/


/*
 * Filename       : TADLstFed.java
 * Description    : test add, delete, list search template for FedDemo
 *                : Need to run the following program first
 *                 java TADLServersFed <cmbdb> <userid> <pw> add
 *                 java TADLsmFed      <cmbdb> <userid> <pw> add
 *
 *
 */

import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

public class TADLstFed implements DKConstantFed 
{

   // Main method
   public static void main(String argv[]){

      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
        
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 4) {
         System.out.println("Usage Syntax: ");
         System.out.println("\t java TADLstFed <cmbdb> <userid> <pw> <add/del/list>" );
         return;
      }
      
      String libSrv = argv[0];
      String userid = argv[1];
      String pw     = argv[2];
      String cmd    = argv[3];
        
      try {
         // create federated datastore
         System.out.println(" *** connecting to datastore : " + libSrv);
         DKDatastoreFed ds = new DKDatastoreFed();
         ds.connect(libSrv, userid, pw, "");
         System.out.println(" *** datastore connected *** "); 

         // get the Def
         dkDatastoreDef  defFed = ds.datastoreDef();
          
         // set up search criteria memory variable.
         short[] ops = { DKOpCode.EQ, DKOpCode.NEQ, DKOpCode.LIKE, DKOpCode.NOTLIKE, DKOpCode.IN, DKOpCode.NOTIN };
 
         //===================================================================================================
         // Defines search criterias for FedDemo
         //===================================================================================================
            
         // Create search criteria for FedTitle
         DKSearchCriteria scTitle = new DKSearchCriteria("FedDemo", "FedTitle");
                          scTitle.setName("Title");
                          scTitle.setDisplayName("Document Title");
                          scTitle.setDefaultOperator(DKOpCode.LIKE);
                          scTitle.setValidOperators(ops);
                          scTitle.setDisplayWidth((short)40);
                          scTitle.setQueryable(true);
                          scTitle.setDisplayable(true);
 
         // Create search criteria for FedUserID
         DKSearchCriteria scUserID = new DKSearchCriteria("FedDemo", "FedUserId");
                          scUserID.setName("UserID");
                          scUserID.setDisplayName("Document UserID");
                          scUserID.setDefaultOperator(DKOpCode.LIKE);
                          scUserID.setValidOperators(ops);
                          scUserID.setDisplayWidth((short)40);
                          scUserID.setQueryable(true);
                          scUserID.setDisplayable(true);

         // Create search criteria for FedDate
         DKSearchCriteria scDate = new DKSearchCriteria("FedDemo", "FedDate");
                          scDate.setName("Date");
                          scDate.setDisplayName("Document Date");
                          scDate.setDefaultOperator(DKOpCode.LIKE);
                          scDate.setValidOperators(ops);
                          scDate.setDisplayWidth((short)40);
                          scDate.setQueryable(true);
                          scDate.setDisplayable(true);

         //===================================================================================================
         // Defines search templates for FedDemo
         //===================================================================================================
         
         // Create search template: FedUserId AND FedTitle AND FedDate
         DKSearchTemplateFed stANDS = new DKSearchTemplateFed(ds);
                             stANDS.setName("FedDemoST_ANDS");
                             stANDS.setDescription("All ANDS search template for FedDemo.");
                             stANDS.setEntityName("FedDemo");
                             stANDS.setParametricOp(DK_CM_OP_ALL_ANDS);
                                       
          try {
              if (cmd.equalsIgnoreCase("add")) {
                  stANDS.add();     // Add search template to persistent datastore
                  stANDS.addUser(ds.userName(), (short) DK_FED_ST_ACCESS_ADMIN); // Add users to the template
                  stANDS.add(scUserID);
                  stANDS.add(scTitle);
                  stANDS.add(scDate);
                  System.out.println("Successfully add search template with criteria FedUserID AND FedTitle AND FedDate");
              }
              else if (cmd.equalsIgnoreCase("del")) {
                  stANDS.retrieve();
                  stANDS.del();
                  System.out.println("Successfully delete search template with criteria FedUserID AND FedTitle AND FedDate");
              }
          }
          catch (Exception e) {
              e.printStackTrace();
          }
              
         // Create search template: FedUserId OR FedTitle OR FedDate
         DKSearchTemplateFed stORS = new DKSearchTemplateFed(ds);
                             stORS.setName("FedDemoST_ORS");
                             stORS.setDescription("All ORS search template for FedDemo.");
                             stORS.setEntityName("FedDemo");
                             stORS.setParametricOp(DK_CM_OP_ALL_ORS);
                                       
          try {
              if (cmd.equalsIgnoreCase("add")) {
                  stORS.add();     // Add search template to persistent datastore
                  stORS.addUser(ds.userName(), (short) DK_FED_ST_ACCESS_ADMIN); // Add users to the template
                  stORS.add(scUserID);                  
                  stORS.add(scTitle);
                  stORS.add(scDate);
                  System.out.println("Successfully add search template with criteria FedUserID OR FedTitle OR FedDate");
              }
              else if (cmd.equalsIgnoreCase("del")) {
                  stORS.retrieve();
                  stORS.del();
                  System.out.println("Successfully delete search template with criteria FedUserID OR FedTitle OR FedDate");
              }
          }
          catch (Exception e) {
              e.printStackTrace();
          }


         //===================================================================================================
         // Print available search templates for FedDemo
         //===================================================================================================

         dkCollection coll = ds.listSearchTemplates();
         dkIterator iter = null;
            
         if (coll != null) {
            iter = coll.createIterator();
            System.out.println("\n-- list Search template -- ");
            DKSearchTemplateFed stObj;

            while (iter.more()) {
               stObj = (DKSearchTemplateFed) iter.next();
               System.out.println("\n *** Search Template Name : " + stObj.getName());
               System.out.println("    Description: " + stObj.getDescription());
               System.out.println("    Folder Name: " + stObj.getFolderName());
               System.out.println("    Entity Name: " + stObj.getEntityName());
               System.out.print  (" Parametric OP : ");
               
               if (stObj.getParametricOp() == DK_CM_OP_ALL_ANDS)
                  System.out.println("DK_CM_OP_ALL_ANDS");
               else if (stObj.getParametricOp() == DK_CM_OP_ALL_ORS)
                  System.out.println("DK_CM_OP_ALL_ORS");
               
               // LIst criterias in search template                        
               dkCollection scColl = stObj.listCriteria();
               dkIterator scIter = scColl.createIterator();
               while (scIter.more()) {
                  DKSearchCriteria scObj = (DKSearchCriteria) scIter.next();
                  System.out.println("     ++ Criteria Name: " + scObj.getName());
                  System.out.println("         Display Name: " + scObj.getDisplayName());
                  System.out.println("          Entity Name: " + scObj.getEntityName());
                  System.out.println("            Attr Name: " + scObj.getAttrName());
                  System.out.println("           Default Op: " + scObj.getDefaultOperator());
                  System.out.println("        Display Width: " + scObj.getDisplayWidth());
                  System.out.println("            Queryable: " + scObj.isQueryable());
                  System.out.println("          Displayable: " + scObj.isDisplayable());
                  System.out.println(" ");
               }
            }
         }
         else 
            System.out.println("No Search template defines");
         
         ds.disconnect();
         System.out.println("Disconnect. Done.");
      }    
      catch (DKException exc) {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc) {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }

   }

}

