/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/* -------------------------------------------------------------------------
 * Class Name  = TLogUtils.java
 * Description = Test utility methods used in FVT (This program is used by
 *               TdkDatastore.java and TQSetup.java)
 * 
 * Author      = Chandra S Potula
 * Create Date = 06/25/2000
 * 
 * Change History
 * -------------------------------------------------------------------------
 */

import com.ibm.mm.sdk.common.*;

import java.io.*;
import java.lang.*;
import java.util.*;

public class TLogUtils implements DKConstant {
	public  static final String STR_PROMPT            = "==>";
	public  static final String STR_COMMAND_DELIMITER = ", \t().=";
	public  static final String STR_TAB               = "    ";
	public  static final String STR_INDENT            = "    ";
	public  static final String STR_ERROR             = "*** error *** ";
	private static final String OUTFILE_EXT           = ".log";
	private static final String ERRFILE_EXT           = ".err";
	private static final String DEF_INFILE            = "TestIn.txt";
	private static final String BLANK_STR             = "          ";

	private String              _strCurIndent         = "";

	private String              _strInFile            = "";
	private String              _strOutFile           = "";

	private static PrintWriter  _pwStdout             = null;
	private PrintWriter         _out                  = null;
	private PrintWriter         _err                  = null;
	private boolean             _fDualOut             = true;
	private boolean             _fAppend              = false;
	private boolean             _fEchoCommand         = true;
	private boolean             _fIgnoreError         = false;

	// Error information
	public  int                 nErrNumber            = 0;
	public  String              strCodeBase           = "";
	public  String              strErrDesc            = "";
	public  String              strErrLoc             = "";
	public  String              strErrInput           = "";

	// we store DKNVPair objects in this for VariableName = _stringValue_
	private   static Vector     _vectVars             = null;
	protected static Vector     _vectMethods          = null;
	private static boolean      _isHelpInfoAdded_TLogUtils;

	static {
		try {
			_isHelpInfoAdded_TLogUtils = false;
			_vectVars    = new Vector();
			_vectMethods = new Vector();
			_pwStdout    = new PrintWriter(System.out, true);
		}
		catch(Exception exp) {
		}
	}

public TLogUtils() {
	setWriter(_pwStdout);
	initVectVars();
}
public TLogUtils(String[] args) {
	this();
	processCommandLineArgs(args);
}
public TLogUtils(TLogUtils aParent) {
	this();

	if(aParent != null) {
		setWriter(aParent.getWriter());
		setIndentTab(aParent.getIndentTab());
	}
}
public static String getItemName(short shItemType) {
	String lName = "Unknown";

	switch (shItemType) {
		case DK_DOCUMENT :
			lName = "Document";
			break;
		case DK_FOLDER :
			lName = "Folder";
			break;
		case DK_WORKBASKET :
			lName = "Workbasket";
			break;
		case DK_WORKFLOW :
			lName = "Workflow";
			break;
		default:
			lName = "Unknown";
			break;
	}
	return lName;
}
public void logmsgThrowable(Throwable aTh) {
	if(!isIgnoreErrorOn()) {
		// print diagnostic information to err file
		logErrorInfo();
		logerr("Got following Exception...");

		if(aTh instanceof DKException) {
			DKException aDKExp = (DKException)aTh;
	
			logmsg(STR_ERROR + "DKException occurred");
			logmsgDual("    Name        : " + aDKExp.name().trim());
			logmsgDual("    Message     : " + aDKExp.getMessage().trim());
			logmsgDual("    Error ID    : " + aDKExp.getErrorId());
			logmsgDual("    Error code  : " + aDKExp.errorCode());
			logmsgDual("    Error state : " + aDKExp.errorState());
		}
		else {
			logmsg(STR_ERROR + "Exception occurred");
		}
	
		if(getWriter() != _pwStdout) {
			aTh.printStackTrace(_pwStdout);
		}
		if(_err != null) {
			aTh.printStackTrace(_err);
		}
		aTh.printStackTrace(getWriter());
	}
}
public void logErrorInfo() {
	logerr("");
	logerr("-------------------------------------------------------------------------");
	logerr("Test Case   : " + _strInFile);
	logerr("Error #     : " + ++nErrNumber);
	logerr("Test Date   : " + (new Date()).toString());
	logerr("Code Base   : " + strCodeBase);
	logerr("Description : " + strErrDesc);
	logerr("Where       : " + strErrLoc);
	logerr("Input       : " + strErrInput);
}
public void resetErrorInfo() {
	strErrDesc  = "";
	strErrLoc   = "";
	strErrInput = "";
}
public final void logerr(String aErr) {
	if(_err != null) {
		_err.println(aErr);
	}
}
public final void logmsgDual(String aMsg) {
	logmsg(aMsg);
	logerr(aMsg);
}
public void logmsgDKDDO(DKDDO aDKDDO) {
	if(aDKDDO == null) {
		logmsg("DDOItemIs    : null");
	}
	else {
		String   lstrIndentTab = getIndentTab();
		try {
			DKDDO            lDKDDO        = aDKDDO;
			DKFolder         lDKFolder     = null;
			String           lstrDataName  = null;
			Object           lObj          = null;
			dkDataObjectBase lDOBase       = null;

			try {
				// DL has this property but not TS! don't know about QBIC
				short  shItemType  = ((Short)lDKDDO.getPropertyByName(DK_PROPERTY_ITEM_TYPE)).shortValue();
				String strItemName = getItemName(shItemType);
				logmsg("DDOItemIs    : " + strItemName);
			}
			catch(Exception e) {
			}

			int    lDDOCount   = lDKDDO.dataCount();
			logmsg("DDOPidString : " + lDKDDO.getPidObject().pidString());
			logmsg("DDOCount     : " + lDDOCount);

			this.changeIndentTab(true);

			for(short i = 1; i <= lDDOCount; i++) {
				lstrDataName = lDKDDO.getDataName(i);
				lObj         = lDKDDO.getData(i);

				logmsg(sizeStr(i) + "Name : " + sizeStr(lstrDataName, 20) + " Value : " + lObj);
				logmsgObject(lObj);
			}
			this.changeIndentTab(false);
		}
		catch (Exception exc) {
			logmsgThrowable(exc);
		}
		finally {
			setIndentTab(lstrIndentTab);
		}
	}
}
public void logmsgdkCollection(dkCollection adkCollection) {
	if(adkCollection == null) {
		logmsg("CollectionIs : null");
	}
	else {
		String   lstrIndentTab  = getIndentTab();

		try {
			int              lCount    = adkCollection.cardinality();
			dkIterator       lIter     = adkCollection.createIterator();
			DKDDO            lDKDDO    = null;
			Object           lObj      = null;
			dkDataObjectBase lDOBase   = null;
	
			if(     adkCollection instanceof DKFolder) {
				logmsg("CollectionIs : Folder");
				logmsg("Documents    : " + lCount);
			}
			else if(adkCollection instanceof DKParts) {
				logmsg("PartsCount   : " + lCount);
			}
			else {
				logmsg("ItemsCount   : " + lCount);
			}

			this.changeIndentTab(true);
			for(short i = 1; lIter.more(); i++) {
				lObj   = lIter.next();
				logmsg(sizeStr(i) + "Value : " + lObj);
				logmsgObject(lObj);
			}
			this.changeIndentTab(false);
		}
		catch (Exception exc) {
			logmsgThrowable(exc);
		}
		finally {
			setIndentTab(lstrIndentTab);
		}
	}
}
public void logmsgdkResultSetCursor(dkResultSetCursor aRSCursor) {
	if(aRSCursor == null) {
		logmsg("ItemsCount   : 0 (got null cursor)");
	}
	else {
		String   lstrIndentTab  = getIndentTab();

		try {
			logmsg("ItemsCount   : " + aRSCursor.cardinality());

			this.changeIndentTab(true);
			DKDDO lDKDDO = null;
			// *** do we need to check for isEnd()???
			while(aRSCursor.isValid() && !aRSCursor.isEnd()) {
				lDKDDO = aRSCursor.fetchNext();
				logmsg("ItemNumber   : " + aRSCursor.getPosition());
				logmsgDKDDO(lDKDDO);
			}
			this.changeIndentTab(false);
			aRSCursor.destroy();
		}
		catch (Exception exc) {
			logmsgThrowable(exc);
		}
		finally {
			setIndentTab(lstrIndentTab);
		}
	}
}
public void logmsgObject(Object anObject) {
	if(anObject == null) {
		logmsg("    Object is      : null");
	}
	else {
		Object lObj = anObject;
		try {
			if(lObj instanceof dkCollection) {
				logmsgdkCollection((dkCollection)lObj);
			}
			else if(lObj instanceof DKDDO) {
				logmsgDKDDO((DKDDO)lObj);
			}
			else if(lObj instanceof dkDataObjectBase) {
				dkDataObjectBase lDOBase = (dkDataObjectBase)lObj;
				if(lDOBase.protocol() == DK_XDO) {
					logmsg("     XDOPidString : " + ((dkXDO)lDOBase).getPidObject().pidString());
				}
			}
			else if(lObj instanceof dkResultSetCursor) {
				logmsgdkResultSetCursor((dkResultSetCursor)lObj);
			}
		}
		catch (Exception exc) {
			logmsgThrowable(exc);
		}
	}
}
public static final Number toNumber(String aNum) {
	Number rc = null;

	if(aNum != null) {
		try {
			rc   = Integer.valueOf(aNum.trim());
		}
		catch(Exception e) {
		}
	}
	return rc;
}
public void printCommandHelp() {
	logmsg("List of available commands:");
	printCommandHelp("");
}
public void printCommandHelp(String aCmdStr) {
	boolean     fFound  = false;
	_MethodInfo lMInfo;
	Enumeration lEnum;
	boolean     fCmdSpecified = (aCmdStr != null && aCmdStr.trim().length() > 0);
	String      lCmdStr       = (fCmdSpecified) ? aCmdStr.trim().toLowerCase() : "";

	logmsg("");

	if(fCmdSpecified) {
		for(lEnum = _vectMethods.elements(); lEnum.hasMoreElements();) {
			lMInfo = (_MethodInfo)lEnum.nextElement();
			if(lMInfo.strName.equalsIgnoreCase(lCmdStr)) {
				logmsg(lMInfo.strName);
				logmsg(lMInfo.strDesc);
	
				this.changeIndentTab(true);
	
				if(lMInfo.strSyntax.length() <= 0 &&
						lMInfo.strHelp.length() <= 0) {
					logmsg("usage: " + lMInfo.strName + "()");
				}
				else {
					if(lMInfo.strSyntax.length() > 0) {
						logmsg(lMInfo.strSyntax);
					}
					if(lMInfo.strHelp.length() > 0) {
						logmsg(lMInfo.strHelp);
					}
				}
	
				this.changeIndentTab(false);
	
				fFound = true;
			}
		}
	}
	if(!fFound) {
		for(lEnum = _vectMethods.elements(); lEnum.hasMoreElements();) {
			lMInfo = (_MethodInfo)lEnum.nextElement();
			if(!fCmdSpecified || (lMInfo.strName.toLowerCase().indexOf(lCmdStr) >= 0 ||
		   	lMInfo.strSearchWords.toLowerCase().indexOf(lCmdStr) >= 0)) {
				logmsg(sizeStr((lMInfo.strName), 25) + " " + lMInfo.strDesc);
				fFound = true;
			}
		}
	}

	if(!fFound) {
		logmsg("Related command not found.  Try \"help\" for list of all available commands");
	}
}
public void logmsgError(String aErrMsg) {
	if(!isIgnoreErrorOn()) {
		logmsg(STR_ERROR + aErrMsg);
		strErrDesc = aErrMsg;
		logErrorInfo();
	}
}
public void processCommand(String aCmd) {
	try {
		String args[] = getTokens(aCmd);
		if(args != null && args.length > 0) {
			strErrLoc =  aCmd;
			if(     args[0].equalsIgnoreCase("help")) {
				if(args.length >= 2) {
					printCommandHelp(args[1]);
				}
				else {
					printCommandHelp();
				}
			}
			else if(args[0].equalsIgnoreCase("mem")) {
				logmsgMemUsed();
			}
			else if(args[0].equalsIgnoreCase("set")) {
				// its set ~variableName = _StringValue_ form
				if((args[1].charAt(0) == '~') && (aCmd.indexOf('=') > 0)) {
					setVariableValue(args[1], getRHS(aCmd));
				}
				else {
					logmsg("Invalid syntax");
				}
			}
			else if(isVarAssignment(aCmd)) {
				// its a ~variableName = _StringValue_ form
				setVariableValue(args[0], getRHS(aCmd));
			}
			else if(args[0].equalsIgnoreCase("listVariables")) {
				listVariables();
			}
			else if(args[0].equalsIgnoreCase("echo")) {
				boolean fEnable = !(args.length >= 2 && args[1].equalsIgnoreCase("off"));
				enableEcho(fEnable);
			}
			else if(args[0].equalsIgnoreCase("ignoreError")) {
				boolean fEnable = !(args.length >= 2 && args[1].equalsIgnoreCase("off"));
				enableIgnoreError(fEnable);
			}
			else {
				logmsgError("Unknown command : " + args[0]);
			}
		}
	}
	catch(Exception exp) {
		logmsgThrowable(exp);
	}
}
public void enableEcho(boolean fEnable) {
	_fEchoCommand = fEnable;
}
public boolean isEchoOn() {
	return _fEchoCommand;
}
public void enableIgnoreError(boolean fEnable) {
	_fIgnoreError = fEnable;
}
public boolean isIgnoreErrorOn() {
	return _fIgnoreError;
}
public void listVariables() {
	DKNVPair lNVPair = null;
	String   lName   = null;
	String   lValue  = null;

	for(Enumeration lEnum = _vectVars.elements(); lEnum.hasMoreElements();) {
		lNVPair = (DKNVPair)lEnum.nextElement();
		lName  = lNVPair.getName();
		lValue = (String)lNVPair.getValue();
		logmsg( sizeStr(lName, 15) + " = " + lValue);
	}
}
public static Object getVariableValue(String aName) {
	Object rc = null;

	DKNVPair lNVPair = getValuePairFor(aName);
	rc = (lNVPair != null) ? lNVPair.getValue() : "";
	return rc;
}
public static void setVariableValue(String aName, Object aValue) {
	if(aName != null && aName.length() > 0) {
		DKNVPair lNVPair = getValuePairFor(aName);
		if(lNVPair == null) {
			lNVPair = new DKNVPair(aName, aValue);
			_vectVars.addElement(lNVPair);
		}
		else {
			lNVPair.setValue(aValue);
		}
	}
}
private static DKNVPair getValuePairFor(String aName) {
	DKNVPair    lNVPair = null;

	for(Enumeration lEnum = _vectVars.elements(); lNVPair == null && lEnum.hasMoreElements();) {
		lNVPair = (DKNVPair)lEnum.nextElement();
		if(!lNVPair.getName().equals(aName)) {
			lNVPair = null;
		}
	}
	return lNVPair;
}


public void readFromStdIn() throws IOException {
	BufferedReader lBReader = new BufferedReader(new InputStreamReader(System.in));
	String         lstrCmd  = null;

	logmsg("Please enter your command at the prompt...");
	for(;;) {
		try {
			logmsg(STR_PROMPT, false);
			lstrCmd = lBReader.readLine().trim();

			if(lstrCmd.equalsIgnoreCase("exit") ||
			   lstrCmd.equalsIgnoreCase("quit") ||
			   lstrCmd.equalsIgnoreCase("q")    ||
			   lstrCmd.equalsIgnoreCase("e")) {
				 break;
			}
			takeCommand(lstrCmd);
		}
		catch(Exception e) {
			break;
		}
	}
}
public void readInputFile(String aInFile) throws IOException {
	if(aInFile.equalsIgnoreCase("stdin")) {
		readFromStdIn();
	}
	else {
		try {
			String       strCmd   = "";

			_strInFile = DEF_INFILE;
	
			if(aInFile != null && !aInFile.equals("")) {
				_strInFile  = aInFile;
			}
			System.out.println("-->Input file  : " + _strInFile);
	
			FileReader     frIn  = new FileReader(_strInFile);
			BufferedReader brIn  = new BufferedReader(frIn);
		
			while((strCmd = brIn.readLine()) != null) {
				strCmd = strCmd.trim();
				takeCommand(strCmd);
			}
	
			brIn.close();
		}
		catch(IOException e) {
			strErrDesc = "Unable to open input file";
			throw e;
		}
	}
}
public final String printUsage() {
	StringBuffer sbHelp        = new StringBuffer();
	String       lstrClassName = getClass().getName();

	sbHelp.append(lstrClassName + ".java\n");
	sbHelp.append("usage: java " + lstrClassName + " [-options] [[Command], [Command]...]\n\n");
	sbHelp.append("where options include:\n");
	sbHelp.append("  -i{InputFile}     Read input from specified file\n");
	sbHelp.append("  -o{OutputFile}    write output to specified file and stdout\n");
	sbHelp.append("  -O{OutputFile}    same as -o but donot write to stdout\n");
	sbHelp.append("  -a{OutputFile}    same as -o but append to OutputFile\n");
	sbHelp.append("  -A{OutputFile}    same as -O but append to OutputFile\n");
	sbHelp.append("  -v{Version#}      Version/CodeBase information\n");
	sbHelp.append("  -h                Print this help\n");
	sbHelp.append("  -?                Print this help\n\n");
	sbHelp.append("  e.g. java " + lstrClassName + " -iTest1 -oTest1" + OUTFILE_EXT + "\n");
	sbHelp.append("       which will use Test1 as input file and Test1" + OUTFILE_EXT + " as out put file\n");

	String strMsg = sbHelp.toString();
	logmsg(strMsg);
	printCommandHelp();

	return(strMsg);
}
public void processCommandLineArgs(String[] args) {
	String  strCurArg            = "";
	boolean fCmdSpecifiedOnCmdln = false;

	try {
		for(int i=0; i<args.length; i++ ) {
			strCurArg     = args[i].trim();
			if(strCurArg.equals("")) continue;
			char chFirst  = strCurArg.charAt(0);

			if(chFirst == '-' || chFirst == '/') {
				char chSecond = Character.toUpperCase(strCurArg.charAt(1));
				String strArgValue = "";

				if(strCurArg.length() > 2) {
					strArgValue = strCurArg.substring(2);
				}
	
				if(chSecond == 'I' && !strArgValue.equals("")) {
					_strInFile  = strArgValue;
				}
				else if(chSecond == 'O' && !strArgValue.equals("")) {
					enableDualOut(strCurArg.charAt(1) == 'o');
					enableAppend(false);
					setOutFile(strArgValue);
				}
				else if(chSecond == 'A' && !strArgValue.equals("")) {
					enableDualOut(strCurArg.charAt(1) == 'a');
					enableAppend(true);
					setOutFile(strArgValue);
				}
				else if(chSecond == 'V' && !strArgValue.equals("")) {
					strCodeBase = stringValue(strArgValue);
				}
				else if(chSecond == 'H' ||
				        chSecond == '?') {
					if(strArgValue.length() > 0) {
						printCommandHelp(strArgValue);
					}
					else {
						printUsage();
					}
					return;
				}
			}
			else {
				fCmdSpecifiedOnCmdln = true;
				takeCommand(strCurArg);
			}
		}
	
		// if only infile is specified then set out file based on
		// infile name
		if(!_strInFile.equals("")) {
			if(!_strInFile.equalsIgnoreCase("stdin") && 
					(_strOutFile == null || _strOutFile.equals("") || 
					_strOutFile.equalsIgnoreCase("stdout"))) {
				setOutFile(_strInFile + OUTFILE_EXT);
			}
			readInputFile(_strInFile);
		}
		else if(!fCmdSpecifiedOnCmdln) {
			readFromStdIn();
		}
	}
	catch(Throwable e) {
		logmsgThrowable(e);
	}
}
private final void takeCommand(String aCmd) {
	if(aCmd != null && isCommand(aCmd)) {
		String lstrCmd = aCmd;

		char chPrev          = ' ';
		char chCur           = ' ';
		boolean fStrStart    = false;
		boolean fchPrevAdded = false;
		boolean fchAdded     = false;

		// truncate if there is a C++ type of comment in the line
		for(short nInd=0; nInd<aCmd.length(); nInd++) {
			chCur = aCmd.charAt(nInd);
			if((chCur == '\'' || chCur == '"')) {
				fStrStart = !fStrStart;
			}
			else if(chCur == '/' && chPrev == '/') {
				lstrCmd = aCmd.substring(0, nInd - 1).trim();
				break;
			}
			else {
				chPrev = ' ';
				if(chCur != '/' || chPrev != '\\') {
					chPrev = chCur;
				}
			}
		}
		if(isEchoOn()) {
			logmsg("-->Processing  : " + lstrCmd);
		}
		processCommand(lstrCmd);
	}
}
private void setOutFile(String aFile) {
	if(_strOutFile == null || !_strOutFile.equals(aFile) ) {
		if(getWriter() != null && getWriter() != _pwStdout) {
			getWriter().close();
		}

		try {
			_strOutFile = "stdout";
			setWriter(_pwStdout);

			if(aFile != null && aFile.length() > 0 && 
					!aFile.equalsIgnoreCase("stdout")) {
				setWriter(new PrintWriter(new FileWriter(aFile, isAppend()), true));
				_strOutFile = aFile;
				String strErrFile = "";

				if(aFile.equals(_strInFile + OUTFILE_EXT)) {
					strErrFile = _strInFile + ERRFILE_EXT;
				}
				else {
					strErrFile = aFile + ERRFILE_EXT;
				}
	
				if(_err != null) {
					_err.close();
				}
				_err = new PrintWriter(new FileWriter(strErrFile, isAppend()), true);
			}
		}
		catch(Exception e) {
		}		


		if(_strInFile != null && _strInFile.length() > 0) {
			logmsg("Reading input from  : " + _strInFile);
		}
		if(getWriter() != _pwStdout) {
			System.out.println("Writing output to   : " + _strOutFile );
			logmsg((new Date()).toString());
		}
	}
}
public String getIndentTab() {
	return _strCurIndent;
}
public void setIndentTab(String aIndentTab) {
	_strCurIndent = (aIndentTab == null) ? "" : aIndentTab;
}
public void changeIndentTab(boolean fIncrement) {
	if(fIncrement) {
		_strCurIndent = STR_INDENT + _strCurIndent;
	}
	else {
		if(_strCurIndent.startsWith(STR_INDENT)) {
			_strCurIndent = _strCurIndent.substring(STR_INDENT.length());
		}
	}
}
public PrintWriter getWriter() {
	return _out;
}
public void enableDualOut(boolean fEnable) {
	_fDualOut = fEnable;
}
public boolean isDualOut() {
	return(_fDualOut);
}
public void enableAppend(boolean fEnable) {
	_fAppend = fEnable;
}
public boolean isAppend() {
	return(_fAppend);
}
public void setWriter(PrintWriter apwLog) {
	_out = apwLog;
	if(_out == null) {
		_out        = _pwStdout;
		_strOutFile = "stdout";
	}
}
public final void logmsg(String aMsg) {
	logmsg(aMsg, true);
}
public final void logmsg(String aMsg, boolean fNewLine) {
	String lMsg = getIndentTab() + aMsg;

	if(fNewLine) {
		if(_out != _pwStdout) {
			System.out.println(lMsg);
		}
		_out.println(lMsg);
	}
	else {
		if(_out != _pwStdout) {
			System.out.print(lMsg);
			System.out.flush();
		}
		_out.print(lMsg);
		_out.flush();
	}
}
public void logmsgList(Object anArray) {
	logmsgList(anArray, STR_TAB);
}
public void logmsgList(Object anArray, String aStrPrefix) {
	if(anArray != null) {
		Class c        = anArray.getClass();
		if(c.isArray()) {
			Object[] lArray = (Object[])anArray;

			for(int i=0; i<lArray.length; i++) {
				logmsg(aStrPrefix + lArray[i]);
			}
		}
	}
}
public static final void logmsgMemUsed() {
	Runtime lRuntime = Runtime.getRuntime();
	long    lFree    = lRuntime.freeMemory();
	long    lTotal   = lRuntime.totalMemory();
	int     nUsed    = (int) (100 * lFree / lTotal);
	String  strFree  = (nUsed > 9) ? ("" + nUsed) : (" " + nUsed);

	System.out.println("Memory usage (free/total bytes) : " +  strFree + "% Free (" + lFree + "/" + lTotal + ")");
}
public static String sizeStr(String aString, int aLen) {
	return sizeStr(aString, aLen, true);
}
public static String sizeStr(String aString, int aLen, boolean fSufix) {
	String rc = aString;

	if(aString != null && aString.length() < aLen) {
		int    nBlankStrLen = aLen - aString.length();
		String lstrBlank    = "";

		if(nBlankStrLen > 0) {
			for(int nInd=0; nInd< (nBlankStrLen / BLANK_STR.length()); nInd++) {
				lstrBlank += BLANK_STR;
			}
			lstrBlank += BLANK_STR.substring(0, nBlankStrLen - lstrBlank.length());
			if(fSufix) {
				rc = rc + lstrBlank;
			}
			else {
				rc = lstrBlank + rc;
			}
		}
	}

	return rc;
}
public static String sizeStr(short aNum) {
	return sizeStr(aNum, 5, true);
}
public static String sizeStr(short aNum, int aLen) {
	return sizeStr(aNum, aLen, false);
}
public static String sizeStr(short aNum, int aLen, boolean fDotAtEnd) {
	String rc = (fDotAtEnd) ? ". " : " ";

	if(aNum > 0 && aNum < 10 ) {
		rc = " " + aNum + rc;
	}
	else {
		rc = aNum + rc;
	}
	return sizeStr(rc, aLen, false);
}
public static final boolean isCommand(String aString) {
	boolean rc = false;

	if(aString != null) {
		// if line is a blank line or starts with ; or # or // then 
		// then it's not a command line
		String lstrCmd = aString.trim();
		rc = (lstrCmd.length() > 0  &&
			!lstrCmd.startsWith("#" ) &&
			!lstrCmd.startsWith("//")    );
	}

	return rc;
}
public String[] getParms(String aCmdLine) {
	String[] rc = getTokens(aCmdLine);
	replaceVariables(rc, 1);
	return rc;
}
public static final String[] getTokens(String aCmdLine) {
	return getTokens(aCmdLine, 0, STR_COMMAND_DELIMITER);
}
public static final String[] getTokens(String aCmdLine, int numSkip) {
	return getTokens(aCmdLine, numSkip, STR_COMMAND_DELIMITER);
}
public static boolean isVarAssignment(String aCmd) {
	boolean rc = false;
	// first word must be a variable name (i.e. must start with ~)
	// and when tokenized first word must have '=' sign in it or
	// second token must start with '='

	if(aCmd != null) {
		StringTokenizer st = new StringTokenizer(aCmd);
		String          strToken;
		try {
			strToken = st.nextToken();
			if(strToken.charAt(0) == '~') {
				if(strToken.indexOf('=') != -1) {
					rc = true;
				}
				else {
					strToken = st.nextToken();
					if(strToken.charAt(0) == '=') {
						rc = true;
					}
				}
			}
		}
		catch(Exception e) {
		}
	}
	return rc;
}
public static final String[] getTokens(String aCmdLine, int numSkip, String aCmdDelimiter) {
	String  strsTokens[] = null;
	if(aCmdLine != null) {
		String  strCmd        = aCmdLine;
		int     nTokenS       = -1;
		int     nTokenE       = -1;
		char    chCur         = ' ';
		char    chPrev        = ' ';
		boolean fStrStart     = false;
		boolean fTokenFound   = false;
		String  strCurToken   = null;
		Vector  vectTokens    = new Vector();
		int     nDelCount     = numSkip;        // to skip first n arguments
		boolean varAssignment = isVarAssignment(aCmdLine);

		for(nTokenE=0; nTokenE<strCmd.length(); nTokenE++) {
			chCur = strCmd.charAt(nTokenE);
			if(chCur == '/' && chPrev == '/') {
				// skip C++ type comment's
				if(nTokenE > 1) {
					nTokenE--;
				}
				break;
			}
			else if(aCmdDelimiter.indexOf(chCur) == -1) {
				if(chCur == '"') {
					if(fStrStart) {
						if(chPrev != '\\') {
							if((nTokenE +1) < strCmd.length()) {
								strCurToken = strCmd.substring(nTokenS, nTokenE + 1);
							} 
							else {
								strCurToken = strCmd.substring(nTokenS);
							}
	
							strCurToken = stringValue(strCurToken);
							fStrStart = false;
							nTokenS   = -1;
						}
					}
					else {
						fStrStart = true;
						nTokenS   = nTokenE;
					}
				}
				else {
					if(nTokenS < 0) {
						nTokenS = nTokenE;
					}
				}
			}
			else {
				if(nTokenS >= 0 && !fStrStart) {
					strCurToken = strCmd.substring(nTokenS, nTokenE);
					nTokenS = -1;
				}
			}
	
			if(strCurToken != null) {
				if(nDelCount <= 0) {
					if(!varAssignment || vectTokens.size() > 1) {
						strCurToken = stringValue(strCurToken);
					}
					vectTokens.addElement(strCurToken);
				}
				else {
					nDelCount--;
				}
				strCurToken = null;
			}
			chPrev = chCur;
		}
		if(nTokenS >= 0 && nDelCount <= 0 && nTokenS < nTokenE) {
			// get the last token
			if((nTokenE) < strCmd.length()) {
				strCurToken = strCmd.substring(nTokenS, nTokenE);
			} 
			else {
				strCurToken = strCmd.substring(nTokenS);
			}

			if(!varAssignment || vectTokens.size() > 1) {
				strCurToken = stringValue(strCurToken);
			}
			vectTokens.addElement(strCurToken);
		}
		strsTokens = new String[vectTokens.size()];
		vectTokens.copyInto(strsTokens);

		/*
		for(int k=0; k<strsTokens.length; k++) {
			System.out.println(k + "." + strsTokens[k]);
		}
		*/
	
	}
	return strsTokens;
}
public String[] replaceVariables(String[] strsTokens) {
	return replaceVariables(strsTokens, 1);
}
public String[] replaceVariables(String[] aTokens, int nStartIndex) {
	if(aTokens != null && nStartIndex >= 0) {
		for(int nInd=nStartIndex; nInd<aTokens.length; nInd++) {
			if(aTokens[nInd].startsWith("~")) {
				// it's a script variable name replace it with it's 
				// string value
				aTokens[nInd] = (String)(getVariableValue(aTokens[nInd]));
			}
		}
	}
	return aTokens;
}
public final String stringConst(String aString) {
	return stringConst(aString, STR_COMMAND_DELIMITER);
}
public final String stringConst(String aString, String aCmdDelimiter) {
	// opposite of stringValue(String)
	// puts back double-quote marks if string has blanks etc.
	String rc = aString;
	if(rc != null) {
		// *** need to put back-slash if there is a double quote already
		// *** in the string
		for(short nInd=0; nInd<aCmdDelimiter.length(); nInd++) {
			if(rc.indexOf(aCmdDelimiter.charAt(nInd)) >= 0) {
				rc = "\"" + rc + "\"";
				break;
			}
		}
	}
	return rc;
}
public static String stringValue(String aString) {
	// take out backslashes and double-quote marks from string
	// literals
	StringBuffer lstrbuf = new StringBuffer();

	char    chCur;
	int     nStart;
	String  lVarName;
	boolean fEOS         = false;
	boolean fchPrevAdded = false;
	boolean fchAdded     = false;
	char    chPrev       = ' ';

	if(aString != null) {
		for(int nInd=0; nInd<aString.length(); nInd++) {
			fchAdded = false;
			chCur = aString.charAt(nInd);
			while((chCur == '~') && chPrev != '\\') {
				//it's a script variable
				for(nStart=nInd++; nInd<aString.length(); nInd++) {
					chPrev = chCur;
					chCur  = aString.charAt(nInd);
					if("() \t,/;\\[]=~".indexOf(chCur) >= 0) {
						break;
					}
				}
				fEOS = (nInd>=aString.length());
				lVarName = (fEOS) ? aString.substring(nStart) : aString.substring(nStart, nInd);
				lstrbuf.append((String)(getVariableValue(lVarName)));
			}

			if(!fEOS) {
				if(chCur == '"') {
					if(chPrev == '\\' && !fchPrevAdded) {
						lstrbuf.append(chCur);
						fchAdded = true;
					}
				}
				else if(chCur == '\\') {
					if((chPrev == '\\' && !fchPrevAdded)) {
						lstrbuf.append(chCur);
						fchAdded = true;
					}
				}
				else {
					lstrbuf.append(chCur);
					fchAdded = true;
				}

				chPrev       = chCur;
				fchPrevAdded = fchAdded;
			}
		}
	}
	return lstrbuf.toString();
}
public final String getRHS(String aCmd) {
	String lstrLine = "";
	try {
		if(aCmd != null && aCmd.length() > 0) {
			lstrLine = stringValue(aCmd.substring(aCmd.indexOf('=') + 1).trim());
		}
	}
	catch(Exception e) {
		logmsgError("in script syntax.  \n\tIt should be of form ~variableName = _StringValue_");
	}
	return lstrLine;
}
public static void main(String argv[]) {
	TLogUtils      lTLogUtils      = new TLogUtils();
	lTLogUtils.processCommandLineArgs(argv);
}
public static final String stringTypeToStr(short nType) {
	String rc = "unknown";
	switch(nType) {
		case DK_CM_ATTR_NONE              : rc = "NONE"             ; break;
		case DK_CM_ATTR_ALPHA             : rc = "ALPHA"            ; break;
		case DK_CM_ATTR_ALPHANUM          : rc = "ALPHANUM"         ; break;
		case DK_CM_ATTR_ALPHANUM_EXT      : rc = "ALPHANUM_EXT"     ; break;
		case DK_CM_ATTR_NUMERIC           : rc = "NUMERIC"          ; break;
		case DK_CM_ATTR_OTHER             : rc = "OTHER"            ; break;
		case DK_CM_ATTR_VAR_NONE          : rc = "VAR_NONE"         ; break;
		case DK_CM_ATTR_VAR_ALPHA         : rc = "VAR_ALPHA"        ; break;
		case DK_CM_ATTR_VAR_ALPHANUM      : rc = "VAR_ALPHANUM"     ; break;
		case DK_CM_ATTR_VAR_ALPHANUM_EXT  : rc = "VAR_ALPHANUM_EXT" ; break;
		case DK_CM_ATTR_VAR_NUMERIC       : rc = "VAR_NUMERIC"      ; break;
		case DK_CM_ATTR_VAR_OTHER         : rc = "VAR_OTHER"        ; break;
		case DK_CM_ATTR_FIX_NONE          : rc = "FIX_NONE"         ; break;
		case DK_CM_ATTR_FIX_ALPHA         : rc = "FIX_ALPHA"        ; break;
		case DK_CM_ATTR_FIX_ALPHANUM      : rc = "FIX_ALPHANUM"     ; break;
		case DK_CM_ATTR_FIX_ALPHANUM_EXT  : rc = "FIX_ALPHANUM_EXT" ; break;
		case DK_CM_ATTR_FIX_NUMERIC       : rc = "FIX_NUMERIC"      ; break;
		case DK_CM_ATTR_FIX_OTHER         : rc = "FIX_OTHER"        ; break;
	}
	return rc;
}
public static final String attrTypeToStr(short nType) {
	String rc = "unknown";
	switch(nType) {
		case DK_CM_UNDEFINED              : rc = "UNDEFINED"        ; break;
		case DK_CM_VSTRING                : rc = "VSTRING"          ; break;
		case DK_CM_FSTRING                : rc = "FSTRING"          ; break;
		case DK_CM_SHORT                  : rc = "SHORT"            ; break;
		case DK_CM_LONG                   : rc = "LONG"             ; break;
		case DK_CM_FLOAT                  : rc = "FLOAT"            ; break;
		case DK_CM_DECIMAL                : rc = "DECIMAL"          ; break;
		case DK_CM_DATE                   : rc = "DATE"             ; break;
		case DK_CM_TIME                   : rc = "TIME"             ; break;
		case DK_CM_TIMESTAMP              : rc = "TIMESTAMP"        ; break;
		case DK_CM_DOUBLE                 : rc = "DOUBLE"           ; break;
		case DK_CM_BIT                    : rc = "BIT"              ; break;
		case DK_CM_DDOOBJECT              : rc = "DDOOBJECT"        ; break;
		case DK_CM_XDOOBJECT              : rc = "XDOOBJECT"        ; break;
		case DK_CM_DATAOBJECTBASE         : rc = "DATAOBJECTBASE"   ; break;
		case DK_CM_COLLECTION             : rc = "COLLECTION"       ; break;
		case DK_CM_COLLECTION_DDO         : rc = "COLLECTION_DDO"   ; break;
		case DK_CM_COLLECTION_XDO         : rc = "COLLECTION_XDO"   ; break;
	}
	return rc;
}
protected void initVectVars() {
	if(!_isHelpInfoAdded_TLogUtils) {
		//_vectMethods.addElement(new _MethodInfo("", "", "", "", ""));
		_vectMethods.addElement(new _MethodInfo("help",          "Provide help information for commands.",    "help [command]", "e.g. help exit", ""));
		_vectMethods.addElement(new _MethodInfo("exit",          "exit/quit application.",                    "", "", "close"));
		_vectMethods.addElement(new _MethodInfo("quit",          "exit/quit application.",                    "", "", ""));
		_vectMethods.addElement(new _MethodInfo("listVariables", "List all script variables defined so far.", "", "", "listvariablename, variables, names"));
		_vectMethods.addElement(new _MethodInfo("echo",          "Turns command-echoing on or off.",          "echo {ON | OFF}", "e.g. echo ON", "ignoreError"));
		_vectMethods.addElement(new _MethodInfo("ignoreError",   "Turns error logging on or off.",            "ignoreError {ON | OFF}", "e.g. ignoreError ON", "echo"));
		_vectMethods.addElement(new _MethodInfo("set",           "Sets script variables.",                    "~variableName = string\n    variable name must start with ~", "e.g. ~Name=CHANDRA", "~, setvariablename, varaibles"));

		_isHelpInfoAdded_TLogUtils = true;
	}
}
class _MethodInfo {
	public _MethodInfo(String aName, String aDesc, String aSyntax, 
			String aHelp, String aSearchWords) {
		strName        = aName;
		strDesc        = aDesc;
		strSyntax      = aSyntax;
		strHelp        = aHelp;
		strSearchWords = aSearchWords;
	}
	public String toString() {
		String rc = (strName == null) ? "" : strName;
		return rc;
	}
	public String strName;
	public String strDesc;
	public String strSyntax;
	public String strHelp;
	public String strSearchWords;
}
}
