/*
 * @(#) TdkCallback.java    v2.0   1998/08/15
 *BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

/*
 * Filename       : TdkCallback.java
 * Description    : Server Definition class for DL
 * Change History :
 *
 * Date        Name  Description     
 * -------     ----- ---------------------------------------
 * 2000/06/28  CSP   Created from TCallbackDL sample program
 *
 */

import com.ibm.mm.sdk.common.*;

import java.util.*;
import java.lang.*;

/**
 TdkCallback : This class represents a server definition.
 */
	//implements dkCallback, DKConstantDL, DKMessageIdDL, java.io.Serializable

public class TdkCallback extends TLogUtils
	implements dkCallback
{
 
 /**
  *  Constructs an callback for DL
  */
	public TdkCallback() {
		this(null);
	}
	public TdkCallback(TLogUtils aParent) {
		super(aParent);
		_threshold = 5;
	}

 /**
  *  Sets the number of returned hits for each data callback.
  *  This method should be set by applications for the datastore.
  *  @param   nValue the number of hits in the result collection
  *           for each callback.
  */
	public void setThreshold(int nValue) {
		_threshold = nValue;
	}
 
 /**
  *  Gets the number of returned hits that the server return
  *  for each callback.  The datastore will call this method for
  *  each query.
  *  @return	the threshold value,
  */
	public int getThreshold() {
		//logmsg("--> TdkCallback  : getThreshold() called ");
		return _threshold; 
	}
 
 /**
  *  Query done check
  *  @return	true when done
  */
	public boolean isDone() {
		return _isDone; 
	}
 
 /**
  *  Indicates that the user has canceled the query.
  *  It should be called by the application to let the server
  *  knows to stop the query.
  */
	public void cancelQuery() {
		_cancel = true;
	}
 
 /**
  *  Returns nHits from the backend datastore
  *  @param 	aDDOList a collection of DKDDO objects
  *  @param 	isDone   a boolean value indicating whether the server is
  *  			done with the query.  That is, there is more hits.
  *  @return	true if query is done or user has canceled the query.
  */
	public boolean dataCallback(dkCollection aDDOList, boolean isDone) {
		boolean fDone = false;
		_numCalled++;
		//logmsg("--> TdkCallback  : dataCallback() " + _numCalled + " time called");

		if (_cancel == true || isDone == true) {
			fDone   = true;
			_isDone = true;
		}

		if(aDDOList != null && (aDDOList.cardinality() > 0 || !fDone)) {
			logmsgdkCollection(aDDOList);
		}
		return fDone;
	}
  
	private boolean _cancel    = false;  
	private boolean _isDone    = false;  
	private int     _threshold;
	private int     _numCalled = 0;
}
