/* -------------------------------------------------------------------
 * @copyright(disclaimer) 
 * IBM Enterprise Information Portal for Multiplatforms V7.1  (program number 5697-G29) 
 * (c ) Copyright IBM Corp. 1998, 2000. All Rights Reserved. 
 * IBM Content Manager for Multiplatforms V6.1  (program number 5648-D03) 
 * (c ) Copyright IBM Corp. 1994, 2000. All Rights Reserved. 
 * 
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.  Licensed Materials - Property of IBM 
 * 
 * DISCLAIMER OF WARRANTIES : 
 * 
 * Permission is granted to copy and modify this code, and to 
 * distribute modified versions provided that both the copyright 
 * notice and this permission notice appear in all copies and modified 
 * versions. THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM DISCLAIMS 
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE. IBM 
 * WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE 
 * OF THE CODE.
 * 
 * @endCopyright
 */

/*
 * Filename       : TCallbackIC.java
 * Description    : Callback class to receive results from IC
 *
 */

import com.ibm.mm.sdk.common.*;
import java.util.*;
import java.lang.*;


public class TCallbackIC 
	implements dkCallback, DKConstantIC, DKMessageIdIC, java.io.Serializable
{
 
  // constructor
 /**
  *  Constructs an callback for IC
  */
  public TCallbackIC()
  {
     _threshold = 10;
  }

 /**
  *  Sets the number of returned hits for each data callback.
  *  This method should be set by applications for the datastore.
  *  @param 	nValue the number of hits in the result collection
  *  		for each callback.
  */
 public void setThreshold(int nValue)
 {
    _threshold = nValue;
 }
 
 /**
  *  Gets the number of returned hits that the server return
  *  for each callback.  The datastore will call this method for
  *  each query.
  *  @return	the threshold value,
  */
 public int getThreshold()
 {
    return _threshold; 
 }
 
 /**
  *  Query done check
  *  @return	true when done
  */
 public boolean isDone()
 {
    return _isDone; 
 }
 
 /**
  *  Indicates that the user has canceled the query.
  *  It should be called by the application to let the server
  *  knows to stop the query.
  */
 public void cancelQuery()
 {
    _cancel = true;
 }
 
 /**
  *  Returns nHits from the backend datastore
  *  @param 	ddoList a collection of DKDDO objects
  *  @param 	isDone  a boolean value indicating whether the server is
  *  			done with the query.  That is, there is more hits.
  *  @return	true if query is done or user has canceled the query.
  */

public boolean dataCallback(dkCollection ddoList, boolean isDone)
 {
    boolean fDone = false;
    _numCalled++;
    System.out.println("---" + _numCalled + " time called ---");

    if (_cancel == true || isDone == true) {
      fDone = true;
     _isDone = true;
      if (ddoList != null)
      {
         System.out.println("--- ddoList contains items ---");
         try
         {
            processResults(ddoList);
          }
          catch (DKException exc)
          {
           System.out.println("Exception name " + exc.name());
           System.out.println("Exception message " + exc.getMessage());
           System.out.println("Exception error code " + exc.errorCode());
           System.out.println("Exception error state " + exc.errorState());
           exc.printStackTrace();
          }
          catch (Exception exc)
          {
           System.out.println("Exception message " + exc.getMessage());
           exc.printStackTrace();
          }
         } 
      }
      else {
         try {
            processResults(ddoList);
         }
          catch (DKException exc)
          {
           System.out.println("Exception name " + exc.name());
           System.out.println("Exception message " + exc.getMessage());
           System.out.println("Exception error code " + exc.errorCode());
           System.out.println("Exception error state " + exc.errorState());
           exc.printStackTrace();
          }
          catch (Exception exc)
          {
           System.out.println("Exception message " + exc.getMessage());
           exc.printStackTrace();
          }
         } 
    return fDone;
 }
  
 /**
  *  Report an exception from the backend datastore
  *  @param 	exc an exception from the backend datastore
  */
 public void reportException(DKException exc)
 {
  _exc = exc;
 }

 /**
  *  Get an exception from the backend datastore
  *  @return an exception from the backend datastore
  */
 public DKException getException()
 {
  return _exc;
 }

 /**
  *  Clear exception object
  */
 public void resetException()
 {
   _exc = null;
 }

  public void processResults(dkCollection results) 
                throws DKException, Exception
  {
     Short    sVal;
     short    itemType = 0;
     DKDDO    item = null;
     short    i = 0;
     String   dataName;
     DKParts  parts = null;
     DKFolder folder = null;
     System.out.println( "--- number of items:  " + results.cardinality() + " ---");
     System.out.println( "");
 
     dkIterator iter = results.createIterator();
 
     while(iter.more()) {
        item = (DKDDO) iter.next();
        if (item != null) {
   	       item.retrieve();
   	       sVal = (Short)item.getPropertyByName(DK_PROPERTY_ITEM_TYPE);
   	       itemType = sVal.shortValue();
   	       switch(itemType) {
   	       case DK_DOCUMENT:
   	          System.out.println("--- Item Information ---");
   	          for (i = 1; i <= item.dataCount(); i++) {
   	            dataName = item.getDataName(i);
   	            System.out.println(dataName + ": " + item.getData(i));
   	            if (dataName.equals(DKPARTS) == true) {
   	                parts = (DKParts)item.getData(i);
   	                if (parts != null) 
   		                System.out.println("   this document has " + parts.cardinality() + " parts ");
   	            }
   	            }
   	            break;
   	         case DK_FOLDER:
   	            System.out.println(" Item is a folder,    number of data item : " + item.dataCount());
   	     
   	            for (i = 1; i <= item.dataCount(); i++) {
   	               dataName = item.getDataName(i);
   	               System.out.println(dataName + ": " + item.getData(i));
   	               if (dataName.equals(DKPARTS) == true) {
   	                 parts = (DKParts)item.getData(i);
   	                 if (parts != null) 
   	                    System.out.println("   this folder has " + parts.cardinality() + " parts ");
   	                 }
   	                 if (dataName.equals(DKFOLDER) == true) {
   	                    folder = (DKFolder)item.getData(i);
   	                    System.out.println("   this folder has " + folder.cardinality() + " documents ");
   	                 }  
   	             }
   	      
   	             if (folder != null && folder.cardinality() > 0) {
   		             System.out.println("     --------- list of folder contents --------  ");
   		             processResults(folder);
   		             System.out.println("     ----------- end of list ------------------ ");
   	             }
   	             else 
   	                 System.out.println("  empty folder ");
   	             break;
   	          default:
   	             System.out.println("  Item is not recognized ");
   	             break;
   	       } 
        }
     } 
  }

 private boolean _cancel = false;  
 private boolean _isDone = false;  
 private int _threshold;
 private int _numCalled = 0;
 private DKException _exc = null;
}
