/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Connecting to a Datastore
    The first step in using the IBM Conent Management (ICM) connector is to establish
    a connection with the datastore.  The DKDatastoreICM.connect() method takes up to
    four pieces of information.  The most important, and most oftent used are the first
    three, Database Name, User Name, and Password.  The fourth parameter, connection
    options, can be left blank.  Please refer to the "Connection Options" section for
    more information.
    
Disconnecting from the Datastore
    When disconnecting from the datastore, use the disconnect() and destroy() methods.
    The destroy() methods become important especially dealing with Remote, Client, & CS
    Packages.

Error Handling
    The program model below demonstrates how to properly catch & print Errors.  The most
    important exception to catch is the DKException class.  In Java, java.lang.Exception
    must also be handled.  
    
    IMPORTANT NOTE:
        Exceptions should not be used for program logic.  User programs should not rely 
        on catching exceptions to detect if something exists in the datastore or not, or
        for any case other than truely exceptional cases.  If a program is written 
        correctly, exceptions should never be thrown.  
        
        Using exceptions in program logic will decrease performance and can render 
        trace & log information useless for debugging & support.
    
    Please examine all of the exception information, which includes up to six key pieces
    of information.  Print function are available in this sample to help print the most
    important information.
    
    There are numerous sub-classes of DKException.  Depending on the program, it may be
    best to handle each or some exceptions individually.

    DKException 
    Information  Description
    -----------  -----------------------------------------------------
        Name     Exception Class Name.  Will contain sub-class name.
      Message    A specific message wlll explain the error.  The 
                 message may contain a great deal of information at
                 times, sometimes encapsulating important variable
                 states at the time the error was detected.
    Message ID   A unique Message ID identifies this error type and
                 matches it to a core message used above.
    Error State  May contain additional error information regarding
                 the state of the OOAPI or Library Server Error. 
                 In the case when Library Server detects an error,
                 the following four pieces of information are packaged
                 here
                        Library Server Failure State
                        ----------------------------
                           -  Return Code  
                           -  Reason Code
                           -  Ext / SQL Return Code
                           -  Ext / SQL Reason Code
    Error Code   May contain the Library Server Return Code.               
    Stack Trace  A very important piece of information indicating the
                 failure point in the user program and exactly where
                 the error was last detected or handled by the OOAPI. 
    ------------------------------------------------------------------

Connection Options
    The 4th parameter of the DKDatastoreICM.connect() method, the connection string, is 
    an optional parameter will be left blank in almost all cases.  The information that
    can be optionally specified here will be taken care of when the product was 
    installed.  Configuration/ini files will be used to gather this information.  It is 
    used to specify addition options for connecting to the datastore.

    An example use of this parameter would be to set the DB2/SQL User Name & Password if
    it is both different than the ICM User Name & Password and was not specified when 
    when requested during install or has changed since without updating the product 
    setup.

    Again, in almost all circumstances, it is expected that "" be used as the value for
    this parameter.

    Format:
        Specify the options seperated by semi-colons (;).  

                 Options   Values                      Description
        ----------------   --------------------------  -----------------------------------
                  SQLUID = <DB2 User Name>             DB2 / SQL User Name or ID
                  SQLPWD = <DB2 User Name's Password>  DB2 / SQL User Name's Password
                    NPWD = <New ICM Password>          New ICM Password
                  SCHEMA = <DB2 Schema>                In DB2, Tables are created under 
                                                       a schema under which is qualifies 
                                                       the tables.
                    LANG = <Language Code to Use>      Language Code to Use
        TRACESERVERLEVEL = <LS Trace Level>            Trace level to use for the Library 
                                                       Server
                  DBAUTH = <"CLIENT" or "SERVER">      Turns on or off DB2 Authentication.
                LANCACHE = <"YES" or "NO">             Turn Lan Caching On, which allows 
                                                       caching of Resources in the default 
                                                       Resource Manager for objects stored 
                                                       on other Resource Managers.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SConnectDisconnectICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Connect, Disconnect, Error Handling, Printing Complete Error Information. 
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Connect
 *                    Disconnect
 *                    Error Handling, Catching
 *                    Printing Complete Error Messages
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SConnectDisconnectICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: None
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: None
 ************************************************************************************************/
public class SConnectDisconnectICM{
    
    // Public Constants for Defaults Used by Samples
    public static String DEFAULT_DATABASE = "icmnlsdb";
    public static String DEFAULT_USERNAME = "icmadmin";
    public static String DEFAULT_PASSWORD = "password";

    public static String EIP_VERSION      = "8.2";

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = DEFAULT_DATABASE;
        String userName = DEFAULT_USERNAME;
        String password = DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SConnectDisconnectICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;

        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SConnectDisconnectICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // Note: The 4th paramater of the connect method can be left
            //       blank in almost all circumstances.  The system can
            //       determine this information through other means.  If 
            //       desired, options include DB2 User Name & Password, 
            //       New ICM Password, DB2 Schema, Language Code, Trace Level,
            //       DB2 Authentication, and Lan Caching.  Please consult
            //       the APG, Programming References, and eSupport for more information.
            //-------------------------------------------------------------
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //******************************************************************
    // printException
    //******************************************************************
    /**
    * Prints the specified exception.
    * @param exc  DKException to print.
    */
    public static void printException(DKException exc){
        System.out.println("");
        System.out.println("XXXXXXXXXXXXXXXXXXXXXXXXXXXX");
        System.out.println("X     !!! Exception !!!    X");
        System.out.println("XXXXXXXXXXXXXXXXXXXXXXXXXXXX");
        System.out.println("       Name: " + exc.name());
        System.out.println("    Message: " + exc.getMessage());
        System.out.println(" Message ID: " + exc.getErrorId());
        System.out.println("Error State: " + exc.errorState());
        System.out.println(" Error Code: " + exc.errorCode());
        exc.printStackTrace();
        System.out.println("----------------------------------");
    }
    /**
    * Prints the specified exception.
    * @param exc  Exception to print.
    */
    public static void printException(Exception exc){
        System.out.println("");
        System.out.println("XXXXXXXXXXXXXXXXXXXXXXXXXXXX");
        System.out.println("X     !!! Exception !!!    X");
        System.out.println("XXXXXXXXXXXXXXXXXXXXXXXXXXXX");
        System.out.println("    Name: " + exc.getClass().getName());
        System.out.println(" Message: " + exc.getMessage());
        exc.printStackTrace();
        System.out.println("----------------------------------");
    }

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /** 
    * Connect to ICM datastore.
    * @param database  Name of the ICM database.
    * @param userName  ICM User Name.
    * @param password  Password for the specified User Name.
    * @param options   Leave blank, "", in almost all circumstances. 
    *                  The system can determine this information through 
    *                  other means.  If desired, options include DB2 
    *                  User Name & Password, new ICM Password, DB2 
    *                  Schema, Language Code, Trace Level, DB2 
    *                  Authentication, and Lan Caching.  Please consult
    *                  the APG, Programming References, and eSupport 
    *                  for more information.
    * @return  Returns a connected DKDatastoreICM object.
    **/
    public static DKDatastoreICM connect(String database, String userName, String password, String connectionOptions) throws DKException, Exception{
        System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

            DKDatastoreICM dsICM = new DKDatastoreICM(); // Create new datastore object.
            dsICM.connect(database,userName,password,connectionOptions); // Connect to the datastore.
            // Note: In almost all cases, connectionOptions = "".

        System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");
        return(dsICM);
    }

   /**
    * Disconnect from datastore & destroy reference
    * @param datastore     Connected datastore to disconnect from and destroy reference.
    **/
    public static void disconnect(dkDatastore datastore) throws DKException, Exception{
        System.out.println("Disconnecting from datastore & destroying reference...");            

            datastore.disconnect();
            datastore.destroy();

        System.out.println("Disconnected from datastore & destroying reference.");
    }

    
}//end class SConnectDisconnectICM
              
