/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Delete Rules
    Delete rules are set at design-type / data model definition and apply to references
    and child component types / multivalue attribute definitions.
    
    Delete Rules may prevent deletion of an item with children or references.
    
    1. Please refer to SItemTypeCreationICM for complete information on delete rules 
       that are set for child component types.

    2. Please refer to SReferenceAttrDefCreationICM for complete information on delete
       rules that are set for reference attributes.

Deleting Child Components / Multivalue Attributes
    Remember that Child Components are multivalue attributes of the parent.  
    Multivalue attributes must be modified, added, and removed / deleted through
    the parent.  All operations must be made persistent through the root component's
    DKDDO.add() or DKDDO.update() functions.
    
    Deleting a child component is performed through simply removing the particular
    DKDDO object from the parent's DKChildCollection such as using the 
    DKChildCollection.removeElementAt(dkIterator) function and then calling
    DKDDO.update() on the root component.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SItemDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Items, Deleting All Items of an Item Type
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Items
 *                    Deleting All Items of an Item Type
 *                    Deletes all Items created by SItemCreationICM, SItemRetrievalICM,
 *                        & SItemUpdateICM.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SItemDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemTypeRetrievalICM.java
 *                    SItemCreationICM.java
 ************************************************************************************************/
public class SItemDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SItemDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SItemDeletionICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create an Sample Items
            //-------------------------------------------------------------
            // create 3 Items, one of Item Proprty type Document, one Folder, & one Item.
            // Will use the function for creating a simple item with children, with the
            // attribute data set from the SItemCreationICM Sample.
            System.out.println("Creating & Saving Sample Items...");

	            DKDDO ddoDocument = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_DOCUMENT);
	            DKDDO ddoFolder   = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_FOLDER);   
	            DKDDO ddoItem     = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_ITEM);     

            System.out.println("Created & Saved Sample Items.");

            //-------------------------------------------------------------
            // Deleting items from persistent datastore
            //-------------------------------------------------------------
            // Delete Rules may prevent deletion of an item with children or
            // references.
            // 
            // 1. Please refer to SItemTypeCreationICM for complete information
            //    on delete rules that are set for child component types.
            // 2. Please refer to SReferenceAttrDefCreationICM for complete
            //    information on delete rules that are set for reference 
            //    attributes.
            System.out.println("Deleting Items...");

                ddoDocument.del();
                ddoFolder.del();
                ddoItem.del();

            System.out.println("Deleted Items.");

            //-------------------------------------------------------------
            // Delete all items of an item type
            //-------------------------------------------------------------
            // The wrapper provided at the bottom of this sample will delete
            // all items of any specified Item Type.  This will be used to 
            // clean up all sample items created by SItemCreationICM, 
            // SItemRetrievalICM, and SItemUpdateICM.  The wrapper uses
            // Query/Search, which will be explained in another sample.
            System.out.println("Deleting All Items of Item Type 'S_withChild'");

                // The only item type used with this section of samples was "S_withChild".
                deleteAllItems(dsICM,"S_withChild");
            
            System.out.println("Deleting All Items of Item Type 'S_withChild'");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete all items of the specified Item Type.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  Item Type Name for which to delete all items of.
    **/
    public static void deleteAllItems(DKDatastoreICM dsICM, String itemTypeName) throws DKException, Exception{
        System.out.println("Deleting All Items of Type '"+itemTypeName+"'...");

            // First check to see if the Item Type exists, if not, simply return.
            if(!SItemTypeRetrievalICM.doesEntityExist(dsICM,itemTypeName))
                return;

            int count = 0; // used to count number of items deleted.
            
            // Build the query string to look like: "/<ItemType>"
            String queryString = "/"+itemTypeName;

            // Evaluate Query
            DKNVPair parms[] = new DKNVPair[2];
            parms[0] = new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE,    new Integer(DKConstant.DK_CM_CONTENT_IDONLY)); // Only the Pid Information will be needed to delete.
            parms[1] = new DKNVPair(DKConstant.DK_CM_PARM_END,         null);
            DKResults results = (DKResults)dsICM.evaluate(queryString, DKConstant.DK_CM_XQPE_QL_TYPE, parms); // Evaluate Query
    
            // Process Results
            if (results != null) {
                dkIterator iter = results.createIterator();
    
                while(iter.more()) { // While there are items left to delete
                    DKDDO item = (DKDDO) iter.next();  // get the next item in the result list
                    if (item != null) { // if an item was retrieved, add it to the vector.
                        item.del();
                        count++;        // count the number deleted
                    }
                }
            }//end if
            
        System.out.println("Deleted All '"+count+"' Items of Type '"+itemTypeName+"'.");
    }//end deleteAllItems
       
}//end class SItemDeletionICM
              
