/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Extension of SResourceItem3rdPartyIngestICM Sample
    This Sample is an extension of the documentation of the 3rd Party Ingest
    Sample.  This Sample will explain concepts that pertain specifically to 
    Items with Video Resource Content.  Please refer to SResourceITem3rdPartyIngestICM
    for all information on importing from a 3rd party source that is not
    specific to Video Resource Items.
    
Creating Video Items
    Video Items extend standard resource functionality.  They include all 
    functionality covered in DKLobICM as shown in SResourceItemCreationICM.  
    Content may be leaded from the local file system, by stream, from memory,
    and from 3rd party sources.  Video Items extend this the functionality 
    further to new functionality in the DKStreamICM & DKVideoStreamICM objects.
    The class functionality extension is shown below.
    
        DKLobICM -> DKStreamICM -> DKVideoStreamICM
        
    This particular sample will use 3rd Party Ingest to Create Video Items.
    
    Additional Requirements
        Video Charger must installed and properly configured with Resource Manager.
        
    Additional Options & Properties
        Additional options & properties that apply to Stream & Video Stream objects.
        Please refer to the Application Programming Reference (javadoc) for complete
        information on DKStreamICM & DKVideoStreamICM.  A few of these properties are
        explained below:
        
        Stream Group (Optional)
            Group that a video or stream will belong to within the Video Charger.
            If not specified, the Video Charger's default group will be used.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.sql.*;
import java.io.*;
import java.net.*;

/************************************************************************************************
 *          FILENAME: SVideoItemCreateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating Items with Video Resource Content.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Items with Video Resource Content.
 *                    Storing Videos from a 3rd-Pary Source.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SVideoItemCreateICM  <3rd Party Host Name>
 *                                              <3rd Party Port>
 *                                              <3rd Party Protocol (http|ftp|file)>
 *                                              <3rd Party User Name>
 *                                              <3rd Party Password>
 *                                              <3rd Party Fully-Qualified File Name>
 *                                              <3rd Party File Path>
 *                                              <3rd Party MIME Type>
 *                                              <3rd Party Content Length>
 *                                              <database> <userName> <password>
 *
 *                    NOTE:  All 3rd Party Information is Required
 *                        
 *                    NOTE:  Database, UserName, & Password are optional.  If all not specified
 *                           defaults will be used.
 *                           
 *                    Ex: java SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428
 *                    Ex: java SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Sample Video Resource Item Type(s) must already be defined.  If not,
 *                    please run the following sample:
 *                        - SVideoItemTypeCreateICM
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    provided videos in the root directory:
 *                        - SVideoItemICM_Video1.mpg
 *                        - SVideoItemICM_Video2.mpg
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SVideoItemICM_Video1.mpg -> On 3rd Party Host
 *                    AND/OR SVideoItemICM_Video2.mpg -> On 3rd Party Host
 *                    AND/OR Other Video -> On 3rd Party Host
 ************************************************************************************************/
public class SVideoItemCreateICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // 3rd Party Information
        String thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        String thirdPartyProtocol       = "NOT_SPECIFIED";
        String thirdPartyUserName       = "NOT_SPECIFIED";
        String thirdPartyPassword       = "NOT_SPECIFIED";
        String thirdPartyFullFileName   = "NOT_SPECIFIED";     
        String thirdPartyFilePath       = "NOT_SPECIFIED";
        String thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 12) { // if not all 12 arguments were specified, report usage.
            System.out.println("Usage: " );
            System.out.println("  java SVideoItemCreateICM  <3rd Party Host Name>");
            System.out.println("                            <3rd Party Port>");
            System.out.println("                            <3rd Party Protocol (http|ftp|file)>");
            System.out.println("                            <3rd Party User Name>");
            System.out.println("                            <3rd Party Password>");
            System.out.println("                            <3rd Party Fully-Qualified File Name>");
            System.out.println("                            <3rd Party File Path>");
            System.out.println("                            <3rd Party MIME TYpe>");
            System.out.println("                            <3rd Party Content Length>");
            System.out.println("                            <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE:  All 3rd Party Information is Required.");
            System.out.println("");
            System.out.println("  NOTE:  Database, UserName, & Password are optional.  If all not specified");
            System.out.println("         defaults will be used.");
            System.out.println("");
            System.out.println("  Ex: java SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428");
            System.out.println("  Ex: java SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password");
            System.out.println("");
        }
        if(argv.length < 9){ // If any 3rd Party info not specified, terminate
            System.out.println("===================================================");
            System.out.println(" TERMINATED:  All 3rd Party info must be specified.");
            System.out.println("===================================================");
            return;        
        } else if(argv.length < 12) {
            System.out.println("*** Some parameters not specified.  Using defaults for rest. ***");
        }
        // Set Command-line options
        if (argv.length > 0)  thirdPartyHostName      = argv[0];
        if (argv.length > 1)  thirdPartyPort          = Integer.valueOf(argv[1]).intValue();
        if (argv.length > 2)  thirdPartyProtocol      = argv[2];
        if (argv.length > 3)  thirdPartyUserName      = argv[3];
        if (argv.length > 4)  thirdPartyPassword      = argv[4];
        if (argv.length > 5)  thirdPartyFullFileName  = argv[5];
        if (argv.length > 6)  thirdPartyFilePath      = argv[6];
        if (argv.length > 7)  thirdPartyMIMEType      = argv[7];
        if (argv.length > 8)  thirdPartyContentLength = Integer.valueOf(argv[8]).intValue();
        if (argv.length > 9)  database                = argv[9];
        if (argv.length > 10) userName                = argv[10];
        if (argv.length > 11) password                = argv[11];
        
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SVideoItemCreateICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Content Manager:");
        System.out.println("      Database:  "+database);
        System.out.println("      UserName:  "+userName);
        System.out.println(" 3rd Party:");
        System.out.println("      Host Name:  "+thirdPartyHostName);
        System.out.println("           Port:  "+thirdPartyPort);
        System.out.println("       Protocol:  "+thirdPartyProtocol);
        System.out.println("       UserName:  "+thirdPartyUserName);
        System.out.println("       Password:  "+thirdPartyPassword);
        System.out.println("           File:  "+thirdPartyFullFileName);
        System.out.println("      Mime Type:  "+thirdPartyMIMEType);
        System.out.println(" Content Length:  "+thirdPartyContentLength);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Empty Video Resource Items
            //-------------------------------------------------------------
            // Both Video Stream and the class that it exends, Stream, are 
            // usefull classes that the new DDO may be cast to.
            System.out.println("Creating Empty Video Resource Items...");
    
                DKVideoStreamICM video1 = (DKVideoStreamICM)dsICM.createDDO("S_videoType", DKConstant.DK_CM_ITEM); 
                DKVideoStreamICM video2 = (DKVideoStreamICM)dsICM.createDDO("S_videoType", DKConstant.DK_CM_ITEM); 

            System.out.println("Created Empty Video Resource Items.");

            //-------------------------------------------------------------
            // Set the User-Defined Attributes
            //-------------------------------------------------------------
            System.out.println("Setting User-Defined Attributes...");     

                video1.setData(video1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_author"), "IBM"); 
                video1.setData(video1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_length"), "48Hours"); 

                video2.setData(video1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_author"), "IBM"); 
                video2.setData(video1.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_length"), "48Hours"); 

            System.out.println("Set User-Defined Attributes.");

            //-------------------------------------------------------------
            // Setting the MIME Type (type of content)
            //-------------------------------------------------------------
            // mimeType: The type of content, such as "image/mpg"
            //
            // For a list of common MIME Types, please refer to the 
            // header documentation and SResourceItemMimeTypesICM.txt.
            System.out.println("Setting MIME Type / Type of Content...");

                video1.setMimeType(thirdPartyMIMEType);
                video2.setMimeType(thirdPartyMIMEType);

            System.out.println("Set MIME Type / Type of Content.");

            //-------------------------------------------------------------
            // Set 3rd Party Server Info
            //-------------------------------------------------------------
            // When the resource content is to be obtained from a 3rd party,
            // the 3rd party information may be specified as shown below.   
            // Please refer to the Application Programming Reference
            // (javadoc) for more informaiton.
            System.out.println("Setting 3rd Party Server Info...");

                // 3rd Party Info for Video 1
                DKThirdPartyServerDef thirdParty1 = new DKThirdPartyServerDef();
                thirdParty1.setHostName(thirdPartyHostName);
                thirdParty1.setPort(thirdPartyPort);
                thirdParty1.setProtocol(thirdPartyProtocol);
                thirdParty1.setUserId(thirdPartyUserName);
                thirdParty1.setPassword(thirdPartyPassword);
                thirdParty1.setFileName(thirdPartyFullFileName);
                thirdParty1.setFilePath(thirdPartyFilePath);
                thirdParty1.setInputContentLength(thirdPartyContentLength);

                // 3rd Party Info for Video 2
                DKThirdPartyServerDef thirdParty2 = new DKThirdPartyServerDef();
                thirdParty2.setHostName(thirdPartyHostName);
                thirdParty2.setPort(thirdPartyPort);
                thirdParty2.setProtocol(thirdPartyProtocol);
                thirdParty2.setUserId(thirdPartyUserName);
                thirdParty2.setPassword(thirdPartyPassword);
                thirdParty2.setFileName(thirdPartyFullFileName);
                thirdParty2.setFilePath(thirdPartyFilePath);
                thirdParty2.setInputContentLength(thirdPartyContentLength);

            System.out.println("Set 3rd Party Server Info.");

            //-------------------------------------------------------------
            // Synchronous Store
            //-------------------------------------------------------------
            // By default, the DKThirdPartyServerDef object is set for
            // syncronous.
            System.out.println("Storing Synchronously...");

                video1.add(thirdParty1,0);  // Add from the 3rd Party specified above.

            System.out.println("Stored Synchronously.");

            //-------------------------------------------------------------
            // Asynchronous Store
            //-------------------------------------------------------------
            // The DKThirdPartyServerDef must be set for Async.
            System.out.println("Storing Asynchronously...");

                // Specify Async. in 3rd Party Info.
                thirdParty2.setAsyncOperation(true); 
                                
                // REQUIREMENT:  In order for Asynchronous operations to complete,
                // the Resource Manager must be able to talk to the Library Server
                // independently, after this operation has completed.  In v8.2
                // this requires that both Library Server be cataloged on the
                // Resource Manager and/or be on the same machine.  If your system
                // meets these conditions, you may uncomment out the line below.

                //video2.add(thirdParty2,0);  // Add from the 3rd Party specified above.

            System.out.println("Stored Asynchronously.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SVideoItemCreateICM
		         
	       		    
			
