/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

// sample connect/execute/retrieve/disconnect to OnDemand

/*
 * Filename       : TRetrieveFolderWithCallbackOD.java
 * Description    : Connect to OnDemand Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                  if folders found display them and their documents/
 *                  folders.
 *                  Disconnect from OD Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TRetrieveFolderWithCallbackOD class
public class TRetrieveFolderWithCallbackOD implements DKConstantOD
{
  static String folderName;

  static int       NumCriteria = 0;
  static String[]  CriteriaNames = null;
  static String[]  CriteriaValues = null;
  static short[]   CriteriaOperators = null;

  // Main method
  public static void main(String argv[])
  {
    int threshold = 1, iterations = 1;
    boolean retrieveHits = true;

    DKDatastoreOD dsOD = null;
    DKFolderOD folder = null;

    try
    {
      //*** for input parameters
      String libSrv = new String(""),     // library server
             userid = new String(""),     //userid
             pw = new String(""),         // pw
             connect_string = new String(""),
             configuration_string = new String(""),
             threshold_string = new String(""),
             iterations_string = new String("");

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TRetrieveFolderWithCallbackOD " + "<libSrv> <userid> <pw> <connect string> <configuration string> threshold iterations" );
        return;
      }
      else
      {
        libSrv = argv[0];
        userid = argv[1];
        pw = argv[2];
        if (argv.length > 3)
          connect_string = argv[3];
        if (argv.length > 4)
          configuration_string = argv[4];
        if (argv.length > 5 )
        {
          threshold_string = argv[5];
          threshold = Integer.parseInt(threshold_string);
        }
        if (argv.length > 6 )
        {
          iterations_string  = argv[6];
          iterations = Integer.parseInt(iterations_string);
        }
      }
      System.out.println("connection string=>" + connect_string + "<=");
      System.out.println("configuration string=>" + configuration_string + "<=");

      if (configuration_string.length() > 0)
      {
        configuration_string += ";ENTITY_TYPE=TEMPLATES";
        dsOD = new DKDatastoreOD(configuration_string);
      }
      else
      {
        dsOD = new DKDatastoreOD("ENTITY_TYPE=TEMPLATES");
      }

      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      // Select a testcase by changing this number!!!
      int testcase = 3;
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      setFolderQueryCriteria(testcase, libSrv);

      System.out.println("\nUsing folder =>"+folderName+"<=");

      System.out.println("connecting to datastore");
      dsOD.connect(libSrv,userid,pw,connect_string);
      System.out.println("datastore connected, libSrv: " + libSrv + ", userid: " +
                         userid );

      DKNVPair[] parms = new DKNVPair[2];
      parms[0] = new DKNVPair("MAX_RESULTS",
                 new String(Integer.toString(threshold*iterations)));
      parms[1] = new DKNVPair("CONTENT", new String("ATTRONLY"));

      // Get datastore definition object
      DKDatastoreDefOD datastoreDefOD = new DKDatastoreDefOD(dsOD);
      folder = (DKFolderOD)datastoreDefOD.getEntity(folderName);

      int i = 0;
      if (folder != null)
      {
        // Set the search criteria
        for (i = 0; i < NumCriteria; i++)
        {
          DKSearchCriteria crit = folder.getCriteria(CriteriaNames[i]);
          System.out.println("CriteriaNames["+i+"]=>"+CriteriaNames[i]+"<=");
          if (crit != null)
          {
            crit.setValue(CriteriaValues[i]);
            crit.setOperator(CriteriaOperators[i]);
            System.out.println("  Name: " + CriteriaNames[i]);
            System.out.println("  =>value: " + CriteriaValues[i]);
            System.out.println("  =>operator: " + Op2Str(CriteriaOperators[i]));

            //System.out.println("\n******");
            //DisplayCritValues(crit);
            //System.out.println("******\n");
          }
        }

        System.out.println("executing query");

        long  t = 0;
        Date date  = new Date();
        System.out.println("Before query, time is " + date.toString());
        t = (new Date()).getTime();

        // Execute with callback!!!
        TCallbackOD callback = new TCallbackOD(dsOD,
                                               threshold,
                                               iterations,
                                               retrieveHits); // whether to save hit for retrieval
        System.out.println("***Threshold is set to "+threshold+", iterations is set to "+iterations );
        dsOD.executeWithCallback(folder, null, callback);
        // wait until the callback object is done...
        while (!callback.isDone())
          Thread.sleep (1000);

        // check for exception
        DKException cbExc = callback.getException();
        if (cbExc != null)
        {
          System.out.println("Exception name " + cbExc.name());
          System.out.println("Exception message " + cbExc.getMessage());
          System.out.println("Exception error code " + cbExc.errorCode());
          System.out.println("Exception error state " + cbExc.errorState());
          cbExc.printStackTrace();
          callback.resetException(); // reset since it is not needed any more
        }

        t = (new Date()).getTime() - t;
        System.out.println("Execute elapsed time: " + t + " msec");
        date  = new Date();
        System.out.println("After query, time is " + date.toString());
        System.out.println("datastore executed query");

        // retrieve and display the hits (DDOs)
        if (retrieveHits)
        {
          System.out.println("");
          System.out.println("================================");
          System.out.println("==the retrieval process begins==");
          System.out.println("================================");
          callback.retrieveDDO();
        }
      }
      else
      {
        System.out.println("Folder =>"+folderName+"<= not found on "+libSrv+"!!!");
      }

      dsOD.disconnect();
      System.out.println("\ndatastore disconnected");

    dsOD.destroy(); // Finished with the datstore
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
    System.out.println("TRetrieveFolderWithCallbackOD done ...");
  }

  // --------------------------------------------------------------------
  //   Op2Str
  // --------------------------------------------------------------------
  public static String Op2Str(short Op)
  {
    String OpStr = new String("");
    switch (Op)
    {
      case DK_CM_OPCODE_EQ         : OpStr = "EQ"         ; break;
      case DK_CM_OPCODE_NEQ        : OpStr = "NEQ"        ; break;
      case DK_CM_OPCODE_GT         : OpStr = "GT"         ; break;
      case DK_CM_OPCODE_LT         : OpStr = "LT"         ; break;
      case DK_CM_OPCODE_LIKE       : OpStr = "LIKE"       ; break;
      case DK_CM_OPCODE_NOTLIKE    : OpStr = "NOTLIKE"    ; break;
      case DK_CM_OPCODE_BETWEEN    : OpStr = "BETWEEN"    ; break;
      case DK_CM_OPCODE_IN         : OpStr = "IN"         ; break;
      case DK_CM_OPCODE_NOTIN      : OpStr = "NOTIN"      ; break;
      case DK_CM_OPCODE_GEQ        : OpStr = "GEQ"        ; break;
      case DK_CM_OPCODE_LEQ        : OpStr = "LEQ"        ; break;
      case DK_CM_OPCODE_NOTBETWEEN : OpStr = "NOTBETWEEN" ; break;
      case DK_CM_OPCODE_AND        : OpStr = "AND"        ; break;
      case DK_CM_OPCODE_OR         : OpStr = "OR"         ; break;
      case DK_CM_OPCODE_NOT        : OpStr = "NOT"        ; break;
      case DK_CM_OPCODE_UNDEFINED  : OpStr = "UNDEFINED"  ; break;
      default                      : OpStr = "Unknown!!!" ; break;
    }
    return OpStr;
  }

  // --------------------------------------------------------------------
  // Set the folder query criteria
  // --------------------------------------------------------------------
  public static void setFolderQueryCriteria(int testcase, String libSrv)
  {
    // Folder and search criteria
    // OD/390 v2.1
    if (libSrv.equalsIgnoreCase("9.91.121.31") == true)
    {
      NumCriteria = 3;
      CriteriaNames     = new String[NumCriteria];
      CriteriaValues    = new String[NumCriteria];
      CriteriaOperators = new short[NumCriteria];

      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "CHKSTMN1";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
        case 2:
          folderName = "COMPAT2";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
        case 3:
          folderName = "COMPATD";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
        case 4:
          folderName = "DELINQCY";
          CriteriaNames[0] = "LOAN NUMBER";
          break;
        case 5:
          folderName = "FSINV";
          CriteriaNames[0] = "INVOICE#";
          break;
        case 6:
          folderName = "INSURE";
          CriteriaNames[0] = "POLICY NUMBER";
          break;
        case 7:
          folderName = "INSURTST";
          CriteriaNames[0] = "POLICY NUMBER";
          break;
        case 8:
          folderName = "K0560";
          CriteriaNames[0] = "FIL#";
          break;
        case 9:
          folderName = "KEYBANK";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
      }
      CriteriaValues   [0] = "%";
      CriteriaOperators[0] = DK_CM_OPCODE_EQ;
      CriteriaNames    [1] = "FROM (YYYY-MM-DD)";
      CriteriaValues   [1] = "1990-05-26";
      CriteriaOperators[1] = DK_CM_OPCODE_GEQ;
      CriteriaNames    [2] = "TO   (YYYY-MM-DD)";
      CriteriaValues   [2] = "2001-12-31";
      CriteriaOperators[2] = DK_CM_OPCODE_LEQ;
    }
    // OD/390 v7.1
    else if (libSrv.equalsIgnoreCase("9.91.121.34") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP data
          folderName = "Credit Card Statements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account";
          CriteriaValues   [0] = "%"; //"000-000-010";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 2:
          folderName = "CHKSTMNT";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "ACCOUNT NUM";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 3:  // line data
          folderName = "Delinquency";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Page Number";
          CriteriaValues   [0] = "0";
          CriteriaOperators[0] = DK_CM_OPCODE_NEQ;
          break;
        case 4:  // line data large object
          folderName = "LOAN LO";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Date";
          CriteriaValues   [0] = "10/01/94";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
        case 5:  // pdf
          folderName = "PDF Landscape";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Region";
          CriteriaValues   [0] = "'%'";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        case 6:  // AFP
          folderName = "afpdata";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Tax Year";
          CriteriaValues   [0] = "1995";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
      }
    }
    // OD/400 v4.5
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400c") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "CHECKSTMTS";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account Number";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    // OD/400 v5.1
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400g") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          folderName = "CheckStatements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Customer Name";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 6 &&
             libSrv.substring(0,6).equalsIgnoreCase("reagan") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "load-db2-CRD";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account Name";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("everest") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // PDF
          folderName = "43MBPDF";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Report Title";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("akira") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "rl-DFAC";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "nomcli"; //"exploit";
          CriteriaValues   [0] = "MR PEREZ ERIC"; //"%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 2:
          folderName = "LARpmr94348";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Customer Id"; ;
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 3:
          folderName = "Credit Card Statements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account"; ;
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("banff") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "load-db2-afpdata";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Tax Year";
          CriteriaValues   [0] = "1995-01-01";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
        case 2:  // Line
          folderName = "load-db2-LOAN";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Date";
          CriteriaValues   [0] = "1994-10-01";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
      }
    }
    else //if (libSrv.equalsIgnoreCase("gunnar") == true)
    {
      // Single object AFP
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "Credit Card Statements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account";
          CriteriaValues   [0] = "%"; //"000-000-010";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
  }

  // --------------------------------------------------------------------
  // Display search criteria
  // --------------------------------------------------------------------
  public static void DisplayCritValues(DKSearchCriteria crit)
  {
    int i = 0;
    System.out.println("Name          =>"+crit.getName()+"<=");
    System.out.println("DisplayName   =>"+crit.getDisplayName()+"<=");
    System.out.println("EntityName    =>"+crit.getEntityName()+"<=");
    System.out.println("AttrName      =>"+crit.getAttrName()+"<=");
    System.out.println("CritId        = "+crit.getId());

    short[] validOps = crit.getValidOperators();
    if (validOps != null)
    {
      System.out.println("\nNumber of valid operators = "+validOps.length);
      for (i=0;i<validOps.length;i++)
      {
        System.out.println((i+1)+". =>"+Op2Str(validOps[i])+"<= \t("+validOps[i]+")");
      }
    }
    System.out.println("\nDefaultOperator=>"+Op2Str(crit.getDefaultOperator())+"<= \t("+crit.getDefaultOperator()+")");
    String[] defValues = crit.getDefaultValues();
    if (defValues != null)
    {
      System.out.println("\nNumber of default values = "+defValues.length);
      for (i=0;i<defValues.length;i++)
      {
        System.out.println((i+1)+". =>"+defValues[i]+"<=");
      }
    }
    System.out.println("\nOperator=>"+Op2Str(crit.getOperator())+"<= \t("+crit.getOperator()+")");
    System.out.println("Value()=>"+crit.getValue()+"<=");

    String[] values = crit.getValues();
    if (values != null)
    {
      System.out.println("\nNumber of Values = "+values.length);
      for (i=0;i<values.length;i++)
      {
        System.out.println((i+1)+". =>"+values[i]+"<=");
      }
    }
    System.out.println("\n");
    System.out.println("DisplayPosition = "+crit.getDisplayPosition());
    System.out.println("DisplayWidth    = "+crit.getDisplayWidth());
    System.out.println("CriteriaPosition= "+crit.getCriteriaPosition());
    System.out.println("isDisplayable   = "+crit.isDisplayable());
    System.out.println("isQueryable     = "+crit.isQueryable());
    System.out.println("getType         = "+crit.getType());
  }
}

