/**
@copyright(disclaimer)
Licensed Materials - Property of IBM

IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE
IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS
BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE
DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import com.ibm.mm.viewer.annotation.*;

/**
 * TAnnotationEditor sample shows how to build a GUI application to perform annotation editing using the Java Viewer
 * Annotation Services support framework. Additionally demonstrates how to plug in a custom annotation type to the
 * annotation support.
 *<br>
 *
 */
public class TAnnotationEditor extends JFrame {

  private CMBAnnotationServices annoServices;
  private CMBAnnotationSet annotationSet;
  private Image backImage;
  private AnnotationComponent annoComponent;
  private CMBAnnotationView annoView;

  private JToolBar annoToolBar = new JToolBar();
  private boolean traceEnabled = false;

  public TAnnotationEditor() {
      annoServices = new CMBAnnotationServices( new AnnotationServicesCallbacks());
  }

  private void initialize()
  {
    int screenWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
    int screenHeight = Toolkit.getDefaultToolkit().getScreenSize().height;
    int viewerWidth = screenWidth*3/4;
    int viewerHeight = screenHeight*3/4;
    setLocation((screenWidth-viewerWidth)/2,0);
    setTitle("Sample Annotation Services");
    String imageFileName = JOptionPane.showInputDialog("Enter a image filename to be used as a background:");
    backImage =  Toolkit.getDefaultToolkit().createImage(imageFileName);
     try
        {
          MediaTracker tracker = new MediaTracker(this);
          tracker.addImage(backImage,0);
          tracker.waitForID(0);
        } catch (InterruptedException e) {
          System.out.println(e);
        }
   createToolbar();
   this.getContentPane().setLayout( new BorderLayout());
   this.getContentPane().add( annoToolBar, BorderLayout.NORTH );

   annoComponent = new AnnotationComponent();
   prepareAnnotationServices(null);
   this.getContentPane().add( annoComponent, BorderLayout.CENTER );

    this.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
              System.exit(0);
            }
     });
    pack();
    setSize(600,600);
    show();

  }
   public void createToolbar()
  {
    JButton load = new JButton("Load annotations");
    annoToolBar.add( load );
    load.addActionListener(new ActionListener(){
        public void actionPerformed( ActionEvent e){
              String annoFileName = JOptionPane.showInputDialog("Enter the anno file name to load:");
              FileInputStream annoStream = null;
              try {
                annoStream = new FileInputStream(annoFileName );
                prepareAnnotationServices(annoStream);

              }catch(Exception ex )
              {
                annoStream = null;
              }

        }
      });

    for( int i=1; i< CMBAnnotationConstants.ANN_NAMES.length; i++ )
    {
      JButton  b=new JButton(CMBAnnotationConstants.ANN_NAMES[i]);
      annoToolBar.add(b);
      final int j = i;
      b.addActionListener( new ActionListener(){
        public void actionPerformed( ActionEvent e){
           annoServices.prepareToAddAnnotation( j, 1 );
        }
      });
    }
   annoToolBar.addSeparator();
   //Add custom annotation - image annotation
   JButton customAnno = new JButton("Image");
    annoToolBar.add( customAnno );
    customAnno.addActionListener( new ActionListener(){
        public void actionPerformed( ActionEvent e){
           annoServices.prepareToAddAnnotation(TImageAnnotation.ANN_IMAGE, "TImageAnnotation",1);
        }
    });

    JButton e = new JButton("Erase");
    annoToolBar.add( e );
    e.addActionListener( new ActionListener(){
        public void actionPerformed( ActionEvent e){
           annoServices.setEraseMode();
        }
      });
    annoToolBar.addSeparator();
    JButton save = new JButton("Save annotations");
    annoToolBar.add( save );
    save.addActionListener(new ActionListener(){
        public void actionPerformed( ActionEvent e){
          try {
           annoServices.saveAnnotationSet(annotationSet);
          }catch(Exception aee)
          {
            aee.printStackTrace();
          }
        }
      });
  }


  public void prepareAnnotationServices(InputStream annoStream)
  {
    try {
      annotationSet = annoServices.loadAnnotationSet(annoStream, "application/vnd.ibm.modcap", 1390,1,0);
      annoView = annoServices.prepareAnnotationView( annoComponent, annotationSet );
      annoComponent.setAnnotationView( annoView );
      annoView.refreshEntireDrawArea();

    }catch(Exception e)
    {
      e.printStackTrace();
    }
  }



  public static void main(String[] args) {
    TAnnotationEditor t = new TAnnotationEditor();
    t.initialize();

  }


  /**
   *
   */
   private class AnnotationComponent extends CMBAnnotationComponent
   {
      public AnnotationComponent()
      {
          super();
      }

     public void paint(Graphics g) {
       if (backImage != null){
           g.drawImage(backImage,0,0,this);
           super.paint(g);
       }
      }
   }

    /**
     * AnnotationServicesCallbacks class handles callbacks from CMBAnnotationServices.
     */
    private class AnnotationServicesCallbacks extends CMBAnnotationServicesCallbacks implements Serializable
    {
        AnnotationServicesCallbacks()
        {
        }

        /** Retrieves a document annotation. */
        public InputStream getAnnotationPart(CMBAnnotationSet annoSet, int annotationNumber)
        {
            System.out.println("Annotation Engine requesting additional annotation parts");
            return null;
        }
      /** Retrieves the privilege for an annotation. */
      public boolean getPrivilege(CMBAnnotationSet annoSet, int privilegeID )
      {
        return true;
      }

     /** Saves the annotation part. */
     public int addAnnotationPart(CMBAnnotationSet annotationSet, byte[] annotationData)
     {
       try {
            String annoSave = JOptionPane.showInputDialog("Enter the anno file name to save:");
            FileOutputStream fout = new FileOutputStream( annoSave );
            fout.write( annotationData );
            fout.flush();
            fout.close();
        // return the ID for the new annotation
          return 0;
       } catch (Exception e) {
         e.printStackTrace();
         return -1;
       }
     }

    /** Updates a page annotation. */
     public void updateAnnotationPart(CMBAnnotationSet annotationSet, byte[] annotationData, int annotationNumber)
     {

            String annoSave = JOptionPane.showInputDialog("Enter the anno file name to save:");
        try {
            FileOutputStream fout = new FileOutputStream( annoSave );
            fout.write( annotationData );
            fout.flush();
            fout.close();
        }catch(Exception e){
        }

     }

    /** Removes an annotation part. */
     public void removeAnnotationPart(CMBAnnotationSet annotationSet, int annotationNumber )
     {
        System.out.println("Annotation Engine attempting to delete an annotation part");

     }
     /** Returns true if tracing is enabled */
     public boolean traceEnabled() {
         return traceEnabled;
     }

     /** Writes a trace message. */
     public void trace(String message) {
         if (traceEnabled) {
             System.out.println("TAnnotationEditor:" + message );
         }
     }

    }// end of AnnotationServicesCallbacks class

}
